/*
==========================================================================
    ElectroDacus Solar BMS "SBMS100 & SBMS60" main file.
    v1.0c
    Copyright (C) 2015  <Dacian Todea>
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
===========================================================================
*/

#include "stm32f37x.h"
#include "stm32f37x_gpio.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_pwr.h"
#include "stm32f37x_rtc.h"
#include "stm32f37x_exti.h"
#include "stm32f37x_misc.h"
#include "stm32f37x_syscfg.h"
#include "stm32f37x_tim.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_usart.h"
#include <stdio.h>

#include "SBMS_ili9341.h"
#include "tm_stm32f4_fonts.h"

#include "SBMS_S25FL127S.h"

#include "delay.h"
#include "defines.h"
#include "sdadc.h"

#include "tsl_user.h"
#include "tsl.h"

#include "stm320518_eval_i2c_ee.h"
#include "tsl_touchkey.h"

#include "webpage.h"


// Select to use or not the Low-Power mode and RTC
#define USE_LPM 1 // 0=No, 1=Yes
//--------------------------------------------------------------------------------------------------------
#define SBMS100 1                                  //0=SBMS60 1=SBMS100
//--------------------------------------------------------------------------------------------------------

#if SBMS100==0
#include "bmsmenu60.h"
#endif

#if SBMS100>0
#include "bmsmenu100.h"
#endif


// Private defines

// LEDs definition
#define LCDLED_ON    {GPIOC->BSRR = (uint32_t)((uint32_t)1 <<  PIN_13);}
#define LCDLED_OFF   {GPIOC->BSRR = (uint32_t)((uint32_t)1 << (PIN_13 + 16));}
#define WIFI_ON      {GPIOF->BSRR = (uint32_t)((uint32_t)1 <<  PIN_06);}
#define WIFI_OFF     {GPIOF->BSRR = (uint32_t)((uint32_t)1 << (PIN_06 + 16));}

// Ext GPIO's
#define EXT3_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_11);}
#define EXT3_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_11 + 16));}
#define EXT4_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_12);}
#define EXT4_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_12 + 16));}
#define EXT5_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_14);}
#define EXT5_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_14 + 16));}
#define EXT6_ON      {GPIOA->BSRR = (uint32_t)((uint32_t)1 <<  PIN_13);}
#define EXT6_OFF     {GPIOA->BSRR = (uint32_t)((uint32_t)1 << (PIN_13 + 16));}

// I2C
#define RAMStart       0x80
#define ROMUStart      0x50
#define ROMSize        88
#define RAMSize        44
#define ROMUSize       8

uint8_t ROMt[4];
uint8_t ROMrx[ROMSize];
uint8_t ROMtemp[ROMSize];
uint8_t RAMrx[RAMSize];
uint8_t ROMUrx[ROMUSize];
uint16_t ROMdec[50];
uint16_t ROMdec2[3];
char* ROMunit[50];
uint8_t tempL;
uint8_t tempH;
uint8_t d[50][2];
uint16_t td[5],bk[4],Serial[3],EXT[8],uSD[2];
uint32_t WiFis[7],EXTADC1[8];


volatile uint16_t NumData = 0;
uint32_t  sEETimeout;

// Private macros
#define TEST_TKEY(NB) (MyTKeys[(NB)].p_Data->StateId == TSL_STATEID_DETECT)


// Private variables
//GPIO_InitTypeDef        GPIO_InitStructure;
TSL_tTick_ms_T ECS_last_tick; // Hold the last time value for ECS
TSL_tTick_sec_T debug_tick;
uint32_t process_sensor;

// Private functions prototype

void clock_init(void);
void Init_Std_GPIOs(void);
void TSL_Action1(void);
void Time1 (uint8_t r);
void Time11 (void);
void Logo1 (void);
void ProcessSensors(void);
void LoadONOFF(void);

void ROMv1bd(unsigned int pp,unsigned int loop22);
void ROMunitbd(unsigned int pp,unsigned int loop22);
void ROMv1db(unsigned int pp,unsigned int loop22);
void ROMunitdb(unsigned int pp,unsigned int loop22,unsigned int timeu);

void ROMconvertdb(void);
void ROMconvertbd(void);
void ROMreadall(void);
void RAMreadall(void);
void ROMread(uint8_t ROMStart);
void ROMwrite(uint8_t ROMaddress,uint8_t Tx1);
void RAMwrite(uint8_t RAMaddress,uint8_t Tx1);
void valueset(void);
void scrollhelp(void);
void mainmenu1(void);
void monitoring(void);
void monitor(void);
void parameterset(void);
void parametersetsimple(void);
void deviceset(void);
void extioadc(void);
void diagnostics(void);
void manual(void);
void Install1(void);
void Manual1(void);
void about(void);
void timeanddate(void);
void valueset2(void);
void backlight(void);
void valueset3(void);
void bk1(void);
void dataaquisition(void);
void Datalogg(void);
void bktimer(void);
void USART_Config(void);
void timestamp1(void);
void datestamp1(void);
void USARTMenu(void);
void valueset4(void);
void onesecondcounter(void);
void uSDMenu(void);
void EXT1ANDEXT2(void);
void EXTADC(void);
void UsartDataHeader(void);
void counterR(void);
void graphzoom(void);
void graphzoomh(void);
void largegraph(void);
void loadonoffm(void);

void unitCP(void);
void externalIO(void);
void valueset5(void);
void externalTEMP(void);
void lk1(void);
void unlock();
void lockKtimer();
void USART3_read(uint16_t delayx);
void WiFiMenu(void);
void valueset6(void);
void valueset7(void);
void valueset8(void);
void WiFi_Init(void);
void WiFi_UART(void);
void WiFi_CWSAP(void);
void WiFi_close(void);
void WiFi_set(void);
void WiFi_send(void);
void WiFi_send_flash(void);
void WiFi_On(void);
void WiFi_On_new(void);
void WiFi_Off(void);
void WiFi_logo(void);
void compdata(void);
void compdataflash (void);
void compenergyflash (void);
void compdevsetflash (void);
void compdataUSART (uint8_t interval);
void comp(uint32_t decval,uint8_t nr,uint8_t* dataout,uint16_t pos);
uint32_t dcomp(uint8_t nr,uint8_t* datain,uint16_t pos);
void webgraph(uint32_t *val1,uint32_t *val2,uint16_t pos,uint16_t nr,uint16_t nr2,uint16_t nr3,uint16_t smax,uint8_t power);
uint16_t pow(int base, int exp);
void decbin(uint8_t n);
void Download_flash(void);
void WiFi_DataHeader(void);

//adc
void read_all_SDADC(uint16_t nrep);
void read_all_SDADC_noautorange(uint16_t nrep);
//void get_SDADC036(void);
//void get_SDADC146(void);
//void get_SDADC256(void);


#if USE_LPM > 0
void RTC_Config(void);
void SYSCLKConfig_STOP(void);
void RTC_AlarmConfig(void);
void LowPower(void);
void Init_Reset_GPIOs(void);
void Init_LPM_GPIOs(void);
#endif

GPIO_InitTypeDef GPIO_InitStructure;
RTC_InitTypeDef RTC_InitStructure;
RTC_TimeTypeDef RTC_TimeStructure;
RTC_DateTypeDef RTC_DateStructure;
RTC_TimeTypeDef RTC_TimeStampStructure;
RTC_DateTypeDef RTC_TimeStampDateStructure;

uint32_t b1,b2,b3,b4,b5,b6,c2,c22,c222,c3,c33,c333,c3333,c6,c7,c8,c9,c9cal,c9calx50,c9calx500,ETEMPLimit,c10,sj,j,js,BOK,BMENU,BLoad,BBack,z,nr1,i3,i4,i5,scrolllenght,key1,first1,update1,step1,i2c,new1,USBpower,menucolor,aloc2,wifib,wifit;
uint32_t bc1,bc2,sc1,sc2,gain,SolarCurrent,BatteryCurrent,BatteryVoltage,BatteryVoltage2,buttondir,buttondirm,swtimer1,swtimer2,swtimer3,swtimer4,timer3600;
uint32_t BatteryW,PV1W,PV2W,PV12W,LoadW,ExtLoadW,LoadExtLoadW,SOC1,BatteryCurrent2,BatteryCurrent3,PV1,PV2,ADC2,ADC3,ADC4,PV12,PVt,LoadCurrent,ExtLoadCurrent,count1m,count6m;
uint64_t BatterySOC,BattCapacity,BatteryWh,BatterykWh2,BatteryuWh,BatteryuWh2,PV1uAh,PV1mAh,PV1Wh,PV1kWh2,PV1uWh,PV1uWh2,PV2uAh,PV2mAh,PV2Wh,PV2kWh2,PV2uWh,PV2uWh2,LoaduAh,LoadmAh,LoadWh,LoaduWh,ExtLoaduAh,ExtLoadmAh,ExtLoadWh,ExtLoaduWh,BatteryuAh,BatterymAh,energytempA,energytempW;
uint32_t zz1,zz2,zz3,zz4,zz5,zz6,zz7,release1,release2,refresh1,DFET,CFET,lockkey,lockkey1,lockkey2,UVlock,OVlock,Cdirection,rangeswitch;
uint32_t BV[8][6],BVmin[2],BVmax[2],BVdelta,CB[8],EF[15];
uint8_t power1,graphhorizontal,startdownloadflash,savedevset,factoryreset,freset,hourcounter1,ERROR5;
uint16_t ETEMP,ITEMP,ETEMP2,ITEMP2,ETEMP3,ITEMP3,ERR,datanr,xnr1,xnr2,secondcounter1,daycounter1;

#define PV1color      0xFBE0
#define PV2color      0x8928
#define Battpcolor    0x17E0
#define Battncolor    0xF800
#define Loadcolor     0x0034
#define ExtLoadcolor  0xE819


//ADC

uint32_t ADC16[7][10];// 0..6 channels 0=Value, 1=Gain, 2=Gain_calibration, 3=Real-value, 4=gain flag, 5=offset for 8x, 6=offset for 4x, 7=offset for 2x, 8=offset for 1x, 9=offset for 1/2x
uint32_t ADC16temp[3];
uint32_t Time = 0;
uint32_t Time2[15];

//define webpage variable location
const uint16_t varsbms=1522,varxsbms=1595,vargsbms=1620,vareA=1658,vareW=1711,vars1=1763,vars2=1793;

uint8_t webpage[2048];
uint8_t flash[61];
uint8_t sbmsD[61];
uint32_t graphvaltemp;

//graph log

uint32_t PV1graph[241];
uint32_t PV1graph1h[241];
uint32_t PV1graph24h[241];

uint32_t PV2graph[241];
uint32_t PV2graph1h[241];
uint32_t PV2graph24h[241];

uint32_t Battgraph[241];
uint32_t Battgraph1h[241];
uint32_t Battgraph24h[241];

uint16_t VBattgraph[241];
uint16_t VBattgraph1h[241];
uint16_t VBattgraph24h[241];

uint32_t Loadgraph[241];
uint32_t Loadgraph1h[241];
uint32_t Loadgraph24h[241];

uint32_t ExtLoadgraph[241];
uint32_t ExtLoadgraph1h[241];
uint32_t ExtLoadgraph24h[241];

uint32_t Tempgraph1[241];
uint32_t Tempgraph2[241];


const uint16_t Therm[15]={3718,3652,3544,3380,3144,2836,2472,2078,1690,1338,1040,800,612,470,360};
const float Therm2[15]={1.5151515,0.9259259,0.6097561,0.4237288,0.3246753,0.2747253,0.2538071,0.2577320,0.2840909,0.3355705,0.4166667,0.5319149,0.7042254,0.9090909};


const uint32_t sdadc_gain[5] = {SDADC_Gain_1_2,SDADC_Gain_1,SDADC_Gain_2,SDADC_Gain_4,SDADC_Gain_8};
const float gain_real[5]= {0.5,1,2,4,8};
uint32_t temp1;

char timestamp[8],datestamp[8];
long i8;
char *helptext;
char chr[192];
char* direction;
char* direction2;
uint8_t direction22;
uint8_t logo11[8];


char RXBUFF[512];
uint16_t sizeRX,sizeRX2;
uint32_t flashP[4];

uint8_t yearb,monthb,dayb,hourb,minb,secb;

extern const TSL_TouchKey_T MyTKeys[];
extern TSL_ObjectGroup_T MyObjGroup;

//constants

static float const v1=0.117216117;

#if SBMS100>0
static unsigned int const gain1=60;  //LMP8481
static unsigned int const gain2=20;  //ExtGain
static float const PVshunt=0.0003333;
static float const Battshunt=0.0002;
#endif

#if SBMS100==0  //SBMS60
static unsigned int const gain1=60;  //LMP8481
static unsigned int const gain2=20;  //ExtGain
static float const PVshunt=0.0003333;
static float const Battshunt=0.0003333;
#endif

//=================================================================================================
//                                        Main routine.
//=================================================================================================

int main(void)
{

	clock_init();

	RCC_ClocksTypeDef rcc_clocks;
	RCC_GetClocksFreq(&rcc_clocks);

	SysTick_Config(rcc_clocks.SYSCLK_Frequency / 1000);

    delay_ms(200);

    Init_Std_GPIOs();
    TSL_user_Init();
    WIFI_OFF;

    //LCDLED control
  	LCDLED_ON;
    USBpower=1;
    bk[0]=25; //set default backlight level
    bk1();

    menucolor=2; //default power-up menu color

    TM_ILI9341_Init();//LCDmenu_init();
    TM_ILI9341_Rotate(TM_ILI9341_Orientation_Landscape_1);

    Fill2(0x0000);

    LCDmenu_set_XY(0,0);
    #if SBMS100>0
    LCDmenu_write_string_neg("           SBMS100           ",menucolor);
    #endif
    #if SBMS100==0
    LCDmenu_write_string_neg("           SBMS60            ",menucolor);
    #endif
    delay_ms(400);

    // Low-Power mode configuration

    #if USE_LPM > 0
    RTC_Config();
    #else
    // while(TSL_tim_CheckDelay_sec(1, &debug_tick) == TSL_STATUS_BUSY);
    #endif

    i2c=0;
    LCDmenu_set_XY(0,1);
    LCDmenu_write_string_neg("       Before RAM read       ",menucolor);
    RAMreadall();
    if (USBpower==1){
    LCDmenu_set_XY(0,2);
    LCDmenu_write_string_neg("      Before RAM write       ",menucolor);
    RAMwrite(0x85,32); //set current amplifier gain x50 (00) x5 (16) x500 (32) x500 (48)
    LCDmenu_set_XY(0,3);
    LCDmenu_write_string_neg("        Before EEPROM        ",menucolor);
    ROMread(0x00);
    ROMreadall(); //read the entire EEPROM
    LCDmenu_set_XY(0,4);
    LCDmenu_write_string_neg("           ROM-OK            ",menucolor);
    USBpower=0;
    }

//=================================================================================================

  aloc2=0;
  wifib=0;
  b1=0;
  b2=0;
  b3=0;
  b4=0;
  c2=100;//main menu
  startdownloadflash=0;
  datanr=0;

  key1=0;
  c33=0;
  c3=1;
  update1=1;
  step1=1;
  buttondir=0;
  swtimer1=3601;
  swtimer3=0;
  swtimer4=3601;
  ROMdec2[0]=5;
  ROMdec2[1]=3;
  ROMdec2[2]=2;
  BattCapacity=(100*1000000);
  BatterySOC=BattCapacity*1.5;  //set the initial SOC at 50% after a full charge the SOC will be correct.
  refresh1=1;
  lockkey=0;
  lockkey1=0;
  lockkey2=0;
  zz4=6;
  c7=0;
  ExtLoadCurrent=0;
  count1m=0;
  count6m=0;
  power1=0;
  graphhorizontal=0;

  bk[0]=25; //set backlight to about 70%
  bk[2]=5; //default backlight off set at 5 minutes

  bk[3]=15;  //lockkey set at 15 min
  bk[1]=menucolor;
  lk1();

  EXT[0]=0;
  EXT[1]=90;
  EXT[2]=0;
  EXT[3]=30;
  EXT[4]=0;
  EXT[5]=50;
  EXT[6]=0;
  EXT[7]=50;

  EXTADC1[0]=0;
  EXTADC1[1]=1500;
  EXTADC1[2]=0;
  EXTADC1[3]=PVshunt*10000000;
  EXTADC1[4]=0;
  EXTADC1[5]=PVshunt*10000000;
  EXTADC1[6]=0;
  EXTADC1[7]=Battshunt*10000000;

  Serial[0]=0;    //disable USART datalog at power ON
  Serial[1]=1152; //USART baud rate set default at 9600/115.2k
  Serial[2]=30;   //data log interval set default at 30s.
  swtimer2=0;

 // WiFis[0]=0;
  WiFis[1]=1;

  WiFis[2]=60;
  #if SBMS100>0
  WiFis[2]=100;
  #endif

  WiFis[3]=0;
  WiFis[4]=0;
  WiFis[5]=0;

  uSD[0]=1;

  flashP[0]=0;
  flashP[1]=0;
  flashP[2]=0;
  flashP[3]=0;

  secondcounter1=0;
  hourcounter1=0;
  daycounter1=0;// read spi flash for this if is FF is new so set to zero.

  freset=0;

  yearb=0;

//=================================================================================================

  for (sj=0;sj<=240;sj++)
    {
  	PV1graph[sj]=0;
  	PV1graph1h[sj]=0;
  	PV1graph24h[sj]=0;

  	PV2graph[sj]=0;
  	PV2graph1h[sj]=0;
  	PV2graph24h[sj]=0;

  	Battgraph[sj]=12000000;
  	Battgraph1h[sj]=12000000;
  	Battgraph24h[sj]=12000000;

  	VBattgraph[sj]=0;
  	VBattgraph1h[sj]=0;
  	VBattgraph24h[sj]=0;

  	Loadgraph[sj]=0;
  	Loadgraph1h[sj]=0;
  	Loadgraph24h[sj]=0;

  	ExtLoadgraph[sj]=0;
  	ExtLoadgraph1h[sj]=0;
  	ExtLoadgraph24h[sj]=0;

    }
//=================================================================================================

  LCDmenu_set_XY(0,5);
  LCDmenu_write_string_neg("          WiFi Init          ",menucolor);

  WIFI_ON;
  USART_Config();
  WiFi_On_new();// configuring a brand new WIFI module to work instead of 115.2kbps to 921.6kbps
  delay_ms(1000);
  WIFI_OFF;
  delay_ms(200);
  WIFI_ON;
  delay_ms(1000);
  WiFi_On();

  S25FL127S_Init(); //Init the 128Mbit SPI flash.

  LCDmenu_set_XY(0,6);
  LCDmenu_write_string_neg("Calibration ISL94203  |",menucolor);

  for (sj=0;sj<10;sj++)     //PV & Batt-x500 current calibration (get the ADC offset)
  {
	  RAMreadall();
	  c9=(RAMrx[15]*256)+RAMrx[14];  //battery current
	  c7=c9+c7;
   	  delay_ms(40);//50ms original

  }

  c9calx500=c7/10;

  c7=0;
  RAMwrite(0x85,0); //set current amplifier gain x50 (00) x5 (16) x500 (32) x500 (48)
  LCDmenu_set_XY(0,6);
  LCDmenu_write_string_neg("Calibration ISL94203  |  |",menucolor);

  for (sj=0;sj<10;sj++)     //Batt-x50 current calibration (get the ADC offset)
   {
 	  RAMreadall();
 	  c9=(RAMrx[15]*256)+RAMrx[14];  //battery current
 	  c7=c9+c7;
      delay_ms(40);
   }

  c9calx50=c7/10;

  LCDmenu_set_XY(0,6);
  LCDmenu_write_string_neg("Calibration ISL94203  |  |  |",menucolor);

  ROMconvertbd();

  //RAMreadall();
  //tempL=(((RAMrx[8])&0xFE)+0);
  //RAMwrite(0x88,tempL); //set the isl idle forever

  init_all_SDADC();

       //   get_SDADC036();
       //	temp1=ADC16[6][0];
       //	get_SDADC146();
       //	temp1=(ADC16[6][0]+temp1);
       //	get_SDADC256();
       //	ADC16[6][0]=(ADC16[6][0]+temp1)/3;
       read_all_SDADC_noautorange(6);

       	for (js=0 ;js<5;++js){
       	for (j=0 ;j<7;++j){
       	ADC16[j][5+js]=0;  //set all ADC offset to 0
       	     	}
       	}

   LCDmenu_set_XY(0,7);
   LCDmenu_write_string_neg("Calibration SDADC ",menucolor);
   delay_ms(1000);
   LCDmenu_write_string_neg(" ",menucolor);
   delay_ms(1000);


  for (j=0 ;j<7;++j)  {ADC16[j][1]=0;}  // set all channels to max gain of 1/2 (0)

  for (sj=0;sj<10;sj++)     //PV (get the SDADC offset)
   {
	read_all_SDADC_noautorange(5);
	Puts(210+(sj*11), 140, ">", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][0]);
	for (j=0;j<7;j++){ADC16[j][5]=ADC16[j][0]+ADC16[j][5];}
   }

  for (j=0;j<7;j++){ADC16[j][5]=ADC16[j][5]/10;}


     ReadFlash(262144,flash,18);
     if(flash[0]!=255)
     {
    	daycounter1= dcomp(3,flash,0);
    	PV1uWh2= dcomp(5,flash,3)*100000;
    	PV2uWh2= dcomp(5,flash,8)*100000;
    	BatteryuWh2= dcomp(5,flash,13)*100000;
     }

     delay_ms(30);

     ReadFlash(262144,flash,18); //just to be sure read again :)
          if(flash[0]!=255)
          {
         	daycounter1= dcomp(3,flash,0);
         	PV1uWh2= dcomp(5,flash,3)*100000;
         	PV2uWh2= dcomp(5,flash,8)*100000;
         	BatteryuWh2= dcomp(5,flash,13)*100000;
          }

          WiFi_On();

          ReadFlash(327680,flash,56);
          if(flash[0]!=255)
          {
        	  bk[0]=dcomp(1,flash,0);
        	  bk1();
        	  bk[2]=dcomp(1,flash,1);
        	  bk[3]=dcomp(1,flash,2);
        	  menucolor=dcomp(1,flash,3);
        	  bk[1]=menucolor;

        	  Serial[0]=dcomp(1,flash,4);

        	  Serial[1]=dcomp(3,flash,5);
        	  Serial[2]=dcomp(2,flash,8);

        	  WiFis[0]=dcomp(1,flash,10);
        	  if (WiFis[0]==0){WiFi_Off();}
        	  if (Serial[0]==1){USART_Config();}
        	  WiFis[1]=dcomp(1,flash,11);

        	  WiFis[2]=dcomp(2,flash,12);

        	  WiFis[3]=dcomp(1,flash,14);
        	  WiFis[4]=dcomp(3,flash,15);
        	  WiFis[5]=dcomp(3,flash,18);

        	  EXT[0]=dcomp(1,flash,21);
        	  EXT[1]=dcomp(3,flash,22);
        	  EXT[2]=dcomp(1,flash,25);
        	  EXT[3]=dcomp(3,flash,26);
        	  EXT[4]=dcomp(1,flash,29);
        	  EXT[5]=dcomp(3,flash,30);
        	  EXT[6]=dcomp(1,flash,33);
        	  EXT[7]=dcomp(3,flash,34);

        	  EXTADC1[0]=dcomp(1,flash,37);
        	  EXTADC1[1]=dcomp(3,flash,38);
        	  EXTADC1[2]=dcomp(2,flash,41);
        	  EXTADC1[3]=dcomp(3,flash,43);
        	  EXTADC1[4]=dcomp(2,flash,46);
        	  EXTADC1[5]=dcomp(3,flash,48);
        	  EXTADC1[6]=dcomp(2,flash,51);
        	  EXTADC1[7]=dcomp(3,flash,53);
          }


 BattCapacity=(ROMdec[2]*1000000);
 BatterySOC=BattCapacity*1.5;  //set the initial SOC at 50% after a full charge the SOC will be correct.

//=================================================================================================
//                                       Main loop
//=================================================================================================

  for (;;)
  {
   bktimer();
   lockKtimer();
   dataaquisition();
   onesecondcounter();


if (key1==0)
{
    TSL_Action1();
	ProcessSensors();
}


if (key1!=0)
{
 bk1();
 lk1();
}


if ((zz3==1)&&(lockkey==1)&&(zz4<6))
	{
	zz4++;
	zz3=0;
	i5=0;
	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1(1,0,zz4,menucolor);
	}

if (zz4==5){Fill_menu(0x0000); update1=1; zz4=6; first1=1; zz6=1;}//LCDmenu_clear()

if ((key1!=0)&&(lockkey==1))
{
update1=0;
if (zz4>=6)
{
zz4=0;
lockkey1=0;
lockkey2=0;
}
unlock();
key1=0;
c3=c3333;
c2=c222;
BOK=0;
BBack=0;
BMENU=0;
}


if (zz1==1 && (c2==7||c2==31)&&((zz4==6)||lockkey!=1))
{
	zz1=0;
	if(c2==7){Time11();Logo1();}
	if(c2==31){Time1(0);}
}

if (c2==8||c2==9||c2==10||c2==19||c2==20||c2==21||c2==24||c2==25||c2==26||c2==27||c2==28||c2==29||c2==40||(c2>=45&&c2!=61&&c2!=62)){c2=100;}

if (update1==1)
{
   RAMreadall();
}

    if((key1==1)||(update1==1))
{
    update1=0;
    key1=0;
	mainmenu1();          //c2>=100;
    valueset();           //c2=22;
    monitoring();         //c2=1;
    parameterset();       //c2=23;
    parametersetsimple(); //c2=2
    deviceset();          //c2=3;
    extioadc();           //c2=4;
    diagnostics();  	  //c2=5;
    manual();             //c2=6;
    Install1();           //c2=61;
    Manual1();            //c2=62;
    about();              //c2=7;
    valueset2();          //c2=32;
    timeanddate();        //c2=31;
    valueset3();          //c2=34;
    backlight();          //c2=33;
    valueset4();          //c2=36;
    USARTMenu();          //c2=35;
    valueset6();          //c2=38;
    WiFiMenu();           //c2=37;
    valueset7();          //c2=30;
    uSDMenu();            //c2=39;
    EXT1ANDEXT2();        //c2=41;
    valueset5();          //c2=42;
    EXTADC();             //c2=43;
    valueset8();          //c2=44;
    counterR();           //c2=11;
    graphzoom();          //c2=12;
    largegraph();         //c2=13,14,15;
    graphzoomh();         //c2=16;
    unitCP();             //c2=17;
    loadonoffm();         //c2=18;
}

    scrollhelp();

//=================================================================================================
  if ((zz6==1)&&(c2!=2)&&(c2!=22)&&(c2!=4)){

    DrawHline(0,320,180,bmscolor[menucolor][25]);
    if (SOC1<10){
    	Puts(0, 187, "  ", &TM_Font_16x26, 0xFFFF, 0x0000);
    	WriteDec(32, 187, &TM_Font_16x26, 0xFFFF, 0x0000, 1,0,SOC1,0);
    }
    else if (SOC1==100)
    {
    	WriteDec(0, 187, &TM_Font_16x26, 0xFFFF, 0x0000, 3,0,SOC1,0);
    }
    else
    {
    	Puts(0, 187, " ", &TM_Font_16x26, 0xFFFF, 0x0000);
    	WriteDec(16, 187, &TM_Font_16x26, 0xFFFF, 0x0000, 2,0,SOC1,0);
    }
    Puts(51, 187, "%", &TM_Font_11x18, 0xFFFF, 0x0000);


if (EF[12]==1)
{
Puts(0, 230, "PV1+PV2: ", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][50]);
if      (BatteryCurrent3>100000)  {WriteDec(63, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 5,2,PV12/10,1);}
else if (BatteryCurrent3>10000)   {WriteDec(63, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 5,3,PV12,1);}
else  	                          {WriteDec(63, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 4,3,PV12,1);}
Puts(105, 230, "A", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][50]);
DrawHline(0,112,229,bmscolor[menucolor][50]);
}
else
{
Puts(0, 230, "PV1+PV2: ", &TM_Font_7x10, bmscolor[menucolor][20], bmscolor[menucolor][5]);
if      (BatteryCurrent3>100000)  {WriteDec(63, 230, &TM_Font_7x10, bmscolor[menucolor][20],  bmscolor[menucolor][5], 5,2,PV12/10,1);}
else if (BatteryCurrent3>10000)   {WriteDec(63, 230, &TM_Font_7x10, bmscolor[menucolor][20],  bmscolor[menucolor][5], 5,3,PV12,1);}
else  	                          {WriteDec(63, 230, &TM_Font_7x10, bmscolor[menucolor][20],  bmscolor[menucolor][5], 4,3,PV12,1);}
Puts(105, 230, "A", &TM_Font_7x10, bmscolor[menucolor][20], bmscolor[menucolor][5]);
DrawHline(0,112,229,bmscolor[menucolor][5]);
}

if (direction22!=0){Puts(114, 230, "Batt:   +", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][50]);}
else               {Puts(114, 230, "Batt:   -", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][50]);}
if      (BatteryCurrent3>100000)  {WriteDec(177, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 5,2,BatteryCurrent3/10,1);}
else if (BatteryCurrent3>10000)   {WriteDec(177, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 5,3,BatteryCurrent3,1);}
else  	                          {WriteDec(177, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 4,3,BatteryCurrent3,1);}
Puts(219, 230, "A", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][50]);
DrawHline(114,226,229,bmscolor[menucolor][50]);

if (EF[14]==1)
{
Puts(228, 230, "Load: ", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][50]);
if      (BatteryCurrent3>100000)  {WriteDec(270, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 5,2,(LoadCurrent+ExtLoadCurrent)/10,1);}
else if (BatteryCurrent3>10000)   {WriteDec(270, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 5,3,LoadCurrent+ExtLoadCurrent,1);}
else  	                          {WriteDec(270, 230, &TM_Font_7x10, bmscolor[menucolor][0],  bmscolor[menucolor][50], 4,3,LoadCurrent+ExtLoadCurrent,1);}
Puts(312, 230, "A", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][50]);
DrawHline(228,319,229,bmscolor[menucolor][50]);
}
else
{
Puts(228, 230, "Load: ", &TM_Font_7x10, bmscolor[menucolor][20], bmscolor[menucolor][5]);
if      (BatteryCurrent3>100000)  {WriteDec(270, 230, &TM_Font_7x10, bmscolor[menucolor][20],  bmscolor[menucolor][5], 5,2,(LoadCurrent+ExtLoadCurrent)/10,1);}
else if (BatteryCurrent3>10000)   {WriteDec(270, 230, &TM_Font_7x10, bmscolor[menucolor][20],  bmscolor[menucolor][5], 5,3,LoadCurrent+ExtLoadCurrent,1);}
else  	                          {WriteDec(270, 230, &TM_Font_7x10, bmscolor[menucolor][20],  bmscolor[menucolor][5], 4,3,LoadCurrent+ExtLoadCurrent,1);}
Puts(312, 230, "A", &TM_Font_7x10, bmscolor[menucolor][20], bmscolor[menucolor][5]);
DrawHline(228,319,229,bmscolor[menucolor][5]);

}

   if (USBpower!=2)
   {
      for (sj=0;sj<18;sj++)
      {
      if (SOC1>40)                      {j=sj*2.2; DrawHline(76,77+(SOC1*2),188+sj,bmscolor[4][14+j]); DrawHline(76,77+(SOC1*2),206+sj,bmscolor[4][62-j]);}
      else if ((SOC1<=40)&&(SOC1>=20))  {j=sj*2.2; DrawHline(76,77+(SOC1*2),188+sj,bmscolor[1][14+j]); DrawHline(76,77+(SOC1*2),206+sj,bmscolor[1][62-j]);}
      else                              {j=sj*2.2; DrawHline(76,77+(SOC1*2),188+sj,bmscolor[6][14+j]); DrawHline(76,77+(SOC1*2),206+sj,bmscolor[6][62-j]);}

      if (SOC1!=100){ DrawHline(77+(SOC1*2),277,188+sj,bmscolor[0][2+(j/3)]); DrawHline(77+(SOC1*2),277,206+sj,bmscolor[0][16-(j/4)]);}
      }
    }
    else {Puts(105, 192, "USB Power", &TM_Font_16x26, bmscolor[menucolor][50], 0x0000);}

    for (sj=0;sj<19;sj++)
    {
    j=sj*2.6;
    DrawHline(69,76,187+sj,bmscolor[0][(12+j)]);
    DrawHline(69,76,206+sj,bmscolor[0][(62-j)]);
    DrawHline(277,284,187+sj,bmscolor[0][12+j]);
    DrawHline(277,284,206+sj,bmscolor[0][62-j]);
    }
    for (sj=0;sj<13;sj++)
        {
    	j=sj*3;
         DrawHline(285,292,193+sj,bmscolor[0][19+j]);
         DrawHline(285,292,206+sj,bmscolor[0][58-j]);
        }
    DrawHline(76,277,187,bmscolor[0][(20)]);
    DrawHline(76,277,224,bmscolor[0][20]);

    //DrawRectangle(87, 187, 293,222,0xADC2);

    //Time2[2]=Time;
    Puts(0, 216, "Stat", &TM_Font_7x10, bmscolor[menucolor][45], bmscolor[menucolor][0]);
    WriteDec(32, 216, &TM_Font_7x10, bmscolor[menucolor][63],bmscolor[menucolor][0], 5,0,ERR,0);
    //WriteDec(80, 216, &TM_Font_7x10, 0xddbb, 0x0000, 3,0,Time2[4],0);
    //WriteDec(110, 216, &TM_Font_7x10, 0xff00, 0x0000, 3,0,Time2[5]-Time2[4],0);
    //WriteDec(140, 216, &TM_Font_7x10, 0xddbb, 0x0000, 3,0,Time2[6]-Time2[5],0);
    //WriteDec(170, 216, &TM_Font_7x10, 0xff00, 0x0000, 3,0,Time2[7]-Time2[6],0);
    //WriteDec(200, 216, &TM_Font_7x10, 0xddbb, 0x0000, 3,0,Time2[8]-Time2[7],0);
    //WriteDec(230, 216, &TM_Font_7x10, 0xff00, 0x0000, 3,0,Time2[9]-Time2[8],0);


  zz6=0;
  }

  }
}

//=================================================================================================
//      Initializes GPIOs not related to Touch-Sensing or LCD (LEDs, EXT1 and EXT2 pins, ...)
//=================================================================================================

void Init_Std_GPIOs(void)
{

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

	  /* LCD back light */
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOC, &GPIO_InitStructure);

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);

	  /* ESP03 reset */
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(GPIOF, &GPIO_InitStructure);

	 RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);

	  /* Configure PF0 and PF1 output not open drain mode for EXT1 and EXT2 lines but will be reserved for future use as I2C port */
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOF, &GPIO_InitStructure);

	 RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	  /* UART TX */
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);

	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	  /* EXTIO3,EXTIO4,EXTIO6 and EXTIO5 */
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOA, &GPIO_InitStructure);

}

//=================================================================================================
void externalIO(void)
{
if (EXT[0]==0){EXT3_OFF;}
if (EXT[0]==1){if (CFET==1)     {EXT3_ON;}      else {EXT3_OFF;}}
if (EXT[0]==2){if (DFET==1)     {EXT3_ON;}      else {EXT3_OFF;}}
if (EXT[0]==3){if (EXT[1]>=SOC1){EXT3_ON;}      else {EXT3_OFF;}}
if (EXT[0]==4){if (EXT[1]<=SOC1){EXT3_ON;}      else {EXT3_OFF;}}
if (EXT[0]==5){if (ERROR5==1)   {EXT3_ON;}      else {EXT3_OFF;}}

if (EXT[2]==0){EXT4_OFF;}
if (EXT[2]==1){if (CFET==1)     {EXT4_ON;}      else {EXT4_OFF;}}
if (EXT[2]==2){if (DFET==1)     {EXT4_ON;}      else {EXT4_OFF;}}
if (EXT[2]==3){if (EXT[3]>=SOC1){EXT4_ON;}      else {EXT4_OFF;}}
if (EXT[2]==4){if (EXT[3]<=SOC1){EXT4_ON;}      else {EXT4_OFF;}}
if (EXT[2]==5){if (ERROR5==1)   {EXT4_ON;}      else {EXT4_OFF;}}

if (EXT[4]==0){EXT5_OFF;}
if (EXT[4]==1){if (CFET==1)     {EXT5_ON;}      else {EXT5_OFF;}}
if (EXT[4]==2){if (DFET==1)     {EXT5_ON;}      else {EXT5_OFF;}}
if (EXT[4]==3){if (EXT[5]>=SOC1){EXT5_ON;}      else {EXT5_OFF;}}
if (EXT[4]==4){if (EXT[5]<=SOC1){EXT5_ON;}      else {EXT5_OFF;}}
if (EXT[4]==5){if (ERROR5==1)   {EXT5_ON;}      else {EXT5_OFF;}}

if (EXT[6]==0){EXT6_OFF;}
if (EXT[6]==1){if (CFET==1)     {EXT6_ON;}      else {EXT6_OFF;}}
if (EXT[6]==2){if (DFET==1)     {EXT6_ON;}      else {EXT6_OFF;}}
if (EXT[6]==3){if (EXT[7]>=SOC1){EXT6_ON;}      else {EXT6_OFF;}}
if (EXT[6]==4){if (EXT[7]<=SOC1){EXT6_ON;}      else {EXT6_OFF;}}
if (EXT[6]==5){if (ERROR5==1)   {EXT6_ON;}      else {EXT6_OFF;}}

ERROR5=1;
CFET=0;
DFET=0;
}
//=================================================================================================
void externalTEMP(void)
{

if ((ETEMP>ROMdec[28])||(ETEMP<ROMdec[29])||(ITEMP<=1000))//ITEMP 1040 = 60C
{
ETEMPLimit=1;
tempH=((RAMrx[6]) & 0x01);
    if(tempH!=0)
        {
        tempL=(((RAMrx[7])&0xBF)+64);
    	RAMwrite(0x87,tempL); //uC manual control
    	LCDmenu_set_XY(18,0);
    	LCDmenu_write_string("Load&PV OFF",menucolor);
    	tempL=(RAMrx[6] & 0xFC);
    	RAMwrite(0x86,tempL);//load and PV is OFF
    	// need to do also charge OFF
        delay_ms(40);
        }

}


if (ETEMPLimit==1)
{
if (((ETEMP+100)<=ROMdec[28])&&(ETEMP>=(ROMdec[29]+100))&&(ITEMP>1338))//ITEMP 1338= 50C
{
ETEMPLimit=0;
		tempL=(((RAMrx[7])&0xEF)+16);
		RAMwrite(0x87,tempL); //uC manual load monitor control
		tempL=(((RAMrx[6])&0x3F)+128);
		RAMwrite(0x86,tempL); //uC manual load monitor control

		tempL=(((RAMrx[7])&0xBF));
 		RAMwrite(0x87,tempL);        //FET auto enable
 		LCDmenu_set_XY(18,0);
 		LCDmenu_write_string("Load&PV ON ",menucolor);
 		delay_ms(40);
}
}

}

//=================================================================================================
void onesecondcounter(void)
{

	RTC_TimeTypeDef RTC_TimeStructure;
	RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
    uint8_t i;


	WiFi_logo();

    if ((aloc2==0)&&(Time>=800)){Time2[4]=Time/5;RAMreadall();aloc2++;Time2[5]=Time/5;}

    //get_SDADC036();temp1=ADC16[6][0];get_SDADC146();temp1=(ADC16[6][0]+temp1);get_SDADC256();ADC16[6][0]=(ADC16[6][0]+temp1)/3;
    for (i=0;i<7;i++){ ADC16[i][1]=0;}
	if ((aloc2==1)&&(Time>=1800)){Time2[6]=Time/5;read_all_SDADC_noautorange(16); aloc2++;Time2[7]=Time/5;}

	if ((aloc2==2)&&(Time>=4000))
	{

		Time2[8]=Time/5;
		monitor();
		if(c2!=39) {compdata();}
		aloc2++;
		if ((WiFis[0]==1)&&(c2!=39)){WiFi_set();WiFi_send();}
		Time2[9]=Time/5;
	}

	if ((c2==39)&&(startdownloadflash!=0))
	{
		if (WiFis[0]==1){

			Download_flash();

		}

	}


    if (aloc2>2){aloc2=3;}


	timer3600=RTC_TimeStructure.RTC_Seconds;

	if (timer3600<swtimer3){timer3600=timer3600+60;}
	 if (0<(timer3600 - swtimer3)) //see if the one second has elapsed.
	    {
		    swtimer3=RTC_TimeStructure.RTC_Seconds; //get the new time
	    	zz1=1;
            zz2=1;
            zz3=1;
            zz5=1;
            zz6=1;
            zz7=1;

            externalIO();
            externalTEMP();

            //WriteDec(0, 216, &TM_Font_7x10, 0xddbb, 0x0000, 4,3,Time,0);

            if (aloc2==3){if((wifib==0)&&(WiFis[0]==1)&&(c2!=39)){WiFi_close(); WiFis[6]=0;};aloc2=0;Time=0;}
            if ((c2==37) && (WiFis[0]==1)){WiFi_CWSAP();}
            if (c2==7){update1=1;}
            Time2[3]=Time/5;


            secondcounter1++;//=secondcounter1+1800;//needs to be changed
            if (secondcounter1>=3600)
            {
            	hourcounter1++;secondcounter1=0;
            }
            if (hourcounter1>=24 && c2!=39 && c2!=30)
            {if (secondcounter1==0){EraseSector(262144);	delay_ms(350);}
            	else {flash[0]=0;ReadFlash(262144,flash,2);}
            	if(flash[0]==255)
            	{
            		daycounter1++;
            		hourcounter1=0;
            		compenergyflash();
            		for (i=0;i<18;i++){WriteFlash(sbmsD[i],262144+i);}//write to flash at 24h interval
            		delay_ms(50);
            	}
            }

            if (c2==3 && savedevset!=0)
            {
            	ReadFlash(327680,flash,2);
            	if(flash[0]==255){savedevset=2;}
            	if (savedevset==1){EraseSector(327680);}
            	if (savedevset==2){ savedevset=3; compdevsetflash(); for (i=0;i<56;i++){WriteFlash(sbmsD[i],327680+i);} delay_ms(20);
            	LCDmenu_set_XY(0,6);
            	LCDmenu_write_string_neg("           Done :)           ",menucolor);
            	}
            }

            if (c2==3 && factoryreset!=0)
            {
            	ReadFlash(327680,flash,2);
            	if(flash[0]==255){factoryreset=2;}
            	if (factoryreset==1){EraseSector(327680);
            	LCDmenu_set_XY(0,7);
            	LCDmenu_write_string_neg("           busy...           ",menucolor);}
            	if (factoryreset==2){factoryreset=0;
            	LCDmenu_set_XY(0,7);
            	LCDmenu_write_string_neg("           Done :)           ",menucolor);
            	}
            	if (factoryreset==3){

            	}

            }

            if (factoryreset==3||savedevset==3){
            freset++;
            if (freset>=5)
            {
            savedevset=0;factoryreset=0;freset=0;
            new1=0;
            key1=1;
            }
            }

        Datalogg();
	    }
}

//=================================================================================================


void bktimer(void)
{

	if (swtimer1>3601){return;}

	uint8_t tempbk;
	RTC_TimeTypeDef RTC_TimeStructure;
	RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
    timer3600=((RTC_TimeStructure.RTC_Minutes*60)+RTC_TimeStructure.RTC_Seconds);

    if (timer3600<swtimer1){timer3600=timer3600+3600;}

    if ((bk[2]*60)<(timer3600 - swtimer1) && bk[2]>0 && swtimer1<3601) //for LCD backlight
    {
        tempbk = bk[0];
    	while (bk[0]>5)
    	{
        bk[0]--;
    	delay_ms(6); //reduce backlight to level 5
    	bk1();
    	}
    	bk[0]=tempbk;
    	swtimer1=3601;
    }
}

//=================================================================================================
void unlock(void)
{

	buttondir=0;
	Fill_menu(0x0000);//LCDmenu_clear();
	Puts(38, 27, "Press Keys to Unlock", &TM_Font_11x18, bmscolor[menucolor][58], bmscolor[menucolor][0]);

	DrawFilledCircle(50,100,30,bmscolor[menucolor][10]);
	DrawFilledCircle(150,100,30,bmscolor[menucolor][10]);
	DrawFilledCircle(250,100,30,bmscolor[menucolor][10]);
	Puts(34, 87, "OK", &TM_Font_16x26, bmscolor[menucolor][40], bmscolor[menucolor][10]);
	for (j=0;j<60;j++){	DrawHline(134+(j/2), 134+30-(j/2),  112-j, bmscolor[menucolor][40]);}
	for (j=0;j<60;j++){	DrawHline(234+(j/2), 234+30-(j/2),  90+j, bmscolor[menucolor][40]);}


	new1=0;

    if (BOK==1)
    {
    DrawFilledCircle(50,100,30,bmscolor[menucolor][50]);
    Puts(34, 87, "OK", &TM_Font_16x26, bmscolor[menucolor][5], bmscolor[menucolor][50]);
    lockkey1=1;
    new1=0;
    }
    if ((b4>0)&&(lockkey1==1))
    {
    DrawFilledCircle(50,100,30,bmscolor[menucolor][50]);
    Puts(34, 87, "OK", &TM_Font_16x26, bmscolor[menucolor][5], bmscolor[menucolor][50]);
    DrawFilledCircle(150,100,30,bmscolor[menucolor][50]);
    for (j=0;j<60;j++){	DrawHline(134+(j/2), 134+30-(j/2),  112-j, bmscolor[menucolor][5]);}
    lockkey2=1;
    new1=0;
    }
    if ((b3>0)&&(lockkey1==1)&&(lockkey2==1))
    {
    DrawFilledCircle(50,100,30,bmscolor[menucolor][50]);
    Puts(34, 87, "OK", &TM_Font_16x26, bmscolor[menucolor][5], bmscolor[menucolor][50]);
    DrawFilledCircle(150,100,30,bmscolor[menucolor][50]);
    for (j=0;j<60;j++){	DrawHline(134+(j/2), 134+30-(j/2),  112-j, bmscolor[menucolor][5]);}
    DrawFilledCircle(250,100,30,bmscolor[menucolor][50]);
    for (j=0;j<60;j++){	DrawHline(234+(j/2), 234+30-(j/2),  90+j, bmscolor[menucolor][5]);}

    lockkey1=0;
    lockkey2=0;
    lockkey=0;
    update1=1;
    delay_ms(200);
    Fill_menu(0x0000);//LCDmenu_clear();
    new1=0;
    first1=1;
    zz6=1;
    buttondir=buttondirm;
    }
first1=1;

}
//=================================================================================================
void lockKtimer(void)
{
	if (swtimer4>3601){return;}

	RTC_TimeTypeDef RTC_TimeStructure;
	RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
    timer3600=((RTC_TimeStructure.RTC_Minutes*60)+RTC_TimeStructure.RTC_Seconds);

    if (timer3600<swtimer4){timer3600=timer3600+3600;}//test 3s delay need more test ?????????????

    if ((bk[3]*60)<(timer3600 - swtimer4) && bk[3]>0 && swtimer4<3601) //for LCD backlight
    {
    	lockkey=1;
    	c3333=c3;
    	c222=c2;
    	i5=0;
    	buttondirm=buttondir;
    	swtimer4=3601;
    	new1=0;
    }
}

//=================================================================================================
void UsartDataHeader(void)
{
	uint8_t nextln;

#if SBMS100==0
char intro1[] = "ElectroDacus SBMS60";
#endif

#if SBMS100>0
char intro1[] = "ElectroDacus SBMS100";
#endif

	char datalogheader[] = "                    ";
	char *l1,*l2;
	l1 = intro1;
	l2 = datalogheader;
	nextln=10;  //char for new line
	while(*l1)
	{
	 while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
	 USART_SendData(USART3, *l1++);
	}

	while (USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
	USART_SendData(USART3, nextln);   //new line

	while(*l2)
		{
		 while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
		 USART_SendData(USART3, *l2++);
		}

	while (USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
	USART_SendData(USART3, nextln);   //new line

}


//=======================================data log===================================================
void Datalogg(void)
{
if ((yearb==0)||(c2==31)||(c2==32)||(c2==39)||(c2==30)){return;}

uint8_t minc,i;
minc=minb/2;
flashP[1]=((((monthb-1)*22320)+((dayb-1)*720)+(hourb*30)+minc)*60);

if ((flashP[1]!=flashP[0]))//&&(yearb!=0)&&(c2!=31)&&(c2!=32)
{
flashP[2]=(flashP[1]+524288+60)/65536;

if(flashP[3]!=flashP[2])//delete 64k
{flashP[3]=flashP[2];if (flashP[1]!=16027200) { EraseSector(flashP[1]+524288+60);delay_ms(350);} }

else{
flashP[0]=flashP[1];
ReadFlash(524288+flashP[1],flash,2);
if(flash[0]==255){  compdataflash();for (i=0;i<60;i++){WriteFlash(sbmsD[i],flashP[1]+524288+i);}  }//write 60byte if that location is empty
}

}
}

//==================================================================================================
void Download_flash(void)
{
	uint16_t colordata,i;

	if (datanr==0)
	{WiFi_DataHeader();
	LCDmenu_set_XY(0,2);//
	LCDmenu_write_string_neg("           busy...           ",menucolor);}

	ReadFlash(524277+(datanr*1800)+((uSD[0]-1)*1339200),webpage,1812);

		webpage[1814]=10;//new line
		webpage[1813]=32;//space
		webpage[1812]=59;//;
		webpage[1811]=34;//"

		webpage[10]=34;//"
		webpage[9]=61;//=
		webpage[8]=(datanr % 10) + 0x30;
		webpage[7]=(datanr % 100/10 ) + 0x30;
		webpage[6]=(datanr % 1000/100 ) + 0x30;
		webpage[5]=104;//h
		webpage[4]=32;// space
		webpage[3]=114;//r
		webpage[2]=97;//a
		webpage[1]=118;//v
	    webpage[0]=32;//space


		if (datanr==0) {
			for (i=0;i<744;i++)
			{
				colordata=bmscolor[menucolor][10];
				xnr2=i/62;
				xnr1=(i - (xnr2*62))*5;
				DrawVline(68+(9*xnr2),75+(9*xnr2),6+xnr1,colordata);
				DrawVline(68+(9*xnr2),75+(9*xnr2),7+xnr1,colordata);
				DrawVline(68+(9*xnr2),75+(9*xnr2),8+xnr1,colordata);
			}
			sizeRX2=5;
			WiFi_set(); xnr1=0;xnr2=0;

			delay_ms(50);
		}


		if (sizeRX2!=0&&sizeRX2<40){colordata=bmscolor[menucolor][sizeRX2+20];}
		else {colordata=0xf242;}
		xnr2=datanr/62;
		xnr1=(datanr - (xnr2*62))*5;
		DrawVline(68+(9*xnr2),75+(9*xnr2),6+xnr1,colordata);
		DrawVline(68+(9*xnr2),75+(9*xnr2),7+xnr1,colordata);
		DrawVline(68+(9*xnr2),75+(9*xnr2),8+xnr1,colordata);

		sizeRX2=5;

		WiFi_send_flash();
		if (datanr>=743){startdownloadflash=0;delay_ms(50); if (Serial[0]!=1) { WiFi_close();} WiFis[6]=0;
		LCDmenu_set_XY(0,2);//
		LCDmenu_write_string_neg("      Download complete.     ",menucolor);
		}
		datanr++;

}

//=================================================================================================
void dataaquisition(void)
{

	if (Serial[0]!=1){return;}
	//USART_Config();
	uint8_t nextln;
	int ttr;

	RTC_TimeTypeDef RTC_TimeStructure;
	RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
	timer3600=((RTC_TimeStructure.RTC_Minutes*60)+RTC_TimeStructure.RTC_Seconds);  //load current time

    if (timer3600<swtimer2){timer3600=timer3600+3600;}

    if ((Serial[2])<((timer3600 - swtimer2)+1)) //see if the set time interval has elapsed.
    {
    	swtimer2=((RTC_TimeStructure.RTC_Minutes*60)+RTC_TimeStructure.RTC_Seconds); //get the new time

    	nextln=10;  //char for new line


    	compdataUSART(Serial[2]);

    	for (ttr=0;ttr<60;ttr++){
    	while (USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	USART_SendData(USART3, sbmsD[ttr]);   //send data.
    	}


    	while (USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	USART_SendData(USART3, nextln);   //new line

    }

}


//=================================================================================================
// Perform the TS acquisition and the processing of the sensors state machine
//=================================================================================================


void TSL_Action1(void)
{
  uint32_t idx_bank;
  // Acquire all Banks
  //TSLPRM_TOTAL_BANKS;
  for (idx_bank = 0; idx_bank < 3; idx_bank++)
  {
    // Configure Bank
    TSL_acq_BankConfig(idx_bank);
    // Start Bank acquisition
    TSL_acq_BankStartAcq();
    // Wait Bank End of Acquisition
    while (TSL_acq_BankWaitEOC() == TSL_STATUS_BUSY) {}
    // Get Bank Result
    TSL_acq_BankGetResult(idx_bank, 0, 0);
  }
  // Process Objects
  TSL_obj_GroupProcess(&MyObjGroup);
  // DxS processing (if TSLPRM_USE_DXS option is set)
  TSL_dxs_FirstObj(&MyObjGroup);
  if (TSL_tim_CheckDelay_ms(125, &ECS_last_tick) == TSL_STATUS_OK)
    {
      if (TSL_ecs_Process(&MyObjGroup) == TSL_STATUS_OK)
      {
        process_sensor = 0;
      }
      else
      {
        process_sensor = 1;
      }
    }
}

//========================================================================================
// ISL94203 related functions.
//========================================================================================


void RAMreadall(void)
	{
    if (USBpower==2){return;}
	sEE_Init();
    NumData = RAMSize;
    sEE_ReadBuffer(RAMrx, RAMStart, (uint16_t *)(&NumData));
    sEE_DeInit();

	}

void RAMwrite(uint8_t RAMaddress,uint8_t Tx1)
	{
	if (USBpower==2){return;}
	sEE_Init();
	  sEE_WriteBuffer((uint8_t*)(&Tx1), RAMaddress, 1);  // write one byte to RAM
	sEE_DeInit();
	delay_ms(2);
	}


void ROMwrite(uint8_t ROMaddress,uint8_t Tx1)
	{
	if (USBpower==2){return;}
	RAMwrite(0x89,1);
	delay_ms(2);
	sEE_Init();
 	sEE_WriteBuffer((uint8_t*)(&Tx1), ROMaddress, 1); //write one byte to EEPROM
 	delay_ms(2);
 	sEE_DeInit();

 	RAMwrite(0x89,0);

	}

void ROMreadall()
{
if (USBpower==2){return;}
int y1,y2,y3;

	for(y1=0;y1<=21;y1++)
	{
	y2=y1*4;
	ROMread(y2);
	for(y3=0;y3<=3;y3++)
	{
	ROMrx[y2+y3]=ROMt[y3];
	ROMtemp[y2+y3]=ROMrx[y2+y3];
	}
	}


}


void ROMwriteall()
{
if (USBpower==2){return;}
int y1,y2,y3,j,n,colorb;

    ROMtemp[74]=32; // manual write some EEPROM location
    ROMtemp[40]=255;
    ROMtemp[41]=15;
    ROMtemp[42]=255;
    ROMtemp[43]=15;
    ROMtemp[44]=0;
    ROMtemp[45]=0;
    ROMtemp[46]=0;
    ROMtemp[47]=0;
    ROMtemp[48]=0;
    ROMtemp[49]=0;
    ROMtemp[50]=0;
    ROMtemp[51]=0;
    ROMtemp[52]=255;
    ROMtemp[53]=15;
    ROMtemp[54]=255;
    ROMtemp[55]=15;
    ROMtemp[56]=0;
    ROMtemp[57]=0;
    ROMtemp[58]=0;
    ROMtemp[59]=0;
    ROMtemp[60]=255;
    ROMtemp[61]=15;
    ROMtemp[62]=255;
    ROMtemp[63]=15;

	for(y1=0;y1<=21;y1++)
	{
	y2=y1*4;
	for(y3=0;y3<=3;y3++)
	{
	ROMwrite(y2+y3,ROMtemp[y2+y3]);
	if (c2==2){LCDmenu_set_XY(0,4);}
	if (c2==23){LCDmenu_set_XY(0,8);}
	if (menucolor!=4){colorb=4;}
	else {colorb=5;}

	LCDmenu_write_Dec1_neg(2,0,y2+y3,colorb);
	if ((y2+y3)>5){

	if (c2==2){
	for (n=0; n < 18 ; n++){
	j=n*3;
	DrawHline(22, (5+y2+y3)*3.5,  80+n, bmscolor[colorb][10+j]);
	}
	}

	if (c2==23){
	for (n=0; n < 18 ; n++){
	j=n*3;
	DrawHline(22, (5+y2+y3)*3.5,  160+n, bmscolor[colorb][10+j]);
	}
	}


	//LCDmenu_bar((y2+y3)-5,82);
	}
	}
	delay_ms(2);
	}


}

void ROMread(uint8_t ROMStart)
	{
	if (USBpower==2){return;}
       RAMwrite(0x89,1);

 	   sEE_Init();
 	   NumData = 4;
 	   sEE_ReadBuffer(ROMt, ROMStart, (uint16_t *)(&NumData)); //read 4byte EEPROM

 	   sEE_DeInit();
 	   RAMwrite(0x89,0);

	}
//=================================================================================================
//                                  convert binary to decimal
//=================================================================================================

void ROMv1bd(unsigned int pp,unsigned int loop22)
{
	tempH=((ROMtemp[loop22+1]) & 0x0F);
	ROMdec[pp]=(ROMtemp[loop22]+(tempH*256))*v1+0.5;
}
void ROMunitbd(unsigned int pp,unsigned int loop22)
{
	tempH=((ROMtemp[loop22+1]) & 0x03);
	ROMdec[pp]=(ROMtemp[loop22]+(tempH*256));
	tempL=((ROMtemp[loop22+1]) & 0x0C);
	if(tempL==0){ROMunit[pp]="us ";}
	if(tempL==4){ROMunit[pp]="ms ";}
	if(tempL==8){ROMunit[pp]="s  ";}
	if(tempL==12){ROMunit[pp]="m  ";}
}

void ROMconvertbd(void)
{

unsigned int l,loop1,loop2;
static float const v2=0.4395604;


for (l=0;l<34;l++)   //load limits from table bmsmenu.h
{
d[l][0]=1;
d[l][1]=0;
ROMdec[l]=0;
ROMunit[l]=bmsmenu[l][2];
d[l][0]=bmsmenuval[l][0];
d[l][1]=bmsmenuval[l][1];
}

//-------------------------------------------------------------------

    ROMdec[0]=ROMtemp[80];  //preset cell type stored in user eeprom

    if ((ROMdec[0]==4)||(ROMdec[0]==5)||(ROMdec[0]==6))
          {
          ROMtemp[82]=((ROMtemp[82]) & 0x7F)+128;
          }
    else
    {
     ROMtemp[82]=((ROMtemp[82]) & 0x7F);
    }

//-------------------------------------------------------------------
	if(ROMtemp[73]==131){ROMdec[1]=3;}  //number of cells
	if(ROMtemp[73]==195){ROMdec[1]=4;}
	if(ROMtemp[73]==199){ROMdec[1]=5;}
	if(ROMtemp[73]==231){ROMdec[1]=6;}
	if(ROMtemp[73]==239){ROMdec[1]=7;}
	if(ROMtemp[73]==255){ROMdec[1]=8;}
//-------------------------------------------------------------------
	tempH=((ROMtemp[82]) & 0x7F);
	ROMdec[2]=(ROMtemp[81]+(tempH*256));
	tempL=((ROMtemp[82]) & 0x80);
    if(tempL==0){ROMunit[2]="Ah ";}
	if(tempL==128){ROMunit[2]="F  ";}
//-------------------------------------------------------------------
	 ROMv1bd(3,0); // overvoltage threshold;
//-------------------------------------------------------------------
 	    ROMunitbd(4,16);
//-------------------------------------------------------------------
	 ROMv1bd(5,2); // overvoltage recovery;
//-------------------------------------------------------------------
	 ROMv1bd(6,4); // Undervoltage threshold;
//-------------------------------------------------------------------
	 ROMunitbd(7,18); // undervoltage delay;
//-------------------------------------------------------------------
   loop2=6;  // Undervoltage recovery, Overvoltage Lock-out, Undervoltage Lock-out, EOC,  Low voltage charge;
 	     	for(loop1=8;loop1<13;loop1++)
 	     	{
 	     		ROMv1bd(loop1,loop2);
 	     		loop2=loop2+2;
 	     	}
//-------------------------------------------------------------------
   	    tempH=((ROMtemp[25]) & 0x70);       //Charge current;
   	    if(tempH!=0){ROMdec[13]=realcurrent[tempH/16][2];ROMdec2[0]=tempH/16;}
   	   	else{ROMdec[13]=realcurrent[0][2];ROMdec2[0]=0;}
//-------------------------------------------------------------------
   	    ROMunitbd(14,24);  // Charge Current Time;
//-------------------------------------------------------------------
   	 	tempH=((ROMtemp[23]) & 0x70);            //Discharge current
   	    if(tempH!=0){ROMdec[15]=realcurrent[tempH/16][1];ROMdec2[1]=tempH/16;}
   	    else{ROMdec[15]=realcurrent[0][1];ROMdec2[1]=0;}
//-------------------------------------------------------------------
   	    ROMunitbd(16,22);  // Discharge Current Time;
//-------------------------------------------------------------------
  	 	tempH=((ROMtemp[27]) & 0x70);            //Short circuit current
  	 	if(tempH!=0){ROMdec[17]=realcurrent[tempH/16][3];ROMdec2[2]=tempH/16;}
  	 	else{ROMdec[17]=realcurrent[0][3];ROMdec2[2]=0;}

//-------------------------------------------------------------------
  	 	ROMunitbd(18,26);  // Short circuit current time;
//-------------------------------------------------------------------
  	    tempH=((ROMtemp[75]) & 0x80);
  	   	if(tempH!=0){ROMdec[19]=tempH/128;}
  	   	else{ROMdec[19]=0;}// CB during discharge
//-------------------------------------------------------------------
  	   	tempH=((ROMtemp[75]) & 0x40);
  	   	if(tempH!=0){ROMdec[20]=tempH/64;}
  	   	else{ROMdec[20]=0;}// CB during charge
//-------------------------------------------------------------------
  	   // CB min,max voltage and CB min, max delta
  	   	ROMv1bd(21,28);
  	   	ROMv1bd(22,30);
  	    tempH=((ROMtemp[33]) & 0x0F);
  	  	ROMdec[23]=((ROMtemp[32]+(tempH*256))*(v1*10))+0.5;
  	    tempH=((ROMtemp[35]) & 0x0F);
  	    ROMdec[24]=((ROMtemp[34]+(tempH*256))*(v1*10))+0.5;
//-------------------------------------------------------------------
  	   	ROMunitbd(25,36);    	  // CB ON Time;
//-------------------------------------------------------------------
  	    ROMunitbd(26,38);  // CB OFF Time;
//-------------------------------------------------------------------
  	    //dac ROMdec[27]=ROMtemp[83];
  	    tempH=((ROMtemp[65]) & 0x0F);
  	    ROMdec[27]=(ROMtemp[64]+(tempH*256))*v2+0.5;

//-------------------------------------------------------------------
        //tempH=((ROMtemp[65]) & 0x0F);
        //ROMdec[28]=(ROMtemp[64]+(tempH*256))*v2+0.5;
  	    tempH=ROMtemp[85];
  	    ROMdec[28]=(ROMtemp[84]+(tempH*256));


//-------------------------------------------------------------------
        //tempH=((ROMtemp[67]) & 0x0F);
        //ROMdec[29]=(ROMtemp[66]+(tempH*256))*v2+0.5;
  	    tempH=ROMtemp[87];
  	    ROMdec[29]=(ROMtemp[86]+(tempH*256));
//-------------------------------------------------------------------
        ROMv1bd(30,68); // Low voltage sleep
//-------------------------------------------------------------------
  	    tempH=((ROMtemp[71]) & 0x03);
  	    ROMdec[31]=(ROMtemp[70]+(tempH*256));  // Low voltage sleep delay
  	    tempL=((ROMtemp[71]) & 0x04);
  	    if(tempL==0){ROMunit[31]="ms";}
  	    if(tempL==4){ROMunit[31]="s";}
//-------------------------------------------------------------------
  	    tempH=((ROMtemp[72]) & 0x0F);
  	    ROMdec[32]=tempH;  // Doze Time
//-------------------------------------------------------------------
  	    tempH=((ROMtemp[72]) & 0xF0);
  	    ROMdec[33]=tempH;  // Sleep Time
//-------------------------------------------------------------------
}


//=================================================================================================
//                                  convert decimal to binary
//=================================================================================================

void ROMv1db(unsigned int pp,unsigned int loop22)
{
	uint16_t temp16;
	temp16 = (ROMdec[pp]/v1)+0.5;
	ROMtemp[loop22]=(uint8_t)temp16;
	ROMtemp[loop22+1]=((uint8_t)(temp16>>8));
}
void ROMunitdb(unsigned int pp,unsigned int loop22,unsigned int timeu)
{
	uint16_t temp16;
	temp16 = ROMdec[pp];
	ROMtemp[loop22]=(uint8_t)temp16;
    ROMtemp[loop22+1]=(uint8_t)(temp16>>8)+timeu;
}


void ROMconvertdb(void)
{


static float const v2=0.4395604;
uint16_t temp16;
unsigned int loop,loop2;

//-------------------------------------------------------------------

    ROMtemp[80]=ROMdec[0];  //preset cell type stored in user eeprom

//-------------------------------------------------------------------

	if(ROMdec[1]==3){ROMtemp[73]=131;}  //number of cells
	if(ROMdec[1]==4){ROMtemp[73]=195;}
	if(ROMdec[1]==5){ROMtemp[73]=199;}
	if(ROMdec[1]==6){ROMtemp[73]=231;}
	if(ROMdec[1]==7){ROMtemp[73]=239;}
	if(ROMdec[1]==8){ROMtemp[73]=255;}
//-------------------------------------------------------------------
	 //capacity
	temp16 = ROMdec[2];
	ROMtemp[81]=(uint8_t)temp16;
	tempL=((ROMtemp[82]) & 0x80);
	ROMtemp[82]=(uint8_t)(temp16>>8)+(tempL*128); //for capacitor or battery

//-------------------------------------------------------------------
	// Overvoltage threshold;
	ROMv1db(3,0);
	ROMtemp[1]=ROMtemp[1]+16;//the +16 is for 1ms

//-------------------------------------------------------------------
  	//Overvoltage delay
 	ROMunitdb(4,16,8); //the 8 is for seconds
//-------------------------------------------------------------------
 	// Overvoltage recovery;
 	ROMv1db(5,2);
//-------------------------------------------------------------------
    // Undervoltage threshold;
    ROMv1db(6,4);
    ROMtemp[5]=ROMtemp[5]+16; //the +16 is for 1ms
//-------------------------------------------------------------------
 	// Undervoltage delay;
 	ROMunitdb(7,18,8);// the 8 is for seconds
//-------------------------------------------------------------------
 	loop2=6;    // Undervoltage recovery, Overvoltage Lock-out, Undervoltage Lock-out, EOC,  Low voltage charge;
 	 	     for(loop=8;loop<13;loop++)
 	 	     {
 	 	    	ROMv1db(loop,loop2);
 	 	        loop2=loop2+2;
 	 	     }

//-------------------------------------------------------------------
 	ROMdec[13]=ROMdec2[0];
 	ROMdec[15]=ROMdec2[1];
 	ROMdec[17]=ROMdec2[2];
//-------------------------------------------------------------------
 	ROMunitdb(14,24,4); // Charge Current Time   the 4 is for ms
    ROMtemp[25]=ROMtemp[25]+ROMdec[13]*16;  //Charge current;
//-------------------------------------------------------------------
    ROMunitdb(16,22,4);   // Discharge Current Time  the 4 is for ms
    ROMtemp[23]=ROMtemp[23]+ROMdec[15]*16;  //Discharge current
//-------------------------------------------------------------------
    ROMunitdb(18,26,0);  // Short circuit current time, the 0 for us
    ROMtemp[27]=ROMtemp[27]+ROMdec[17]*16;//Short circuit current
//-------------------------------------------------------------------
  	// CB during discharge
  	ROMtemp[75]=ROMdec[19]*128;
  	// CB during charge
  	ROMtemp[75]=ROMtemp[75]+ROMdec[20]*64;
//-------------------------------------------------------------------
  	// CB min voltage;
  	ROMv1db(21,28);
//-------------------------------------------------------------------
  	// CB max voltage;
  	ROMv1db(22,30);
//-------------------------------------------------------------------
    // CB min delta;
  	temp16 = (ROMdec[23]/(v1*10))+0.5;
  	ROMtemp[32]=(uint8_t)temp16;
  	ROMtemp[33]=((uint8_t)(temp16>>8));
//-------------------------------------------------------------------
    // CB max delta;
    temp16 = (ROMdec[24]/(v1*10))+0.5;
    ROMtemp[34]=(uint8_t)temp16;
    ROMtemp[35]=((uint8_t)(temp16>>8));
//-------------------------------------------------------------------
  	// CB ON Time;
  	ROMunitdb(25,36,8);  // the 8 is for seconds

//-------------------------------------------------------------------
    // CB ON Time;
    ROMunitdb(26,38,8);  // the 8 is for seconds
//-------------------------------------------------------------------
   //dac ROMtemp[83]=ROMdec[27];
   //internal overtemp and recovery
    temp16 = (ROMdec[27]/v2)+0.5;
    ROMtemp[64]=(uint8_t)temp16;
    ROMtemp[65]=(uint8_t)(temp16>>8);

    temp16 = ((ROMdec[27]-40)/v2)+0.5;
    ROMtemp[66]=(uint8_t)temp16;
    ROMtemp[67]=(uint8_t)(temp16>>8);

//-------------------------------------------------------------------
    //dac temp16 = (ROMdec[28]/v2)+0.5;
    //ROMtemp[64]=(uint8_t)temp16;
    //ROMtemp[65]=(uint8_t)(temp16>>8);
    temp16 = ROMdec[28];
    ROMtemp[84]=(uint8_t)temp16;
    ROMtemp[85]=(uint8_t)(temp16>>8);

//-------------------------------------------------------------------
    //temp16 = (ROMdec[29]/v2)+0.5;
    //ROMtemp[66]=(uint8_t)temp16;
    //ROMtemp[67]=(uint8_t)(temp16>>8);
    temp16 = ROMdec[29];
    ROMtemp[86]=(uint8_t)temp16;
    ROMtemp[87]=(uint8_t)(temp16>>8);

//-------------------------------------------------------------------
  	// Low voltage sleep
  	ROMv1db(30,68);
//-------------------------------------------------------------------
  	// Low voltage sleep delay
  	temp16 = ROMdec[31];
  	ROMtemp[70]=(uint8_t)temp16;
  	ROMtemp[71]=(uint8_t)(temp16>>8)+ 0xFC;  //FC is fixed for 31 sec WD and seconds
//-------------------------------------------------------------------
    ROMtemp[72]=ROMdec[32];

  	ROMtemp[72]=ROMtemp[72]+ROMdec[33];  // Sleep Time
//-------------------------------------------------------------------

}

//========================================================================================
//            *****************          Menu                *****************
//========================================================================================
void mainmenu1(void)
{
	if ((c2<100)||(c2==0)){return;}

	unsigned int v2,s,i2;
    s=7;
    refresh1=1;  //to refresh the parameter settings

    if (first1>=1){	DrawFilledRectangle(0, 180, 320, 240, 0x0001);}
    first1=0;

    if (c3<1){c3=s;}
	if (c3>=(s+1)){c3=1;}


	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
	LCDmenu_set_XY(1,0);
	LCDmenu_write_string_neg("/",menucolor);
	LCDmenu_set_XY(2,0);
	LCDmenu_write_Dec1_neg(1,0,s,menucolor);

	LCDmenu_set_XY(3,0);
    LCDmenu_write_string_neg("      Main Menu           ",menucolor);

	if ( (c3<=(s+1)) && (c3>=1) )
		                    {
		            v2=c3-1;
		            if (v2<s)
		            {

		            if (new1==0){
		            LCDmenu_set_XY(0,1); //dacus test menu
				    LCDmenu_write_string(mainmenu[0][0],menucolor);
				    LCDmenu_set_XY(0,2); //dacus test menu
				    LCDmenu_write_string(mainmenu[1][0],menucolor);
				    LCDmenu_set_XY(0,3); //dacus test menu
				    LCDmenu_write_string(mainmenu[2][0],menucolor);
				    LCDmenu_set_XY(0,4); //dacus test menu
				    LCDmenu_write_string(mainmenu[3][0],menucolor);
				    LCDmenu_set_XY(0,5); //dacus test menu
				    LCDmenu_write_string(mainmenu[4][0],menucolor);
				    LCDmenu_set_XY(0,6); //dacus test menu
				   	LCDmenu_write_string(mainmenu[5][0],menucolor);
				   	LCDmenu_set_XY(0,7); //dacus test menu
				   	LCDmenu_write_string(mainmenu[6][0],menucolor);
				   	LCDmenu_set_XY(0,8); //dacus test menu
				   	LCDmenu_write_string(mainmenu[7][0],menucolor);
		            }
		            if (b3>55){b3=35;}
		            if (b4>55){b4=35;}

		                                    if ((b3!=0)&&(c3!=1))
		                                    {
		                                    LCDmenu_set_XY(0,c3-1); //dacus test menu
		                                    LCDmenu_write_string(mainmenu[c3-2][0],menucolor);
		                                    }

		                                    if ((b3!=0)&&(c3==1))
		                                             {
		                                     LCDmenu_set_XY(0,s); //dacus test menu
		                                     LCDmenu_write_string(mainmenu[s-1][0],menucolor);
		                                              }
		                                    if ((b4!=0)&&(c3!=s))
		                                      {
		                                       LCDmenu_set_XY(0,c3+1); //dacus test menu
		                                       LCDmenu_write_string(mainmenu[c3][0],menucolor);
		                                      }

		                                     if ((b4!=0)&&(c3==s))
		                                       {
		                                        LCDmenu_set_XY(0,1); //dacus test menu
		                                        LCDmenu_write_string(mainmenu[0][0],menucolor);
		                                       }


				    LCDmenu_set_XY(0,c3); //dacus test menu
				    LCDmenu_write_string_neg(mainmenu[c3-1][0],menucolor);

				    for(i2=0;i2<32;i2++)  //clear text buffer for the first 10 locations
				    {
				    chr[i2]=0x20;
				    }

				    helptext=mainmenu[c3-1][1];
				    scrolllenght=29;
				    nr1=0;
				    c33=c3;
				    z=0;
				    i4=0;   //delay time for scrolling text
				    i5=1;   //activate scrolling text

				    while (*helptext!='\0')
				                  {
				                     chr[z]=*helptext;
				                     z++;
				                     helptext++;
				                  }
				    if (z>=8)
				    {
				        for (i2=0;i2<scrolllenght;i2++)
				        {
				         chr[z+i2]=chr[i2];
				        }

				    }

new1=1;


		            }

      }


	if (BOK==1)
		{
		BOK=0;
		c2=c3;
		c333=c3;
		c3=1;
		i5=0;
		new1=0;
		if ((c333==1)||(c333==2)||(c333==4)||(c333==5)){key1=1; first1=1;}
		Fill_menu(0x0000);//LCDmenu_clear();
         	}

	}

//=================================================================================================
//          *************************         Monitor            **********************
//=================================================================================================

void monitor (void)

{

	unsigned int k,k1,jjj;
	uint32_t offsetbat;

	for (k=0;k<16;k=k+2)   //cell voltages;
	   {
	    c8=((((RAMrx[17+k]*256)+RAMrx[16+k])*(1.8*8))/(4095*3))*1000;
	     if (k>=2)
	     {
	     BV[k/2][4]=BV[k/2][3];
	     BV[k/2][3]=BV[k/2][2];
	     BV[k/2][2]=BV[k/2][1];
	     BV[k/2][1]=BV[k/2][0];
	     BV[k/2][0]=c8;
	     }
	     else
	     {
		 BV[0][4]=BV[0][3];
		 BV[0][3]=BV[0][2];
	     BV[0][2]=BV[0][1];
	     BV[0][1]=BV[0][0];
	     BV[0][0]=c8;
	     }

	    }

	for(k=0;k<8;k++)  //cell voltage filter;
	{
		 BV[k][5]=((BV[k][0]+BV[k][1]+BV[k][2]+BV[k][3]+BV[k][4])/5);
	}

	BVmin[0]=BV[0][5];
	BVmin[1]=0;
	BVmax[0]=BV[0][5];
	BVmax[1]=0;
	for (k=1;k<8;k++)  //find min and max cell
	{
		if((BV[k][5]<BVmin[0])&&(BV[k][5]>100))
		{BVmin[0]=BV[k][5];BVmin[1]=k;}
		if(BV[k][5]>BVmax[0])
		{BVmax[0]=BV[k][5];BVmax[1]=k;}
	}
   BVdelta=BVmax[0]-BVmin[0];


   //cell balancing.

   k1=0;
   for (k=1;k<129;k=k+k)
   {
	  tempH=((RAMrx[4]) & k);
	  if (tempH>0)
	  {CB[k1]=1;}
	  else
	  {CB[k1]=0;}
	  k1++;
   }


   // error flags

   k1=0;
      for (k=1;k<9;k=k+k)    //OV, OVLO, UV, UVLO
      {
   	  tempH=((RAMrx[0]) & k);
   	  if (tempH>0)
   	  {EF[k1]=1;}
   	  else
   	  {EF[k1]=0;}
   	  k1++;
      }

      if(EF[1]==0 && EF[3]==0){ERROR5=0;}
      else {ERROR5=1;}



      for (k=1;k<33;k=k+k)   //IOT, COC, DOC, DSC, CELLF, OPEN
      {
      tempH=((RAMrx[1]) & k);
      if (tempH>0)
      {EF[k1]=1;}
      else
      {EF[k1]=0;}
      k1++;
      }


       tempH=((RAMrx[1]) & 128);  //EOC
       if (tempH>0)
       {EF[13]=1;}
       else
       {EF[13]=0;}

       tempH=((RAMrx[2]) & 32);  //ECC_Faill
       if (tempH>0)
       {EF[11]=1;}
       else
       {EF[11]=0;}

       tempH=((RAMrx[2]) & 128); //LVC
       if (tempH>0)
       {EF[10]=1;}
       else
       {EF[10]=0;}

       tempH=((RAMrx[6]) & 1);   //DFET
       if (tempH>0)
       {EF[14]=1;DFET=1;}
       else
       {EF[14]=0;DFET=0;}

       tempH=((RAMrx[6]) & 2);   //CFET
       if (tempH>0)
       {EF[12]=1;CFET=1;}
       else
       {EF[12]=0;CFET=0;}
       ERR=0;
       for (k=0;k<15;k++){ERR=ERR+(EF[k]*(pow(2, k)));}



       UVlock=ROMdec[10]*10;   //under voltage lock for the bar graph low limit
       OVlock=ROMdec[9]*10;    //over voltage lock for the bar graph high limit


	    tempH=((RAMrx[2]) & 0x0C);  //+/- charge or discharge
                    if (tempH==0){direction=" ";Cdirection=0;}
          	        if (tempH==4){direction="+";Cdirection=1;}
         	        if (tempH==8){direction="-";Cdirection=0;}



	    tempL=((RAMrx[5]) & 0x30);  //read the battery current amplifier gain
	    if (tempL==0){gain=50;}  //50
	    if (tempL==16){gain=5;}  //5
        if (tempL==32 || tempL==48){gain=500;}  //500

        //ITEMP=((RAMrx[33]*256)+RAMrx[32]);  //Internal temp

        ETEMP=((RAMrx[35]*256)+RAMrx[34]);  //ext temp XT1

        ITEMP=((RAMrx[37]*256)+RAMrx[36]);  //ITEMP power board temp.



        if (ITEMP>Therm[0]){ITEMP2=450;}//lower than -40C
        if (ETEMP>Therm[0]){ETEMP2=450;}//lower than -40C

        for (jjj=0;jjj<4;jjj++){
        if ((ITEMP<=Therm[jjj])&(ITEMP>Therm[jjj+1])){ITEMP2=(300-(jjj*100))+((ITEMP-Therm[jjj+1])*Therm2[jjj]);}
        if ((ETEMP<=Therm[jjj])&(ETEMP>Therm[jjj+1])){ETEMP2=(300-(jjj*100))+((ETEMP-Therm[jjj+1])*Therm2[jjj]);}
        }
        for (jjj=4;jjj<14;jjj++){
        if ((ITEMP<=Therm[jjj])&(ITEMP>Therm[jjj+1])){ITEMP2=((jjj*100)-300)-((ITEMP-Therm[jjj+1])*Therm2[jjj]);}
        if ((ETEMP<=Therm[jjj])&(ETEMP>Therm[jjj+1])){ETEMP2=((jjj*100)-300)-((ETEMP-Therm[jjj+1])*Therm2[jjj]);}
        }

        if (ITEMP<=Therm[14]){ITEMP2=999;}//higher than +99C
        if (ETEMP<=Therm[14]){ETEMP2=999;}//higher than +99C




	    c9=(RAMrx[15]*256)+RAMrx[14];  //battery current

	    if (gain==50) {c9cal=c9calx50;}
	    if (gain==500) {c9cal=c9calx500;}
	    //c8=0;


	/*    if ((gain==50)&&(c9>=250))
	    {
	    c9cal=c9calx50;
	    rangeswitch=0;
	    }
	    if ((gain==50)&&(c9<250))
	    {
	    rangeswitch++; //on high noise loads like inverters stay on x50 gain
	    if (rangeswitch>60){
	    RAMwrite(0x85,32); //set current amplifier gain  x500 (32)
	    j=0;
	    filter3=0;
	    rangeswitch=0;
	    }
	    c9cal=c9calx50;
	    }
	    if ((gain==500)&&(c9<=3500))
	    {
	    c9cal=c9calx500;
	    }
	    if ((gain==500)&&(c9>3500))
	    {
	    RAMwrite(0x85,0); //set current amplifier gain x50 (00) x5 (16) x500 (32) x500 (48)
	    c9cal=c9calx500;
	    j=0;
	    filter3=0;
	    }
	    //LCDmenu_set_XY(2,1);
        //LCDmenu_write_Dec1_neg(4,0,c9);

                    if ((tempH!=4) && (c9>c9cal+3))
                    {c9=c9-c9cal;}
                    if (tempH==4)
                    {c9=c9+c9cal;}
                    if (c9<=c9cal+3)
                    {c9=0;}


                    if (c8>c8cal+8)
                    {c8=c8-c8cal+10;}
                    else{c8=0;}
*/

	   BatteryVoltage=((BV[0][5]+BV[1][5]+BV[2][5]+BV[3][5]+BV[4][5]+BV[5][5]+BV[6][5]+BV[7][5])); //sum of cell voltage. no div/10
       BatteryVoltage2=((((RAMrx[39]*256)+RAMrx[38])*(1.925*32))/(4095))*1000;  //direct battery reading

	    //if (sj>=4)
	    //        {
	    //	    SolarCurrent=(((((cs7+sc1+sc2)/12)*1.8)/(4095*PVshunt*PVgain))*10);   //PV current
	    //        sj=0;
	    //        sc2=sc1;
	    //        sc1=cs7;
	    //        cs7=0;
	    //        filter1=0;
	    //        }
	    //        else
	    //        {
	    //        if ((c8<((cs7+sc1+sc2)/12)-3)&&(c8>((cs7+sc1+sc2)/12)+3)&&(filter1<=4)){filter1++;}
	    //        else {cs7=c8+cs7;sj++;}
	    //        }

       if (ADC16[0][0]>ADC16[0][5+ADC16[0][1]]){
        PV2=(((((((ADC16[0][0]-ADC16[0][5+ADC16[0][1]])/gain_real[ADC16[0][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[5]))*100);  //needs more work
       }
       else {PV2=0;}
       if (PV2>=15){PV2=PV2+EXTADC1[6];}


       if (ADC16[5][0]>ADC16[5][5+ADC16[5][1]]){
        PV1=(((((((ADC16[5][0]-ADC16[5][5+ADC16[5][1]])/gain_real[ADC16[5][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[3]))*100);
       }
       else {PV1=0;}
       if (PV1>=15){PV1=PV1+EXTADC1[4];}


       if (ADC16[1][0]>ADC16[1][5+ADC16[1][1]]){
       ADC2=ADC16[1][0];
       }
       else {ADC2=0;}

       if (ADC16[3][0]>ADC16[3][5+ADC16[3][1]]){
       ADC3=ADC16[3][0];
       }
       else {ADC3=0;}

       if (ADC16[4][0]>ADC16[4][5+ADC16[4][1]]){
       ADC4=ADC16[4][0];
       }
       else {ADC4=0;}



       PV12=PV1+PV2;

       // EXT  ADC ----------------------  needs some changes on the gain1 and PVshunt
       if ((ADC16[2][0]>ADC16[2][5+ADC16[2][1]])){
       ExtLoadCurrent=(((((((ADC16[2][0]-ADC16[2][5+ADC16[2][1]])/gain_real[ADC16[2][1]])*((1.8/32768)/gain2)))*10000000)/(EXTADC1[1]))*100);  //needs more work
       }
       else {ExtLoadCurrent=0;}
       if (ExtLoadCurrent>=15){ExtLoadCurrent=ExtLoadCurrent+EXTADC1[2];}
       if (EXTADC1[0]==0){ExtLoadCurrent=0;}
       if (ExtLoadCurrent>=600000){ExtLoadCurrent=600000;}



	 /*   if (j>=4)
	        {
	        BatteryCurrent=((((((c7+bc1+bc2)/12)*1.8)/(4095*Battshunt*gain1))*10)-0.5);  //Battery current
	    	j=0;
	        bc2=bc1;
	        bc1=c7;
	        c7=0;
	        filter3=0;
	        }
	        else
	        {
	        if ((c9<((c7+bc1+bc2)/12)-3)&&(c9>((c7+bc1+bc2)/12)+3)&&(filter3<=4)){filter3++;}
	        else {c7=c9+c7;j++;}
	        }  */
       BatteryCurrent=((((c9*1.8)/(4095*Battshunt*gain))*10)-0.5);  //Battery current


offsetbat=ADC16[6][5+ADC16[6][1]];


	    if (ADC16[6][0]>=offsetbat)
	       	   		 {
	       		     BatteryCurrent2=(((((((ADC16[6][0]-offsetbat)/gain_real[ADC16[6][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[7]))*100);
	       		     if (PV12>=BatteryCurrent2){LoadCurrent= (PV12)-BatteryCurrent2;}
	       		     else{LoadCurrent=0;}

	       		     if (ExtLoadCurrent>=BatteryCurrent2)
	       		     {
	       		     BatteryCurrent3=ExtLoadCurrent-BatteryCurrent2;
	       		     direction2="-";
	       		     direction22=0;
	       		     }
	       		     else
	       		     {
	       		     BatteryCurrent3=BatteryCurrent2-ExtLoadCurrent;
	       		     direction2="+";
	       		     direction22=1;

	       		     }

	       	   		 }
	       	   		 else
	       	   		 {
	       	   		 BatteryCurrent2=(((((((offsetbat-ADC16[6][0])/gain_real[ADC16[6][1]])*((1.8/32768)/gain1)))*10000000)/(EXTADC1[7]))*100);
	       	   	     direction2="-";
	       	   	     direction22=0;
	       	   	     LoadCurrent= (PV12)+BatteryCurrent2;
	       	   		 BatteryCurrent3=ExtLoadCurrent+BatteryCurrent2;
	       	   		 }


	    BatteryW=((BatteryCurrent3/10)*(BatteryVoltage/10))/1000; //Battery power [W]
	    PV1W=((PV1/10)*(BatteryVoltage/10))/1000; //Solar power [W]
	    PV2W=((PV2/10)*(BatteryVoltage/10))/1000;
	    PV12W=PV1W+PV2W;
	    LoadW=((LoadCurrent/10)*(BatteryVoltage/10))/1000; //Load power [W]
	    ExtLoadW=((ExtLoadCurrent/10)*(BatteryVoltage/10))/1000;
	    LoadExtLoadW=LoadW+ExtLoadW;

	    //calculate SOC and BatteryAh BatteryWh counters
	    if(zz2==1)
	    {
	    	PV1graph[240]=PV1;
	    	PV2graph[240]=PV2;
	    	if (direction22==0){Battgraph[240]=12000000-BatteryCurrent3;}
	    	else {Battgraph[240]=12000000+BatteryCurrent3;}
	    	VBattgraph[240]=BatteryVoltage;
	    	Loadgraph[240]=LoadCurrent;
	    	ExtLoadgraph[240]=ExtLoadCurrent;


	    	//240sec*******************************************************************

	    	 for (jjj = 1; jjj < 241; ++jjj)
	    	     {
	    	     PV1graph[jjj-1]=PV1graph[jjj];//pv1 240sec buffer
	    	     PV2graph[jjj-1]=PV2graph[jjj];
	    	     Battgraph[jjj-1]=Battgraph[jjj];
	    	     VBattgraph[jjj-1]=VBattgraph[jjj];
	    	     Loadgraph[jjj-1]=Loadgraph[jjj];
	    	     ExtLoadgraph[jjj-1]=ExtLoadgraph[jjj];
	    	     }

	    	count1m++;

	    	//240min*******************************************************************
	    	if (count1m==60)
	    	{

            //PV1----------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 181; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+PV1graph[jjj];}

	    	PV1graph1h[240]=graphvaltemp/60;

	    	for (jjj = 1; jjj < 241; ++jjj){PV1graph1h[jjj-1]=PV1graph1h[jjj];}
            //-------------------------------------------------------------------------


	    	//PV2 ---------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 181; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+PV2graph[jjj];}

	    	PV2graph1h[240]=graphvaltemp/60;

	    	for (jjj = 1; jjj < 241; ++jjj){PV2graph1h[jjj-1]=PV2graph1h[jjj];}
	    	//-------------------------------------------------------------------------


	    	//Batt --------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 181; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+Battgraph[jjj];}

	    	Battgraph1h[240]=graphvaltemp/60;

	    	for (jjj = 1; jjj < 241; ++jjj){Battgraph1h[jjj-1]=Battgraph1h[jjj];}
	    	//-------------------------------------------------------------------------


	    	//VBatt -------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 181; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+VBattgraph[jjj];}

	    	VBattgraph1h[240]=graphvaltemp/60;

	    	for (jjj = 1; jjj < 241; ++jjj){VBattgraph1h[jjj-1]=VBattgraph1h[jjj];}
	    	//-------------------------------------------------------------------------


	    	//Load --------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 181; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+Loadgraph[jjj];}

	    	Loadgraph1h[240]=graphvaltemp/60;

	    	for (jjj = 1; jjj < 241; ++jjj){Loadgraph1h[jjj-1]=Loadgraph1h[jjj];}
	    	//-------------------------------------------------------------------------

	    	//ExtLoad -----------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 181; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+ExtLoadgraph[jjj];}

	    	ExtLoadgraph1h[240]=graphvaltemp/60;

	    	for (jjj = 1; jjj < 241; ++jjj){ExtLoadgraph1h[jjj-1]=ExtLoadgraph1h[jjj];}
	    	//-------------------------------------------------------------------------

	    	count1m=0;
	    	count6m++;


	    	}
	    	//24h**********************************************************************

	    	if (count6m==6)
	    	{

	        //PV1----------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 235; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+PV1graph1h[jjj];}

	    	PV1graph24h[240]=graphvaltemp/6;

	    	for (jjj = 1; jjj < 241; ++jjj){PV1graph24h[jjj-1]=PV1graph24h[jjj];}
            //-------------------------------------------------------------------------

	        //PV2----------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 235; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+PV2graph1h[jjj];}

	    	PV2graph24h[240]=graphvaltemp/6;

	    	for (jjj = 1; jjj < 241; ++jjj){PV2graph24h[jjj-1]=PV2graph24h[jjj];}
            //-------------------------------------------------------------------------

	        //Batt---------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 235; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+Battgraph1h[jjj];}

	    	Battgraph24h[240]=graphvaltemp/6;

	    	for (jjj = 1; jjj < 241; ++jjj){Battgraph24h[jjj-1]=Battgraph24h[jjj];}
            //-------------------------------------------------------------------------

	        //VBatt--------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 235; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+VBattgraph1h[jjj];}

	    	VBattgraph24h[240]=graphvaltemp/6;

	    	for (jjj = 1; jjj < 241; ++jjj){VBattgraph24h[jjj-1]=VBattgraph24h[jjj];}
            //-------------------------------------------------------------------------

	        //Load---------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 235; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+Loadgraph1h[jjj];}

	    	Loadgraph24h[240]=graphvaltemp/6;

	    	for (jjj = 1; jjj < 241; ++jjj){Loadgraph24h[jjj-1]=Loadgraph24h[jjj];}
            //-------------------------------------------------------------------------

	        //ExtLoad------------------------------------------------------------------
	    	graphvaltemp=0;
	    	for (jjj = 235; jjj < 241; ++jjj){graphvaltemp=graphvaltemp+ExtLoadgraph1h[jjj];}

	    	ExtLoadgraph24h[240]=graphvaltemp/6;

	    	for (jjj = 1; jjj < 241; ++jjj){ExtLoadgraph24h[jjj-1]=ExtLoadgraph24h[jjj];}
            //-------------------------------------------------------------------------

	    	count6m=0;

	    	}
	    	//*************************************************************************************


	    	if (direction22!=1)
	    	{
	    	BatterySOC=BatterySOC-(((BatteryCurrent3)*10)/36);

	    	BatteryuAh=(((BatteryCurrent3)*10)/36)+BatteryuAh;
	    	BatterymAh=(BatteryuAh/1000);

	    	BatteryuWh=(((((BatteryCurrent3)*10)/36)*(BatteryVoltage/10))/100)+BatteryuWh;
	    	BatteryWh=(BatteryuWh/100000);//0.1Wh

	    	BatteryuWh2=(((((BatteryCurrent3)*10)/36)*(BatteryVoltage/10))/100)+BatteryuWh2;
	    	BatterykWh2=(BatteryuWh2/1000000000);
	    	}
	    	else
	    	{
	    	BatterySOC=(((BatteryCurrent3)*10)/36)+BatterySOC;
	    	}


	    	PV1uAh=((PV1*10)/36)+PV1uAh;
	    	PV1mAh=PV1uAh/1000;

	    	PV1uWh=((((PV1*10)/36)*(BatteryVoltage/10))/100)+PV1uWh;
	    	PV1Wh=(PV1uWh/100000);//0.1Wh

	    	PV1uWh2=((((PV1*10)/36)*(BatteryVoltage/10))/100)+PV1uWh2;//eeprom
	    	PV1kWh2=(PV1uWh2/1000000000);//eeprom

	    	PV2uAh=((PV2*10)/36)+PV2uAh;
	    	PV2mAh=PV2uAh/1000;

	    	PV2uWh=((((PV2*10)/36)*(BatteryVoltage/10))/100)+PV2uWh;
	    	PV2Wh=(PV2uWh/100000);//0.1Wh

	    	PV2uWh2=((((PV2*10)/36)*(BatteryVoltage/10))/100)+PV2uWh2;//eeprom
	    	PV2kWh2=(PV2uWh2/1000000000);//eeprom

	    	LoaduAh=((LoadCurrent*10)/36)+LoaduAh;
	    	LoadmAh=LoaduAh/1000;

	    	LoaduWh=((((LoadCurrent*10)/36)*(BatteryVoltage/10))/100)+LoaduWh;
	    	LoadWh=(LoaduWh/100000);//Wh/10 , 0.1Wh


	    	ExtLoaduAh=((ExtLoadCurrent*10)/36)+ExtLoaduAh;
	    	ExtLoadmAh=ExtLoaduAh/1000;

	    	ExtLoaduWh=((((ExtLoadCurrent*10)/36)*(BatteryVoltage/10))/100)+ExtLoaduWh;
	    	ExtLoadWh=(ExtLoaduWh/100000);//Wh/10 , 0.1Wh


	    		    	if ((BatterySOC>=BattCapacity*2)||(EF[13]==1)){BatterySOC=BattCapacity*2;} //I need to add end of charge as condition
	    		    	if (BatterySOC<=BattCapacity){BatterySOC=BattCapacity;}                    //cell undervoltage needed as condition.

	    		    	if (BatterymAh>=100000000000){BatterymAh=0;BatteryuAh=0;}  //reset BatteryAh counter when over 99.999MAh
	    		    	if (BatteryWh>= 1000000000){BatteryWh=0;BatteryuWh=0;}  //reset BatteryWh counter when over    99.999MWh
	    		    	if (BatterykWh2>=1000000){BatterykWh2=0;BatteryuWh2=0;}  //reset BatteryWh counter when over  999.999MWh

	    		    	if (PV2mAh>=50000000000){PV2mAh=0;PV2uAh=0;}  //reset SolarPV2Ah counter when over 49.999MAh
	    		    	if (PV2Wh>= 500000000){PV1Wh=0;PV2Wh=0;}  //reset SolarPV2Wh counter when over     49.999MWh
	    		    	if (PV2kWh2>=500000){PV2kWh2=0;PV2uWh2=0;}  //reset SolarPV2Wh counter when over  499.999MWh

	    		    	if (PV1mAh>=50000000000){PV1mAh=0;PV1uAh=0;}  //reset SolarPV1Ah counter when over 49.999MAh
	    		    	if (PV1Wh>= 500000000){PV1Wh=0;PV1Wh=0;}  //reset SolarPV1Wh counter when over     49.999MWh
	    		    	if (PV1kWh2>=500000){PV1kWh2=0;PV1uWh2=0;}  //reset SolarPV1Wh counter when over  499.999MWh

	    		    	if (LoadmAh>=100000000000){LoadmAh=0;LoaduAh=0;}  //reset LoadAh counter when over 99.999MAh
	    		    	if (LoadWh>= 1000000000){LoadWh=0;LoaduWh=0;}  //reset LoadWh counter when over    99.999MWh

	    		    	if (ExtLoadmAh>=100000000000){ExtLoadmAh=0;ExtLoaduAh=0;}  //reset LoadAh counter when over 99.999MAh
	    		    	if (ExtLoadWh>= 1000000000){ExtLoadWh=0;ExtLoaduWh=0;}  //reset LoadWh counter when over    99.999MWh


                        if ((ROMdec[0]==4)||(ROMdec[0]==5)||(ROMdec[0]==6))
                        {
                            if ((BVmin[0]/10)>(ROMdec[6]))
                            		{
                        	SOC1=(((BVmin[0]/10)*(BVmin[0]/10))-(ROMdec[6]*ROMdec[6]))/(((ROMdec[11]*ROMdec[11])-(ROMdec[6]*ROMdec[6]))/100);
                            if (SOC1>=100) {SOC1=100;}
                            if (SOC1<=1) {SOC1=1;}
                            		}
                            if ((BVmin[0]/10)<=(ROMdec[6])) {SOC1=0;}
                        }
                        else
                        {
                        	SOC1=((BatterySOC-BattCapacity)*100)/BattCapacity;
                        }

	    }

zz2=0; //reset the 1 second timer for the energy counter.

}

//=====================================================================================

void compenergyflash (void)//

{
	comp(daycounter1,3,sbmsD,0);
	comp(PV1uWh2/100000,5,sbmsD,3);
	comp(PV2uWh2/100000,5,sbmsD,8);
	comp(BatteryuWh2/100000,5,sbmsD,13);
}

//=====================================================================================

void compdevsetflash (void)//2 minutes interval

{
	   comp(bk[0],1,sbmsD,0);
	   comp(bk[2],1,sbmsD,1);
	   comp(bk[3],1,sbmsD,2);
	   comp(menucolor,1,sbmsD,3);

	   comp(Serial[0],1,sbmsD,4);
	   comp(Serial[1],3,sbmsD,5);
	   comp(Serial[2],2,sbmsD,8);

	   comp(WiFis[0],1,sbmsD,10);
	   comp(WiFis[1],1,sbmsD,11);

	   comp(WiFis[2],2,sbmsD,12);

	   comp(WiFis[3],1,sbmsD,14);
	   comp(WiFis[4],3,sbmsD,15);
	   comp(WiFis[5],3,sbmsD,18);

	   comp(EXT[0],1,sbmsD,21);
	   comp(EXT[1],3,sbmsD,22);
	   comp(EXT[2],1,sbmsD,25);
	   comp(EXT[3],3,sbmsD,26);
	   comp(EXT[4],1,sbmsD,29);
	   comp(EXT[5],3,sbmsD,30);
	   comp(EXT[6],1,sbmsD,33);
	   comp(EXT[7],3,sbmsD,34);


	   comp(EXTADC1[0],1,sbmsD,37);
	   comp(EXTADC1[1],3,sbmsD,38);
	   comp(EXTADC1[2],2,sbmsD,41);
	   comp(EXTADC1[3],3,sbmsD,43);
	   comp(EXTADC1[4],2,sbmsD,46);
	   comp(EXTADC1[5],3,sbmsD,48);
	   comp(EXTADC1[6],2,sbmsD,51);
	   comp(EXTADC1[7],3,sbmsD,53);

}

//=====================================================================================

void compdataflash (void)//2 minutes interval

{
	uint8_t i;
	uint32_t Batttemp;
	//date and time
	sbmsD[0]=yearb+35;
	sbmsD[1]=monthb+35;
	sbmsD[2]=dayb+35;
	sbmsD[3]=hourb+35;
	sbmsD[4]=minb+35;
	sbmsD[5]=secb+35;

	//var sbms
	comp(SOC1,2,sbmsD,6);
	for (i=0;i<8;i++){comp(BV[i][5],2,sbmsD,8+(i*2));} //cell voltage
	if (ITEMP<=Therm[4]){comp(ITEMP2+450,2,sbmsD,24);}
	else {comp(ITEMP2-450,2,sbmsD,24);}
	if (ETEMP<=Therm[4]){comp(ETEMP2+450,2,sbmsD,26);}
	else {comp(ETEMP2-450,2,sbmsD,26);}
	Batttemp=(Battgraph1h[240]+Battgraph1h[239])/2;
	if (Batttemp>=12000000){Batttemp=Batttemp-12000000;sbmsD[28]=43;}
	else {Batttemp=12000000-Batttemp;sbmsD[28]=45;}
	comp(Batttemp,3,sbmsD,29);
	comp((PV1graph1h[240]+PV1graph1h[239])/2,3,sbmsD,32);
	comp((PV2graph1h[240]+PV2graph1h[239])/2,3,sbmsD,35);
	comp((ExtLoadgraph1h[240]+ExtLoadgraph1h[239])/2,3,sbmsD,38);
	comp(ADC2,3,sbmsD,41);
	comp(ADC3,3,sbmsD,44);
	comp(ADC4,3,sbmsD,47);
	comp(0,3,sbmsD,50); //Digital MPPT heat controller 1  will be added here in the future.
	comp(0,3,sbmsD,53); //Digital MPPT heat controller 2  will be added here in the future.
	comp(ERR,3,sbmsD,56);
    sbmsD[59]=32;
}

//=====================================================================================

void compdataUSART (uint8_t interval)//max 4 minutes interval

{
	uint8_t i;
	uint32_t Batttemp;
	//date and time
	sbmsD[0]=yearb+35;
	sbmsD[1]=monthb+35;
	sbmsD[2]=dayb+35;
	sbmsD[3]=hourb+35;
	sbmsD[4]=minb+35;
	sbmsD[5]=secb+35;

	//var sbms
	comp(SOC1,2,sbmsD,6);
	for (i=0;i<8;i++){comp(BV[i][5],2,sbmsD,8+(i*2));} //cell voltage
	if (ITEMP<=Therm[4]){comp(ITEMP2+450,2,sbmsD,24);}
	else {comp(ITEMP2-450,2,sbmsD,24);}
	if (ETEMP<=Therm[4]){comp(ETEMP2+450,2,sbmsD,26);}
	else {comp(ETEMP2-450,2,sbmsD,26);}

	graphvaltemp=0;
	for (i = 241-interval; i < 241; i++){graphvaltemp=graphvaltemp+Battgraph[i];}
	Batttemp=graphvaltemp/interval;

	if (Batttemp>=12000000){Batttemp=Batttemp-12000000;sbmsD[28]=43;}
	else {Batttemp=12000000-Batttemp;sbmsD[28]=45;}
	comp(Batttemp,3,sbmsD,29);

	graphvaltemp=0;
	for (i = 241-interval; i < 241; i++){graphvaltemp=graphvaltemp+PV1graph[i];}

	comp(graphvaltemp/interval,3,sbmsD,32);

	graphvaltemp=0;
	for (i = 241-interval; i < 241; i++){graphvaltemp=graphvaltemp+PV2graph[i];}

	comp(graphvaltemp/interval,3,sbmsD,35);

	graphvaltemp=0;
	for (i = 241-interval; i < 241; i++){graphvaltemp=graphvaltemp+ExtLoadgraph[i];}

	comp(graphvaltemp/interval,3,sbmsD,38);

	comp(ADC2,3,sbmsD,41);
	comp(ADC3,3,sbmsD,44);
	comp(ADC4,3,sbmsD,47);
	comp(0,3,sbmsD,50); //Digital MPPT heat controller 1  will be added here in the future.
	comp(0,3,sbmsD,53); //Digital MPPT heat controller 2  will be added here in the future.
	comp(ERR,3,sbmsD,56);
    sbmsD[59]=32;
}


//=====================================================================================
void compdata (void)

{

RTC_TimeTypeDef RTC_TimeStructure;
RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
RTC_DateTypeDef RTC_DateStructure;
RTC_GetDate(RTC_Format_BIN, &RTC_DateStructure);

uint16_t i;

secb=RTC_TimeStructure.RTC_Seconds;
minb=RTC_TimeStructure.RTC_Minutes;
hourb=RTC_TimeStructure.RTC_Hours;

dayb=RTC_DateStructure.RTC_Date;
monthb=RTC_DateStructure.RTC_Month;
yearb=RTC_DateStructure.RTC_Year;


for (i=0;i<1861;i++){webpage[i]=web03[i];}

//date and time
webpage[varsbms]=yearb+35;
webpage[varsbms+1]=monthb+35;
webpage[varsbms+2]=dayb+35;
webpage[varsbms+3]=hourb+35;
webpage[varsbms+4]=minb+35;
webpage[varsbms+5]=secb+35;

//var sbms

comp(SOC1,2,webpage,varsbms+6);
for (i=0;i<8;i++){comp(BV[i][5],2,webpage,(varsbms+8)+(i*2));} //cell voltage
if (ITEMP<=Therm[4]){comp(ITEMP2+450,2,webpage,varsbms+24);}
else {comp(ITEMP2-450,2,webpage,varsbms+24);}
if (ETEMP<=Therm[4]){comp(ETEMP2+450,2,webpage,varsbms+26);}
else {comp(ETEMP2-450,2,webpage,varsbms+26);}
if (direction22==0){webpage[varsbms+28]=45;}else{webpage[varsbms+28]=43;}
comp(BatteryCurrent3,3,webpage,varsbms+29);
comp(PV1,3,webpage,varsbms+32);
comp(PV2,3,webpage,varsbms+35);
comp(ExtLoadCurrent,3,webpage,varsbms+38);
comp(ADC2,3,webpage,varsbms+41);
comp(ADC3,3,webpage,varsbms+44);
comp(ADC4,3,webpage,varsbms+47);
comp(0,3,webpage,varsbms+50); //Digital MPPT heat controller 1  will be added here in the future.
comp(0,3,webpage,varsbms+53); //Digital MPPT heat controller 2  will be added here in the future.
comp(ERR,3,webpage,varsbms+56);

//var xsbms

comp(LoadCurrent,3,webpage,varxsbms);
comp(OVlock,2,webpage,varxsbms+3);
comp(UVlock,2,webpage,varxsbms+5);
comp(ROMdec[0],1,webpage,varxsbms+7);
comp(ROMdec[2],3,webpage,varxsbms+8);

//var s2

for (i=0;i<8;i++){webpage[vars2+(i*2)]=CB[i]+48;}//cell balancing on or off
webpage[vars2+16]=BVmax[1]+49;//cell ID max
webpage[vars2+18]=BVmin[1]+49;//cell ID min
webpage[vars2+20]=EF[12]+48;//PV on off
webpage[vars2+22]=EF[14]+48;//Load on off

//var s1

if (ROMdec[0]<4){webpage[vars1+2]=65;webpage[vars1+3]=104;}//Ah
else{webpage[vars1+2]=70;webpage[vars1+3]=32;}//0F

if (power1==0){webpage[vars1+7]=65;}
else{webpage[vars1+7]=87;}

#if SBMS100>0
webpage[vars1+15]=49;//for SBMS100
webpage[vars1+17]=48;
#endif

//var eA
comp(BatterymAh,6,webpage,vareA);// energy current
comp(PV1mAh,6,webpage,vareA+6);
comp(PV2mAh,6,webpage,vareA+12);
comp(PV1mAh+PV2mAh,6,webpage,vareA+24);
comp(LoadmAh,6,webpage,vareA+30);
comp(ExtLoadmAh,6,webpage,vareA+36);

//var eW
comp(BatteryWh,6,webpage,vareW); //energy power
comp(PV1Wh,6,webpage,vareW+6);
comp(PV2Wh,6,webpage,vareW+12);
comp(PV1Wh+PV2Wh,6,webpage,vareW+24);
comp(LoadWh,6,webpage,vareW+30);
comp(ExtLoadWh,6,webpage,vareW+36);


webgraph(PV1graph,PV2graph,9,60,180,180,6,1*power1);//1m PV
webgraph(PV1graph1h,PV2graph1h,9,60,180,120,3,2*power1);//1h PV
webgraph(PV1graph24h,PV2graph24h,9,120,120,0,0,3*power1);//12h PV

for (i=0;i<60;i++)
{
if (Battgraph[i+180]>12000000){Tempgraph1[i+180]=Battgraph[i+180]-12000000; Tempgraph2[i+180]=0;}
else {Tempgraph1[i+180]=0; Tempgraph2[i+180]=12000000-Battgraph[i+180];}
}
webgraph(Tempgraph1,Tempgraph2,513,60,180,180,15,1*power1);//1m Batt

for (i=0;i<60;i++)
{
if (Battgraph1h[i+180]>12000000){Tempgraph1[i+180]=Battgraph1h[i+180]-12000000; Tempgraph2[i+180]=0;}
else {Tempgraph1[i+180]=0; Tempgraph2[i+180]=12000000-Battgraph1h[i+180];}
}
webgraph(Tempgraph1,Tempgraph2,513,60,180,120,12,2*power1);//1h Batt

for (i=0;i<120;i++)
{
if (Battgraph24h[i+120]>12000000){Tempgraph1[i+120]=Battgraph24h[i+120]-12000000; Tempgraph2[i+120]=0;}
else {Tempgraph1[i+120]=0; Tempgraph2[i+120]=12000000-Battgraph24h[i+120];}
}
webgraph(Tempgraph1,Tempgraph2,513,120,120,0,9,3*power1);//12h Batt


webgraph(Loadgraph,ExtLoadgraph,1017,60,180,180,24,1*power1);//1m PV
webgraph(Loadgraph1h,ExtLoadgraph1h,1017,60,180,120,21,2*power1);//1h PV
webgraph(Loadgraph24h,ExtLoadgraph24h,1017,120,120,0,18,3*power1);//12h PV


//Puts(90,190, webpage, &TM_Font_7x10, bmscolor[menucolor][60], 0x0000);
//WriteDec(170, 190, &TM_Font_7x10, 0xff00, 0x0000, 5,0,ERR,0);
}

//=====================================================================================

void webgraph(uint32_t *val1,uint32_t *val2,uint16_t pos,uint16_t nr,uint16_t nr2,uint16_t nr3,uint16_t smax,uint8_t power)
{
	uint8_t i;
	float max,maxL;
	maxL=0;max=0;
	uint32_t val11[241];
	uint32_t val22[241];

    for (i=0;i<241;i++){val11[i]=0; val22[i]=0;}


	if (power==0) {for (i=0;i<nr;i++){val11[i+nr2]=val1[i+nr2];val22[i+nr2]=val2[i+nr2];}}

	if (power==1) {for (i=0;i<nr;i++){val11[i+nr2]=(val1[i+nr2]*VBattgraph[i+nr2])/100000;   val22[i+nr2]=(val2[i+nr2]*VBattgraph[i+nr2])/100000;} }

	if (power==2) {for (i=0;i<nr;i++){val11[i+nr2]=(val1[i+nr2]*VBattgraph1h[i+nr2])/100000; val22[i+nr2]=(val2[i+nr2]*VBattgraph1h[i+nr2])/100000;} }

	if (power==3) {for (i=0;i<nr;i++){val11[i+nr2]=(val1[i+nr2]*VBattgraph24h[i+nr2])/100000;val22[i+nr2]=(val2[i+nr2]*VBattgraph24h[i+nr2])/100000;} }




	for (i=0;i<nr;i++){if(maxL<(val11[i+nr2]+val22[i+nr2])){maxL=val11[i+nr2]+val22[i+nr2];}}
	if (maxL>=90){max=maxL/90;}else{max=1;}
	for (i=0;i<nr;i++){webpage[pos+nr3+i]=(val11[i+nr2]/max)+35; webpage[pos+252+nr3+i]=(val22[i+nr2]/max)+35;}



	comp(maxL,3,webpage,vargsbms+smax);
}

//=====================================================================================

void comp(uint32_t decval,uint8_t nr,uint8_t* dataout,uint16_t pos)
{
uint8_t i;
uint32_t qw[8],qe[8];

for (i=0;i<nr;i++){qw[i]=qe[i]=0;}
qe[0] = decval;
for (i=0;i<nr;i++){	if(qe[i]>90){qe[i+1] =(qe[i]/91);qw[i] =(qe[i]-(qe[i+1]*91));}else {qw[i]=qe[i];}}
for (i=0;i<nr;i++){dataout[pos+i]=qw[nr-(i+1)]+35;}

}

//function dcmp(p,s,d){xx=0; for (z=0;z<s;z++){xx = xx + ((d.charCodeAt((p+s-1)-z)-35)*Math.pow(91,z));}return xx;}

//=====================================================================================

uint32_t dcomp(uint8_t nr,uint8_t* datain,uint16_t pos)
{
uint8_t i;
uint32_t decval;

decval=0;
for (i=0;i<nr;i++)
{
decval=decval+((datain[(pos+nr-1)-i]-35)*pow(91,i));
}
return decval;
}



//=================================================================================================
//          *************************         Monitoring         **********************
//=================================================================================================

void monitoring(void)
	{


	if (c2!=1){return;}


	buttondir=1;

	unsigned int k,ky,ki,s,it1,level[8];

    s=4;

    if (b3==1||b4==1)
    {
    Fill_menu(0x0000);
    }
    if (b3>25||b4>25){b3=5;b4=5;}


    monitor();

    if (c3<1){c3=s;}
    if (c3>=(s+1)){c3=1;}


    LCDmenu_set_XY(0,0);
    LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
    LCDmenu_set_XY(1,0);
    LCDmenu_write_string_neg("/",menucolor);
    LCDmenu_set_XY(2,0);
    LCDmenu_write_Dec1_neg(1,0,s,menucolor);
    LCDmenu_set_XY(3,0);
    LCDmenu_write_string_neg("       Monitor            ",menucolor);

    if (BOK==0){release1=1;}
    if (BBack==0){release2=1;}

if (c3==1)  //first monitor page

{

	for (k=0;k<8;k++){
			if (BV[k][5]>UVlock)
			{
			level[k]=(166-((BV[k][5]-UVlock)/((OVlock-UVlock)/136)));
			}
			else
			{
			level[k]=166;
			}
	        if (level[k]<=20)
	        {
	        level[k]=20;
	        }
	        if (level[k]>=166)
	        {
	        level[k]=166;
	        }
	}



	for (k=0;k<8;k++){

		if (BV[k][5]!=0)
			    {
	for (it1=0;it1<7;it1++){
	DrawVline(20,level[k]+1,257+(k*8)+it1,bmscolor[menucolor][20-(it1*2)]);
	DrawVline(level[k]+1,168,257+(k*8)+it1,bmscolor[menucolor][60-(it1*5)]);
	}
	WriteDec(257+(k*8), 170, &TM_Font_7x10, bmscolor[menucolor][58], bmscolor[menucolor][18], 1,0,k+1,0);
			    }

	}



for (k=0;k<8;k++)

{
	if (BV[k][5]!=0)
	    {

    if (BVmin[1]==k)
    {

    WriteDec(165, (k*20)+20, &TM_Font_11x18, bmscolor[menucolor][0], bmscolor[menucolor][62],1,0,k+1,0);
    WriteDec(182, (k*20)+20, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],4,3,BV[k][5],0);
    DrawRectangle(164,(k*20)+19, 251,(k*20)+38,bmscolor[menucolor][62]);
    }
    else if (BVmax[1]==k)
    {
    WriteDec(165, (k*20)+20, &TM_Font_11x18, bmscolor[menucolor][5], bmscolor[menucolor][63],1,0,k+1,0);
    WriteDec(182, (k*20)+20, &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5],4,3,BV[k][5],0);
    DrawRectangle(164,(k*20)+19, 251,(k*20)+38,bmscolor[menucolor][63]);
    }
    else
    {

    WriteDec(165, (k*20)+20, &TM_Font_11x18, bmscolor[menucolor][0], bmscolor[menucolor][27],1,0,k+1,0);
    WriteDec(182, (k*20)+20, &TM_Font_11x18, bmscolor[menucolor][40], bmscolor[menucolor][5],4,3,BV[k][5],0);
    DrawRectangle(164,(k*20)+19, 251,(k*20)+38,bmscolor[menucolor][0]);
    }




    if (CB[k]==1)
    {
    Puts(240, (k*20)+20,"<", &TM_Font_11x18, bmscolor[menucolor][0], bmscolor[menucolor][60]);
    }
    else
    {
    Puts(240, (k*20)+20,"v", &TM_Font_11x18, bmscolor[menucolor][20], bmscolor[menucolor][4]);
    }

    }

}

Puts(1,   25, "Battery ", &TM_Font_7x10, bmscolor[menucolor][27], 0x0000);
Puts(137, 20, "V ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);

Puts(1,   45, "Cell ^  ", &TM_Font_7x10, bmscolor[menucolor][27], 0x0000);
Puts(137, 40, "mV", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);

Puts(1, 65,  "Batt", &TM_Font_7x10, bmscolor[menucolor][27], 0x0000);
Puts(1, 85,  "PV2     ", &TM_Font_7x10, bmscolor[menucolor][27], 0x0000);
Puts(1, 105, "PV1     ", &TM_Font_7x10, bmscolor[menucolor][27], 0x0000);
Puts(1, 125, "PV1+PV2 ", &TM_Font_7x10, bmscolor[menucolor][27], 0x0000);
Puts(1, 145, "Load+ExtL", &TM_Font_7x10, bmscolor[menucolor][27], 0x0000);

if (power1==0)
{
//DrawHline(66,63,64,bmscolor[menucolor][5]);
//Puts(66, 65,  " Current ", &TM_Font_7x10, bmscolor[menucolor][27], bmscolor[menucolor][5]);

Puts(137, 60,  "A ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 80,  "A ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 100, "A ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 120, "A ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 140, "A ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
}
else
{
//DrawHline(66,63,64,bmscolor[menucolor][5]);
//Puts(66, 65,  "  Power  ", &TM_Font_7x10, bmscolor[menucolor][27], bmscolor[menucolor][5]);

Puts(137, 60,  "W ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 80,  "W ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 100, "W ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 120, "W ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
Puts(137, 140, "W ", &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);
}

	if (BatteryVoltage>=10000){WriteDec(66, 20, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],5,3,BatteryVoltage,1);}
	else                      {WriteDec(66, 20, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],4,3,BatteryVoltage,1);}



	if (BVdelta>=1000)                   {WriteDec(66, 40, &TM_Font_11x18, bmscolor[menucolor][40], bmscolor[menucolor][5],4,0,BVdelta,1);}
	if ((BVdelta>=100)&&(BVdelta<1000))  {WriteDec(66, 40, &TM_Font_11x18, bmscolor[menucolor][40], bmscolor[menucolor][5],3,0,BVdelta,1);}
	if ((BVdelta>=10)&&(BVdelta<100))    {WriteDec(66, 40, &TM_Font_11x18, bmscolor[menucolor][40], bmscolor[menucolor][5],2,0,BVdelta,1);}
	if (BVdelta<10)                      {WriteDec(66, 40, &TM_Font_11x18, bmscolor[menucolor][40], bmscolor[menucolor][5],1,0,BVdelta,1);}


	if (power1==0)
	{

    if      (BatteryCurrent3>100000){WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5],5,2,BatteryCurrent3/10,1);}
    else if (BatteryCurrent3>10000) {WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],5,3,BatteryCurrent3,1);}
 	else                            {WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],4,3,BatteryCurrent3,1);}
                                        Puts(55, 60, direction2, &TM_Font_11x18, bmscolor[menucolor][55], 0x0000);
                                        Puts(43, 60, direction, &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);



   	if (PV2>100000)       {WriteDec(66, 80, &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5],5,2,PV2/10,1);}
   	else if (PV2>10000)   {WriteDec(66, 80, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],5,3,PV2,1);}
   	else                  {WriteDec(66, 80, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],4,3,PV2,1);}



    if (PV1>100000)       {WriteDec(66, 100, &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5],5,2,PV1/10,1);}
   	else if (PV1>10000)   {WriteDec(66, 100, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],5,3,PV1,1);}
   	else  	              {WriteDec(66, 100, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],4,3,PV1,1);}

   	if (PV12>100000)      {WriteDec(66, 120, &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5],5,2,PV12/10,1);}
   	else if (PV12>10000)  {WriteDec(66, 120, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],5,3,PV12,1);}
   	else                  {WriteDec(66, 120, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],4,3,PV12,1);}

   	if ((LoadCurrent+ExtLoadCurrent)>100000)     {WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5],5,2,(LoadCurrent+ExtLoadCurrent)/10,1);}
   	else if ((LoadCurrent+ExtLoadCurrent)>10000) {WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],5,3,(LoadCurrent+ExtLoadCurrent),1);}
   	else                                         {WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5],4,3,(LoadCurrent+ExtLoadCurrent),1);}
	}

else  // for power it needs changes
	{
	    if  ((BatteryW)<100)                          {WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 2,1,BatteryW,1);}
	    if (((BatteryW)>=100)&((BatteryW)<1000))      {WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 3,1,BatteryW,1);}
	    if (((BatteryW)>=1000)&((BatteryW)<10000))    {WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 4,1,BatteryW,1);}
	    if (((BatteryW)>=10000)&((BatteryW)<100000))  {WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 5,1,BatteryW,1);}
	    if (((BatteryW)>=100000)&((BatteryW)<1000000)){WriteDec(66, 60, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 5,0,BatteryW/10,1);}
	                                                     Puts(55, 60, direction2, &TM_Font_11x18, bmscolor[menucolor][55], 0x0000);
	                                                     Puts(43, 60, direction, &TM_Font_11x18, bmscolor[menucolor][27], 0x0000);


	    if  ((PV2W)<100)                    {WriteDec(66, 80, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 2,1,PV2W,1);}
	    if (((PV2W)>=100)&((PV2W)<1000))    {WriteDec(66, 80, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 3,1,PV2W,1);}
	    if (((PV2W)>=1000)&((PV2W)<10000))  {WriteDec(66, 80, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 4,1,PV2W,1);}
	    if (((PV2W)>=10000)&((PV2W)<100000)){WriteDec(66, 80, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 5,1,PV2W,1);}


	    if  ((PV1W)<100)                    {WriteDec(66, 100, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 2,1,PV1W,1);}
	    if (((PV1W)>=100)&((PV1W)<1000))    {WriteDec(66, 100, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 3,1,PV1W,1);}
	    if (((PV1W)>=1000)&((PV1W)<10000))  {WriteDec(66, 100, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 4,1,PV1W,1);}
	    if (((PV1W)>=10000)&((PV1W)<100000)){WriteDec(66, 100, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 5,1,PV1W,1);}


	    if  ((PV12W)<100)                     {WriteDec(66, 120, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 2,1,PV12W,1);}
	    if (((PV12W)>=100)&((PV12W)<1000))    {WriteDec(66, 120, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 3,1,PV12W,1);}
	    if (((PV12W)>=1000)&((PV12W)<10000))  {WriteDec(66, 120, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 4,1,PV12W,1);}
	    if (((PV12W)>=10000)&((PV12W)<100000)){WriteDec(66, 120, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 5,1,PV12W,1);}


	    if  ((LoadExtLoadW)<100)                              {WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 2,1,LoadExtLoadW,1);}
	    if (((LoadExtLoadW)>=100)&((LoadExtLoadW)<1000))      {WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 3,1,LoadExtLoadW,1);}
	    if (((LoadExtLoadW)>=1000)&((LoadExtLoadW)<10000))    {WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 4,1,LoadExtLoadW,1);}
	    if (((LoadExtLoadW)>=10000)&((LoadExtLoadW)<100000))  {WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 5,1,LoadExtLoadW,1);}
	    if (((LoadExtLoadW)>=100000)&((LoadExtLoadW)<1000000)){WriteDec(66, 140, &TM_Font_11x18, bmscolor[menucolor][60], bmscolor[menucolor][5], 5,0,LoadExtLoadW/10,1);}
	}

	if (ITEMP<=Therm[4]) {Puts(0, 160,  "IT+", &TM_Font_11x18, bmscolor[menucolor][32], bmscolor[menucolor][12]);}
	else             {Puts(0, 160,  "IT-", &TM_Font_11x18, bmscolor[menucolor][32], bmscolor[menucolor][12]);}
	if (ETEMP<=Therm[4]) {Puts(88, 160,  "ET+", &TM_Font_11x18, bmscolor[menucolor][32], bmscolor[menucolor][12]);}
	else             {Puts(88, 160,  "ET-", &TM_Font_11x18, bmscolor[menucolor][32], bmscolor[menucolor][12]);}
	WriteDec(33,  160, &TM_Font_11x18, bmscolor[menucolor][47], bmscolor[menucolor][12], 2,0,ITEMP2/10,0);
	WriteDec(121, 160, &TM_Font_11x18, bmscolor[menucolor][47], bmscolor[menucolor][12], 2,0,ETEMP2/10,0);
	Puts(143, 160,  "C", &TM_Font_11x18, bmscolor[menucolor][32], bmscolor[menucolor][12]);
	Puts(55, 160,  "C  ", &TM_Font_11x18, bmscolor[menucolor][32], bmscolor[menucolor][12]);
}

if (c3==2)  // page 5 Battery power and energy
{

	if (zz7==1){

	Puts(0, 20,   "                             ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);
	Puts(0, 160,  "                             ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);


	for (k=1;k<7;k++)
	{
	if (k==1){Puts(0, 40, "PV2     ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);energytempA=PV2mAh;energytempW=PV2Wh;}
	if (k==2){Puts(0, 60, "PV1     ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);energytempA=PV1mAh;energytempW=PV1Wh;}
	if (k==3){Puts(0, 80, "PV1+PV2 ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);energytempA=PV1mAh+PV2mAh;energytempW=PV1Wh+PV2Wh;}
	if (k==4){Puts(0, 100,"Battery ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);energytempA=BatterymAh;energytempW=BatteryWh;}
	if (k==5){Puts(0, 120,"Load    ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);energytempA=LoadmAh;energytempW=LoadWh;}
	if (k==6){Puts(0, 140,"ExtLoad ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);energytempA=ExtLoadmAh;energytempW=ExtLoadWh;}

	if  ((energytempA)< 10000)                                    {WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 4,3,energytempA,1);        Puts(167, 20+(k*20), "Ah ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
    if (((energytempA)>=10000)&      ((energytempA)<100000))      {WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,3,energytempA,1);        Puts(167, 20+(k*20), "Ah ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempA)>=100000)&     ((energytempA)<1000000))     {WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,2,energytempA/10,1);     Puts(167, 20+(k*20), "Ah ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempA)>=1000000)&    ((energytempA)<10000000))    {WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,1,energytempA/100,1);    Puts(167, 20+(k*20), "Ah ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempA)>=10000000)&   ((energytempA)<100000000))   {WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,3,energytempA/1000,1);   Puts(167, 20+(k*20), "kAh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempA)>=100000000)&  ((energytempA)<1000000000))  {WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,2,energytempA/10000,1);  Puts(167, 20+(k*20), "kAh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempA)>=1000000000)& ((energytempA)<10000000000)) {WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,1,energytempA/100000,1); Puts(167, 20+(k*20), "kAh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempA)>=10000000000)&((energytempA)<100000000000)){WriteDec(100, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,3,energytempA/1000000,1);Puts(167, 20+(k*20), "MAh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}

	if  ((energytempW)< 100)                                        {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 2,1,energytempW,1);       Puts(277, 20+(k*20), "Wh ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempW)>=100)&      ((energytempW)<1000))            {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 3,1,energytempW,1);       Puts(277, 20+(k*20), "Wh ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempW)>=1000)&     ((energytempW)<10000))           {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 4,1,energytempW,1);       Puts(277, 20+(k*20), "Wh ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempW)>=10000)&    ((energytempW)<100000))          {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,1,energytempW,1);       Puts(277, 20+(k*20), "Wh ", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempW)>=100000)&   ((energytempW)<1000000))         {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,3,energytempW/10,1);    Puts(277, 20+(k*20), "kWh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempW)>=1000000)&  ((energytempW)<10000000))        {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,2,energytempW/100,1);   Puts(277, 20+(k*20), "kWh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempW)>=10000000)& ((energytempW)<100000000))       {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,1,energytempW/1000,1);  Puts(277, 20+(k*20), "kWh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	if (((energytempW)>=100000000)&((energytempW)<1000000000))      {WriteDec(210, 20+(k*20), &TM_Font_11x18, bmscolor[menucolor][63], bmscolor[menucolor][5], 5,3,energytempW/10000,1); Puts(277, 20+(k*20), "MWh", &TM_Font_11x18, bmscolor[menucolor][62], bmscolor[menucolor][5]);}
	}
	}
zz7=0;

}

if (c3==3)
{
       ki=0;
        for (k=0;k<5;k++)
        {
        for (ky=0;ky<21;ky=ky+10)
        {
            LCDmenu_set_XY(ky,k+1);
	        if (EF[ki]==0)
	        {
	   	    //LCDmenu_write_string(EFT[ki],menucolor);
	   	    Puts((ky*11)+5, (k*32)+20, EFT[ki][0], &TM_Font_11x18, bmscolor[menucolor][36], bmscolor[menucolor][10]);
	   	    Puts((ky*11)+5, (k*32)+39, EFT[ki][1], &TM_Font_7x10, bmscolor[menucolor][25], bmscolor[menucolor][0]);
	        }
	   	    if (EF[ki]==1)
	   	    {
	   	    //LCDmenu_write_string_neg(EFT[ki],menucolor);
	   	    Puts((ky*11)+5, (k*32)+20, EFT[ki][0], &TM_Font_11x18, bmscolor[menucolor][3], bmscolor[menucolor][60]);
	   	    Puts((ky*11)+5, (k*32)+39, EFT[ki][1], &TM_Font_7x10, bmscolor[menucolor][50], bmscolor[menucolor][0]);
	   	    }
            ki++;
        }
        }

}


if (c3==4)  // page 4  graph
{

 if (zz5==1)
 {

Puts(0, 50,   "PV1 + PV2", &TM_Font_7x10, bmscolor[menucolor][62], 0x0000);
Puts(0, 100,  "Battery  ", &TM_Font_7x10, bmscolor[menucolor][62], 0x0000);
Puts(0, 150,  "Load     ", &TM_Font_7x10, bmscolor[menucolor][62], 0x0000);


EnergyGraph(74,136-(2*53),PV1color,PV2color,menucolor,PV1graph24h,1,PV2graph24h,1,VBattgraph24h,power1,"12h",240,120,40,0);

EnergyGraph(74,136-(1*53),Battpcolor,Battncolor,menucolor,Battgraph24h,1,0,0,VBattgraph24h,power1,"12h",240,120,40,0);

EnergyGraph(74,136-(0*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph24h,1,ExtLoadgraph24h,1,VBattgraph24h,power1,"12h",240,120,40,0);

if (power1!=0){DrawHline(0,70,20,bmscolor[menucolor][5]); Puts(0, 20,  " Power [W]", &TM_Font_7x10, bmscolor[menucolor][27], bmscolor[menucolor][5]);}
else          {DrawHline(0,70,20,bmscolor[menucolor][5]); Puts(0, 20,  "Current[A]", &TM_Font_7x10, bmscolor[menucolor][27], bmscolor[menucolor][5]);}

EnergyGraph(196,136-(2*53),PV1color,PV2color,menucolor,PV1graph1h,1,PV2graph1h,1,VBattgraph1h,power1,"1h",240,60,40,0);

EnergyGraph(196,136-(1*53),Battpcolor,Battncolor,menucolor,Battgraph1h,1,0,0,VBattgraph1h,power1,"1h",240,60,40,0);

EnergyGraph(196,136-(0*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph1h,1,ExtLoadgraph1h,1,VBattgraph1h,power1,"1h",240,60,40,0);


EnergyGraph(258,136-(2*53),PV1color,PV2color,menucolor,PV1graph,1,PV2graph,1,VBattgraph,power1,"1m",240,60,40,0);

EnergyGraph(258,136-(1*53),Battpcolor,Battncolor,menucolor,Battgraph,1,0,0,VBattgraph,power1,"1m",240,60,40,0);

EnergyGraph(258,136-(0*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph,1,ExtLoadgraph,1,VBattgraph,power1,"1m",240,60,40,0);

}

 zz5=0;

}

    if ((BBack==1)&&(release2==1)&&(lockkey!=1))
        {
    	Fill_menu(0x0000);//new
        c3=1;
        key1=1;
        c2=100;
        BBack=0;
        buttondir=0;
        }

    if ((BOK==1)&&(release1==1)&&(lockkey!=1)&&(first1==0))
            {
            c333=c3;
    	    c3=1;
            key1=1;
            if(c333==1){c2=17;} //{c2=17;} dev test
            if(c333==2){c2=11;}
            if(c333==3){c2=18;}
            if(c333==4){c2=12;}
            BOK=0;
            buttondir=0;
            }

   if (lockkey!=1) {update1=1;}
   if (zz4==6) {update1=1;}
   if ((first1>=1)&&(BOK==0)) {first1++;}
   if (first1>3) {first1=0;}
	}

void counterR(void)
 {


	if (c2!=11){return;}

	unsigned int v2,s,i2,i3,f1;
	    s=8;
	    f1=0;

	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}

		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg(" Reset ",menucolor);

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f1,i3+1);
					    	LCDmenu_write_string(counterreset[i3][0],menucolor);

					    }

					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(counterreset[c3-1][0],menucolor);

					    for(i2=0;i2<15;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=counterreset[c3-1][1];
					    scrolllenght=7;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

	if (BOK==1)
	{
	 c2=1;
	 //BOK=0;
	 release1=0;
	 key1=1;

      if (c3==1){}
      if (c3==2){PV2mAh=0;PV2uAh=0;PV2Wh=0;PV2uWh=0;}
      if (c3==3){PV1mAh=0;PV1uAh=0;PV1Wh=0;PV1uWh=0;}
      if (c3==3){PV1mAh=0;PV1uAh=0;PV1Wh=0;PV1uWh=0;PV2mAh=0;PV2uAh=0;PV2Wh=0;PV2uWh=0;}
      if (c3==5){BatterymAh=0;BatteryuAh=0;BatteryWh=0;BatteryuWh=0;}
      if (c3==6){LoadmAh=0;LoaduAh=0;LoadWh=0;LoaduWh=0;}
      if (c3==7){ExtLoadmAh=0;ExtLoaduAh=0;ExtLoadWh=0;ExtLoaduWh=0;}
      if (c3==8){PV1mAh=0;PV1uAh=0;PV1Wh=0;PV1uWh=0;PV2mAh=0;PV2uAh=0;PV2Wh=0;PV2uWh=0;BatterymAh=0;BatteryuAh=0;BatteryWh=0;BatteryuWh=0;LoadmAh=0;LoaduAh=0;LoadWh=0;LoaduWh=0;ExtLoadmAh=0;ExtLoaduAh=0;ExtLoadWh=0;ExtLoaduWh=0;}

	 c3=c333;
	 i5=0;
		}

		if (BMENU==1)
			{
			c2=3;
	        BMENU=0;
	        BOK=0;
	 	    c3=1;
	 	    key1=1;
			}

		if (BBack==1)
			{
			c3=c333;
			release2=0;
			key1=1;
		    c2=1;
		    i5=0;
			}

 }

//=================================================================================================
void graphzoom(void)
 {

	if (c2!=12){return;}

	unsigned int v2,s,i2,i3,f1;
	    s=5;
	    f1=0;



	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg(" ZooM  ",menucolor);

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f1,i3+1);
					    	LCDmenu_write_string(graphzoom1[i3][0],menucolor);

					    }


					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(graphzoom1[c3-1][0],menucolor);



					    for(i2=0;i2<15;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=graphzoom1[c3-1][1];
					    scrolllenght=7;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

	if (BOK==1)
	{
	 c2=1;
	 //BOK=0;
	 release1=0;
	 key1=1;

	 if (c3==1){power1=0;Fill_sub_menu(0x0000);}
	 if (c3==2){power1=1;Fill_sub_menu(0x0000);}
	 if (c3==3){Fill_menu(0x0000);c2=13;}
	 if (c3==4){Fill_menu(0x0000);c2=14;}
	 if (c3==5){Fill_menu(0x0000);c2=15;}

     zz5=1;
	 c3=c333;
	 i5=0;

		}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

		if (BBack==1)
				{
			c3=c333;
			release2=0;
			key1=1;
		    c2=1;
		    //BBack=0;
		    i5=0;
		    Fill_sub_menu(0x0000);
            zz5=1;
			    }

 }

//=================================================================================================
void unitCP(void)
 {

	if (c2!=17){return;}

	unsigned int v2,s,i2,i3,f1;
	    s=2;
	    f1=0;



	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg(" Unit  ",menucolor);

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {
			            v2=c3-1;
			            if (v2<s)
			            {
					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f1,i3+1);
					    	LCDmenu_write_string(graphzoom1[i3][0],menucolor);
					    }

					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(graphzoom1[c3-1][0],menucolor);



					    for(i2=0;i2<15;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=graphzoom1[c3-1][1];
					    scrolllenght=7;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

	if (BOK==1)
	{
	 c2=1;
	 //BOK=0;
	 release1=0;
	 key1=1;

	 if (c3==1){power1=0;Fill_sub_menu(0x0000);}
	 if (c3==2){power1=1;Fill_sub_menu(0x0000);}

     zz5=1;
	 c3=c333;
	 i5=0;

		}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

		if (BBack==1)
				{
			c3=c333;
			release2=0;
			key1=1;
		    c2=1;
		    //BBack=0;
		    i5=0;
		    Fill_sub_menu(0x0000);
            zz5=1;
			    }

 }


//=================================================================================================
void loadonoffm(void)
 {

	if (c2!=18){return;}

	unsigned int v2,s,i2,i3,f1;
	    s=2;
	    f1=0;

	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}

		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg(" Load  ",menucolor);

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f1,i3+1);
					    	LCDmenu_write_string(loadonoffm1[i3][0],menucolor);
					    }
					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(loadonoffm1[c3-1][0],menucolor);

					    for(i2=0;i2<15;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=loadonoffm1[c3-1][1];
					    scrolllenght=7;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

	if (BOK==1)
	{
	 c2=1;
	 //BOK=0;
	 release1=0;
	 key1=1;

	 if (c3==1){
		 RAMreadall();
		 tempH=((RAMrx[6]) & 0x01);
		 if(tempH==0){ //==0
		 tempL=(((RAMrx[7])&0xE7)+(16+8));
		 RAMwrite(0x87,tempL); //uC manual load monitor control and charge monitor control
		 tempL=(((RAMrx[6])&0x0F)+(128+32));
		 RAMwrite(0x86,tempL); //uC manual load monitor control and charge control

		 tempL=(((RAMrx[7])&0xBF));
		 RAMwrite(0x87,tempL);        //FET auto enable
		 LCDmenu_set_XY(18,0);
		 LCDmenu_write_string("Load&PV ON ",menucolor);
		 delay_ms(300);
		 Fill_sub_menu(0x0000);

		 }
	 }



	 if (c3==2){
		    RAMreadall();
		 	tempH=((RAMrx[6]) & 0x01);
		     if(tempH!=0){
		    	tempL=(((RAMrx[7])&0xBF)+64);
		     	RAMwrite(0x87,tempL); //uC manual control
		     	tempL= 0xff;//fc
		     	RAMwrite(0x84,tempL);//cell balance on for test


		     	LCDmenu_set_XY(18,0);
		     	LCDmenu_write_string("Load&PV OFF",menucolor);
		     	tempL=(RAMrx[6] & 0xFC);//fc
		     	RAMwrite(0x86,tempL);//load and PV is OFF
		     	// need to do also charge OFF
		        delay_ms(300);
	            Fill_sub_menu(0x0000);
		     }
	 }
     zz5=1;
	 c3=c333;
	 i5=0;

	}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

		if (BBack==1)
				{
			c3=c333;
			release2=0;
			key1=1;
		    c2=1;
		    //BBack=0;
		    i5=0;
		    Fill_sub_menu(0x0000);
            zz5=1;
			    }

 }

//=================================================================================================
void graphzoomh(void)
 {

	if (c2!=16){return;}

	unsigned int v2,s,i2,i3,f1;
	    s=5;
	    f1=0;



	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg(" ZooMh ",menucolor);
		//LCDmenu_set_XY(0,6);
		//LCDmenu_write_string("_______",menucolor);



		if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f1,i3+1);
					    	LCDmenu_write_string(graphzoom1h[i3][0],menucolor);

					    }


					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(graphzoom1h[c3-1][0],menucolor);



					    for(i2=0;i2<15;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=graphzoom1h[c3-1][1];
					    scrolllenght=7;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }




			            }

			            }

	if (BOK==1)
	{
	 c2=c22;
	 //BOK=0;
	 release1=0;
	 key1=1;

	 if (c3==1){power1=0;Fill_sub_menu(0x0000);}
	 if (c3==2){power1=1;Fill_sub_menu(0x0000);}
	 if (c3==3){graphhorizontal=0;Fill_menu(0x0000);}
	 if (c3==4){graphhorizontal=1;Fill_menu(0x0000);}
	 if (c3==5){graphhorizontal=2;Fill_menu(0x0000);}


     zz5=1;
	 c3=c333;
	 i5=0;

		}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

		if (BBack==1)
				{
			c3=c333;
			release2=0;
			key1=1;
		    c2=c22;
		    //BBack=0;
		    i5=0;
		    Fill_sub_menu(0x0000);
            zz5=1;
			    }

 }

//=================================================================================================
void largegraph(void)
 {

	if ((c2==13)||(c2==14)||(c2==15)){}
	else {return;}

	unsigned int s,cursor5;
			    s=240;
	    buttondir=1;
	    cursor5=0;
	    if (BOK==0){release1=1;}
	    if (BBack==0){release2=1;}

			    if (c3<1){c3=1;}
				if (c3>s){c3=s;}

				if ((b3>2)&(b3<20)){b3=30;}
				if ((b3>37)&(b3<80)){b3=97;}
				if ((b4>2)&(b4<20)){b4=30;}
				if ((b4>37)&(b4<80)){b4=97;}

				if ((b3>80)&(c3<6)){b3=31;}
				if ((b4>80)&(c3<6)){b4=31;}

	if (power1!=0){DrawHline(0,70,20,bmscolor[menucolor][5]); Puts(0, 20,  " Power [W]", &TM_Font_7x10, bmscolor[menucolor][37], bmscolor[menucolor][5]);}
	else          {DrawHline(0,70,20,bmscolor[menucolor][5]); Puts(0, 20,  "Current[A]", &TM_Font_7x10, bmscolor[menucolor][37], bmscolor[menucolor][5]);}
	if (graphhorizontal==0){Puts(5, 167,  "x1 seconds", &TM_Font_7x10, 0xFFFF, 0x0000);}
	if (graphhorizontal==1){Puts(5, 167,  "x1 minutes", &TM_Font_7x10, 0xFFFF, 0x0000);}
	if (graphhorizontal==2){Puts(5, 167,  "x6 minutes", &TM_Font_7x10, 0xFFFF, 0x0000);}

if (c2==13)
{
//---------------------------------------------
		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg("     PV1 & PV2 Graph         ",menucolor);

		Puts(0, 128, "PV1+PV2:", &TM_Font_7x10, bmscolor[menucolor][37], 0x0000);
		Puts(0, 88, "PV1:", &TM_Font_7x10, PV1color, 0x0000);
		Puts(0, 48, "PV2:", &TM_Font_7x10, PV2color, 0x0000);


		if ((zz5==1)||(c3!=cursor5))
		{
		cursor5=c3;
		if (cursor5<1) {cursor5=1;}

		if (graphhorizontal==0)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),PV1color,PV2color,menucolor,PV1graph,1,PV2graph,1,VBattgraph,0,"4m",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),PV1color,PV2color,menucolor,PV1graph,1,PV2graph,1,VBattgraph,1,"4m",240,240,120,cursor5);}
		}
		if (graphhorizontal==1)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),PV1color,PV2color,menucolor,PV1graph1h,1,PV2graph1h,1,VBattgraph1h,0,"4h",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),PV1color,PV2color,menucolor,PV1graph1h,1,PV2graph1h,1,VBattgraph1h,1,"4h",240,240,120,cursor5);}
		}
		if (graphhorizontal==2)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),PV1color,PV2color,menucolor,PV1graph24h,1,PV2graph24h,1,VBattgraph24h,0,"24h",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),PV1color,PV2color,menucolor,PV1graph24h,1,PV2graph24h,1,VBattgraph24h,1,"24h",240,240,120,cursor5);}
		}


		}
//---------------------------------------
}

if (c2==14)
{
//---------------------------------------------
		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg("       Battery Graph         ",menucolor);

		Puts(0, 88, "Charge     ", &TM_Font_7x10, Battpcolor, 0x0000);
		Puts(0, 48, "Discharge  ", &TM_Font_7x10, Battncolor, 0x0000);

		Puts(0, 128, "Battery:", &TM_Font_7x10, bmscolor[menucolor][37], 0x0000);

		if ((zz5==1)||(c3!=cursor5))
		{
		cursor5=c3;
		if (cursor5<1) {cursor5=1;}

		if (graphhorizontal==0)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),Battpcolor,Battncolor,menucolor,Battgraph,1,0,0,VBattgraph,0,"4m",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),Battpcolor,Battncolor,menucolor,Battgraph,1,0,0,VBattgraph,1,"4m",240,240,120,cursor5);}
		}
		if (graphhorizontal==1)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),Battpcolor,Battncolor,menucolor,Battgraph1h,1,0,0,VBattgraph1h,0,"4h",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),Battpcolor,Battncolor,menucolor,Battgraph1h,1,0,0,VBattgraph1h,1,"4h",240,240,120,cursor5);}
		}
		if (graphhorizontal==2)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),Battpcolor,Battncolor,menucolor,Battgraph24h,1,0,0,VBattgraph24h,0,"24h",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),Battpcolor,Battncolor,menucolor,Battgraph24h,1,0,0,VBattgraph24h,1,"24h",240,240,120,cursor5);}
		}




		}
//---------------------------------------
}

if (c2==15)
{
//---------------------------------------------
		LCDmenu_set_XY(0,0);
		LCDmenu_write_string_neg("       Load Graph            ",menucolor);

		Puts(0, 128, "Load+ExtLd:", &TM_Font_7x10, bmscolor[menucolor][37], 0x0000);
		Puts(0, 88, "Load:", &TM_Font_7x10,    Loadcolor, 0x0000);
		Puts(0, 48, "ExtLoad:", &TM_Font_7x10, ExtLoadcolor, 0x0000);


		if ((zz5==1)||(c3!=cursor5))
		{
		cursor5=c3;
		if (cursor5<1) {cursor5=1;}

		if (graphhorizontal==0)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph,1,ExtLoadgraph,1,VBattgraph,0,"4m",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph,1,ExtLoadgraph,1,VBattgraph,1,"4m",240,240,120,cursor5);}
		}
		if (graphhorizontal==1)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph1h,1,ExtLoadgraph1h,1,VBattgraph1h,0,"4h",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph1h,1,ExtLoadgraph1h,1,VBattgraph1h,1,"4h",240,240,120,cursor5);}
		}
		if (graphhorizontal==2)
		{
		if (power1==0){EnergyGraph(79,136-(2*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph24h,1,ExtLoadgraph24h,1,VBattgraph24h,0,"24h",240,240,120,cursor5);}
		else          {EnergyGraph(79,136-(2*53),Loadcolor,ExtLoadcolor,menucolor,Loadgraph24h,1,ExtLoadgraph24h,1,VBattgraph24h,1,"24h",240,240,120,cursor5);}
		}


		}
//---------------------------------------
}

		//zz5=0;


         if ((BOK==1)&&(release1==1)&&(lockkey!=1))
            {
            c333=c3;
    	    c3=1;
            key1=1;
            c22=c2;
            c2=16;
            BOK=0;
            buttondir=0;
            }



		if ((BBack==1)&&(release2==1)&&(lockkey!=1))
						{
					c3=4;
					release2=0;
					key1=1;
				    c2=1;
				    //BBack=0;
				    i5=0;
				    Fill_menu(0x0000);
		            zz5=1;
		            buttondir=0;
					    }

		if (lockkey!=1) {update1=1;}
		if (zz4==6) {update1=1;}

 }

//=================================================================================================
//      *************************     Parameter Settings Simple    ************************
//=================================================================================================

void parametersetsimple(void)
{
	if (c2!=2){return;}
	unsigned int v2,s,i2,i3,f1,f2,f3,cc8;
    s=5;
    f1=0;
    f2=20;
    f3=26;


    if (c3<1){c3=1;}
	if (c3>=(s+1)){c3=s;}


	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1_neg(2,0,c3,menucolor);
	LCDmenu_set_XY(2,0);
	LCDmenu_write_string_neg("/",menucolor);
	LCDmenu_set_XY(3,0);
	LCDmenu_write_Dec1_neg(2,0,s,menucolor);
	LCDmenu_set_XY(5,0);
	LCDmenu_write_string_neg("   Parameter Settings   ",menucolor);

	if (refresh1==1){ROMconvertbd(); refresh1=0;}

	if (first1>=1){

	if (first1!=10) {DrawFilledRectangle(0, 120, 320, 240, 0x0000);}


		 if (ROMdec[0]==1) {ROMunit[0]="LFP";}
			 else if (ROMdec[0]==2) {ROMunit[0]="LCO";}
			 else if (ROMdec[0]==3) {ROMunit[0]="LTO";}
			 else {ROMunit[0]="Cap";}
			 LCDmenu_set_XY(26,1);
			 LCDmenu_write_string(ROMunit[0],menucolor);//5

		   Puts(15, 125, ROMunit[0] , &TM_Font_11x18, bmscolor[menucolor][50],0x0000);
		   WriteDec(0,125, &TM_Font_11x18, bmscolor[menucolor][25] ,0x0000, 1,0,ROMdec[0],0);


		   if (ROMdec[0]==1) {Puts(0, 150,      "LiFePO4 LiFeYPO4 LiFeMnPO4", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000); Puts(60, 125, "LiFePO" , &TM_Font_11x18, 0xF11F,0x0000);Puts(126, 130, "4  " , &TM_Font_11x18, 0xF11F,0x0000);}
		   else if (ROMdec[0]==2) {Puts(0, 150, "LiCoO2 LiMn2O4 LiNiMnCoO2 ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, "LiCoO" , &TM_Font_11x18, 0xF11F,0x0000);Puts(115, 130, "2   " , &TM_Font_11x18, 0xF11F,0x0000);}
		   else if (ROMdec[0]==3) {Puts(0, 150, "Li4Ti5O12  2.4V*          ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, "Li" , &TM_Font_11x18, 0xF11F,0x0000);Puts(82, 130, "4" , &TM_Font_11x18, 0xF11F,0x0000);Puts(93, 125, "Ti" , &TM_Font_11x18, 0xF11F,0x0000);Puts(115, 130, "5" , &TM_Font_11x18, 0xF11F,0x0000);Puts(126, 125, "O" , &TM_Font_11x18, 0xF11F,0x0000);Puts(137, 130, "12" , &TM_Font_11x18, 0xF11F,0x0000);}
		   else if (ROMdec[0]==4) {Puts(0, 150, "SuperCap/UltraCap  2.3V   ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, " 2.3V    " , &TM_Font_11x18, 0xF11F,0x0000);}
		   else if (ROMdec[0]==5) {Puts(0, 150, "SuperCap/UltraCap  2.5V   ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, " 2.5V    " , &TM_Font_11x18, 0xF11F,0x0000);}
		   else {Puts(0, 150,            "SuperCap/UltraCap  2.7V   ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, " 2.7V    " , &TM_Font_11x18, 0xF11F,0x0000);}


	}
	first1=0;


	 if (ROMdec[2]<10){d[2][0]=1;}
	 else if (ROMdec[2]<100){d[2][0]=2;}
	 else if (ROMdec[2]<1000){d[2][0]=3;}
	 else {d[2][0]=4;}



	cellwire(0,120,menucolor);



	for (cc8=0 ;cc8<8;cc8++)
	{
	if (cellsetup[ROMdec[1]-3][cc8]==1){smallcell(0+(cc8*40),218,8-cc8,1);}
	else {smallcell(0+(cc8*40),218,8-cc8,0);}
	}



	scrolllenght=20;
	if ((c3==5)||(c3==4)){scrolllenght=29;}




	if ( (c3<=(s+1)) && (c3>=1) )
		                    {
		            v2=c3+(s-6); //6
		            if (v2<s)
		            {

				    for (i3=0;i3<3;i3++) //5
				    {
				    	LCDmenu_set_XY(f2,i3+1);
				    	LCDmenu_write_Dec(d[i3][0],d[i3][1],ROMdec[i3],menucolor);
				    	LCDmenu_set_XY(f1,i3+1); //line4
				    	LCDmenu_write_string(bmsmenu[i3][0],menucolor);
				    	LCDmenu_set_XY(f3,i3+1);
				    	LCDmenu_write_string(ROMunit[i3],menucolor);
				    }


				       	LCDmenu_set_XY(f1,4); //line4
				       	LCDmenu_write_string(bmsmenu[34][0],menucolor);
				       	LCDmenu_set_XY(f1,5); //line4
				       	LCDmenu_write_string(bmsmenu[35][0],menucolor);

				    if (c3<=3){
				    LCDmenu_set_XY(f2,c3);
				    LCDmenu_write_Dec(d[c3-1][0],d[c3-1][1],ROMdec[c3-1],menucolor);
				    LCDmenu_set_XY(f1,c3); //selected line
				    LCDmenu_write_string_neg(bmsmenu[c3-1][0],menucolor);
				    LCDmenu_set_XY(f3,c3);
				    LCDmenu_write_string(ROMunit[c3-1],menucolor);
				    }

				    if (c3>3){
				    LCDmenu_set_XY(f1,c3); //selected line
				    LCDmenu_write_string_neg(bmsmenu[c3+30][0],menucolor);
				    }





				    for(i2=0;i2<32;i2++)  //clear text buffer for the first 15 locations
				    {
				    chr[i2]=0x20;
				    }

				    if (c3>3){helptext=bmsmenu[c3+30][1];}
				    else {helptext=bmsmenu[c3-1][1];}



				    nr1=0;
				    c33=c3;
				    z=0;
				    i4=0;   //delay time for scrolling text
				    i5=1;   //activate scrolling text

				    while (*helptext!='\0')
				                  {
				                     chr[z]=*helptext;
				                     z++;
				                     helptext++;
				                  }
				    if (z>=21)
				    {
				        for (i2=0;i2<scrolllenght;i2++)
				        {
				         chr[z+i2]=chr[i2];
				        }

				    }
		            }
		            }


	if (BOK==1)
	{
	if (c3==4)
	{
	ROMconvertdb();
	LCDmenu_set_XY(0,5);
	ROMwriteall();
	LCDmenu_set_XY(0,4);//5--------------------------------
	LCDmenu_write_string_neg("       EEPROM write OK       ",menucolor);
	}
	else if(c3==5)
	{
	c22=c2;
	c2=23;
	BOK=0;
	key1=1;
	c33=c3;
	c3=1;
	first1=1;
	DrawFilledRectangle(0, 120, 320, 240, 0x0000);
	zz6=1;
	}
	else
	{
	c22=c2;
	c2=22;
    BOK=0;
    key1=1;
    c33=c3;
    c3=ROMdec[c33-1];

    if ((c33!=14)&&(c33!=16)&&(c33!=18))
     		{
     		c3=ROMdec[c33-1];
     		}
     		if (c33==14)
     		{
     		c3=ROMdec2[0];
     		//ROMdec[c33-1]=realcurrent[c3][2];
     		}
     		if (c33==16)
     		{
     		c3=ROMdec2[1];
     		//ROMdec[c33-1]=realcurrent[c3][1];
     		}
     		if (c33==18)
     		{
     		c3=ROMdec2[2];
     		//ROMdec[c33-1]=realcurrent[c3][3];
     		}

	}
	}

	if (BBack==1)
    {
    c3=2;
    key1=1;
    c2=100;
    BBack=0;
    ROMconvertbd();
    DrawFilledRectangle(0, 120, 320, 240, 0x0000);
    zz6=1;
    }


}

//=================================================================================================
//          *************************     Parameter Settings     ************************
//=================================================================================================

void parameterset(void)
{
	if (c2!=23){return;}
	unsigned int v2,s,i2,i3,f1,f2,f3;
    s=35;
    f1=0;
    f2=20;
    f3=26;


    if (c3<1){c3=1;}
	if (c3>=(s+1)){c3=s;}


	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1_neg(2,0,c3,menucolor);
	LCDmenu_set_XY(2,0);
	LCDmenu_write_string_neg("/",menucolor);
	LCDmenu_set_XY(3,0);
	LCDmenu_write_Dec1_neg(2,0,s,menucolor);
	LCDmenu_set_XY(5,0);//----------------------------
	LCDmenu_write_string_neg("    Advanced Settings   ",menucolor);


	scrolllenght=20;
	if (c3==35){scrolllenght=29;}

	if (refresh1==1){ROMconvertbd(); refresh1=0;}

	   if (ROMdec[2]<10){d[2][0]=1;}
	   else if (ROMdec[2]<100){d[2][0]=2;}
	   else if (ROMdec[2]<1000){d[2][0]=3;}
	   else {d[2][0]=4;}

	if ( (c3<=(s+1)) && (c3>=1) )
		                    {
		            v2=c3+(s-9); //6
		            if (v2<s)
		            {

				    for (i3=0;i3<8;i3++) //5
				    {
				    	LCDmenu_set_XY(f2,i3+1);
				    	LCDmenu_write_Dec(d[i3][0],d[i3][1],ROMdec[i3],menucolor);
				    	LCDmenu_set_XY(f1,i3+1); //line4
				    	LCDmenu_write_string(bmsmenu[i3][0],menucolor);
				    	LCDmenu_set_XY(f3,i3+1);
				    	LCDmenu_write_string(ROMunit[i3],menucolor);
				    }

				    LCDmenu_set_XY(f2,c3);
				    LCDmenu_write_Dec(d[c3-1][0],d[c3-1][1],ROMdec[c3-1],menucolor);
				    LCDmenu_set_XY(f1,c3); //selected line
				    LCDmenu_write_string_neg(bmsmenu[c3-1][0],menucolor);
				    LCDmenu_set_XY(f3,c3);
				    LCDmenu_write_string(ROMunit[c3-1],menucolor);


				    for(i2=0;i2<32;i2++)  //clear text buffer for the first 15 locations
				    {
				    chr[i2]=0x20;
				    }

				    helptext=bmsmenu[c3-1][1];




				    nr1=0;
				    c33=c3;
				    z=0;
				    i4=0;   //delay time for scrolling text
				    i5=1;   //activate scrolling text

				    while (*helptext!='\0')
				                  {
				                     chr[z]=*helptext;
				                     z++;
				                     helptext++;
				                  }
				    if (z>=21)
				    {
				        for (i2=0;i2<scrolllenght;i2++)
				        {
				         chr[z+i2]=chr[i2];
				        }

				    }




		            }
		            if ((v2>=s)&&(v2<(s+(s-8))))//5
		            {

		            for(i3=0;i3<8;i3++)//5
		            {
		            	LCDmenu_set_XY(f2,i3+1);
		            	LCDmenu_write_Dec(d[c3-(8-i3)][0],d[c3-(8-i3)][1],ROMdec[c3-(8-i3)],menucolor);//5
		            	LCDmenu_set_XY(f1,i3+1); //
		            	LCDmenu_write_string(bmsmenu[c3-(8-i3)][0],menucolor);//5
		            	LCDmenu_set_XY(f3,i3+1);
		            	LCDmenu_write_string(ROMunit[c3-(8-i3)],menucolor);//5
		            }

		            LCDmenu_set_XY(f2,8);//5
		            LCDmenu_write_Dec(d[c3-1][0],d[c3-1][1],ROMdec[c3-1],menucolor);
		            LCDmenu_set_XY(f1,8); //selected line 5
		            LCDmenu_write_string_neg(bmsmenu[c3-1][0],menucolor);
		            LCDmenu_set_XY(f3,8);//5
		            LCDmenu_write_string(ROMunit[c3-1],menucolor);


		            for(i2=0;i2<32;i2++)  //clear text buffer
		            {
		            chr[i2]=0x20;
		            }
		            helptext=bmsmenu[c3-1][1];
		            nr1=0;
		            c33=c3;
		            z=0;
		            i4=0;   //delay time for scrolling text
		            i5=1;   //activate scrolling text

		            while (*helptext!='\0')
		            				                  {
		            				                     chr[z]=*helptext;
		            				                     z++;
		            				                     helptext++;
		            				                  }
		            if (z>=21)
		            			 	    {
		            			 	    for (i2=0;i2<scrolllenght;i2++)
		            			 	    {
		            			 	     chr[z+i2]=chr[i2];
		            			 	    }
		            			 	    }



		            }

		            }



	if (BOK==1)
	{
	if (c3==35)
	{
	ROMconvertdb();
	LCDmenu_set_XY(0,8);
	ROMwriteall();
	LCDmenu_set_XY(0,8);//5
	LCDmenu_write_string_neg("       EEPROM write OK       ",menucolor);
	}
	else
	{
	c22=c2;
	c2=22;
    BOK=0;
    key1=1;
    c33=c3;
    c3=ROMdec[c33-1];

    if ((c33!=14)&&(c33!=16)&&(c33!=18))
     		{
     		c3=ROMdec[c33-1];
     		}
     		if (c33==14)
     		{
     		c3=ROMdec2[0];
     		//ROMdec[c33-1]=realcurrent[c3][2];
     		}
     		if (c33==16)
     		{
     		c3=ROMdec2[1];
     		//ROMdec[c33-1]=realcurrent[c3][1];
     		}
     		if (c33==18)
     		{
     		c3=ROMdec2[2];
     		//ROMdec[c33-1]=realcurrent[c3][3];
     		}





	}
	}

	if (BBack==1)
    {
    c3=1;
    key1=1;
    c2=2;
    BBack=0;

    zz6=1;
    first1=1;
    }


}
//=================================================================================================
//set value for parameterset.

void valueset(void)
 {


	if (c2!=22){return;}
    unsigned int l1,f2,cc8;
    f2=20;

	buttondir=1;
	step1=bmsmenuval[c33-1][4];
 	if(c3<bmsmenuval[c33-1][2]){c3=bmsmenuval[c33-1][2];}
 	if(c3>bmsmenuval[c33-1][3]){c3=bmsmenuval[c33-1][3];}

 if (c33==3){
	   if (c3<10){d[2][0]=1;}
	   else if (c3<100){d[2][0]=2;}
	   else if (c3<1000){d[2][0]=3;}
	   else {d[2][0]=4;}
 }

	   if (c33<8)//5
 	   {
 	   LCDmenu_set_XY(f2,c33);
 	   LCDmenu_write_Dec_neg(d[c33-1][0],d[c33-1][1],c3,menucolor);
 	   }



	   if ((c33>=8)&&(c33!=14)&&(c33!=16)&&(c33!=18))//5
 	   {
 	   LCDmenu_set_XY(f2,8);
 	   LCDmenu_write_Dec_neg(d[c33-1][0],d[c33-1][1],c3,menucolor);
 	   }

	   if (c33==14)
	   {
	   LCDmenu_set_XY(f2,8);//5
	   LCDmenu_write_Dec_neg(d[c33-1][0],d[c33-1][1],realcurrent[c3][2],menucolor);
	   }
	   if (c33==16)
	   {
	   LCDmenu_set_XY(f2,8);//5
	   LCDmenu_write_Dec_neg(d[c33-1][0],d[c33-1][1],realcurrent[c3][1],menucolor);
	   }
	   if (c33==18)
	   {
	   LCDmenu_set_XY(f2,8);//5
	   LCDmenu_write_Dec_neg(d[c33-1][0],d[c33-1][1],realcurrent[c3][3],menucolor);
	   }





	   if ((c33==1)&&((c22==2)||(c22==23)))
	   {
		 if (c3==1) {ROMunit[0]="LFP";}
		 else if (c3==2) {ROMunit[0]="LCO";}
		 else if (c3==3) {ROMunit[0]="LTO";}
		 else {ROMunit[0]="Cap";}
		 LCDmenu_set_XY(26,1);
		 LCDmenu_write_string(ROMunit[0],menucolor);//5
	   }

	   if ((c33==1)&&(c22==2))
	   {

	   Puts(15, 125, ROMunit[0] , &TM_Font_11x18, bmscolor[menucolor][50],0x0000);
	   WriteDec(0,125, &TM_Font_11x18, bmscolor[menucolor][25] ,0x0000, 1,0,c3,0);


	   if (c3==1) {Puts(0, 150,      "LiFePO4 LiFeYPO4 LiFeMnPO4", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000); Puts(60, 125, "LiFePO" , &TM_Font_11x18, 0xF11F,0x0000);Puts(126, 130, "4  " , &TM_Font_11x18, 0xF11F,0x0000);}
	   else if (c3==2) {Puts(0, 150, "LiCoO2 LiMn2O4 LiNiMnCoO2 ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, "LiCoO" , &TM_Font_11x18, 0xF11F,0x0000);Puts(115, 130, "2   " , &TM_Font_11x18, 0xF11F,0x0000);}
	   else if (c3==3) {Puts(0, 150, "Li4Ti5O12  2.4V*          ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, "Li" , &TM_Font_11x18, 0xF11F,0x0000);Puts(82, 130, "4" , &TM_Font_11x18, 0xF11F,0x0000);Puts(93, 125, "Ti" , &TM_Font_11x18, 0xF11F,0x0000);Puts(115, 130, "5" , &TM_Font_11x18, 0xF11F,0x0000);Puts(126, 125, "O" , &TM_Font_11x18, 0xF11F,0x0000);Puts(137, 130, "12" , &TM_Font_11x18, 0xF11F,0x0000);}
	   else if (c3==4) {Puts(0, 150, "SuperCap/UltraCap  2.3V   ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, " 2.3V    " , &TM_Font_11x18, 0xF11F,0x0000);}
	   else if (c3==5) {Puts(0, 150, "SuperCap/UltraCap  2.5V   ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, " 2.5V    " , &TM_Font_11x18, 0xF11F,0x0000);}
	   else {Puts(0, 150,            "SuperCap/UltraCap  2.7V   ", &TM_Font_7x10, 0xFFFF,0x0000); Puts(60, 125, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 130, "         " , &TM_Font_11x18, 0xF11F,0x0000);Puts(60, 125, " 2.7V    " , &TM_Font_11x18, 0xF11F,0x0000);}
	   }

	   if ((c22==2)&&(c33==2))
	   {

		   cellwire(0,120,menucolor);

		   	for (cc8=0 ;cc8<8;cc8++)
		   	{
		   	if (cellsetup[c3-3][cc8]==1){smallcell(0+(cc8*40),218,8-cc8,1);}
		   	else {smallcell(0+(cc8*40),218,8-cc8,0);}
		   	}

	   }



 	   if (BOK==1)
 	   {
 		if (c33==1)
 		{
 		 //Load preset parameters from table

 		if ((c3!=4)&&(c3!=5)&&(c3!=6))
 		{
 		ROMunit[2]="Ah ";
 		for (l1=0;l1<34;l1++)
 		{
 		ROMdec[l1]=BatteryType[l1][c3-1];
 		}
 		ROMdec2[0]=7;
 		ROMdec2[1]=3;
 		ROMdec2[2]=5;
 		}

 		if ((c3==4)||(c3==5)||(c3==6))
 		 		{
 		 		ROMunit[2]="F  ";
 		 		for (l1=0;l1<34;l1++)
 		 		{
 		 		ROMdec[l1]=BatteryType[l1][c3-1];
 		 		}
 		 		ROMdec2[0]=4;
 		 		ROMdec2[1]=1;
 		 		ROMdec2[2]=0;
 		 		}
 		}

 		if ((c33!=14)&&(c33!=16)&&(c33!=18))
 		{
 		ROMdec[c33-1]=c3;
 		}
 		if (c33==14)
 		{
 		ROMdec2[0]=c3;
 		ROMdec[c33-1]=realcurrent[c3][2];
 		}
 		if (c33==16)
 		{
 		ROMdec2[1]=c3;
 		ROMdec[c33-1]=realcurrent[c3][1];
 		}
 		if (c33==18)
 		{
 		ROMdec2[2]=c3;
 		ROMdec[c33-1]=realcurrent[c3][3];
 		}
 		c2=c22;
 		BOK=0;
 		c3=c33;
 		buttondir=0;
 		step1=1;
 	   }



 	  if (BBack==1)
 	      {
 		 c2=c22;
 		 BBack=0;
 		 c3=c33;
 		 buttondir=0;
 		 step1=1;
 		 first1=10;
 	      }


 }


//=================================================================================================
//       ***************************          Device Settings       ************************
//=================================================================================================

void deviceset(void)
{
	if (c2!=3){return;new1=0;}



		unsigned int v2,s,i2;
	    s=7;

	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
		LCDmenu_set_XY(1,0);
		LCDmenu_write_string_neg("/",menucolor);
		LCDmenu_set_XY(2,0);
		LCDmenu_write_Dec1_neg(1,0,s,menucolor);
		LCDmenu_set_XY(3,0);
		LCDmenu_write_string_neg("       Device Settings    ",menucolor);


		if ( (c3<=(s+1)) && (c3>=1) )
			                    {
			            v2=c3-1;
			            if (v2<s)
			            {

                        if (new1==0){
			            LCDmenu_set_XY(0,1); //dacus test menu
					    LCDmenu_write_string(devset[0][0],menucolor);

			            LCDmenu_set_XY(0,2); //dacus test menu
					    LCDmenu_write_string(devset[1][0],menucolor);

					    LCDmenu_set_XY(0,3); //dacus test menu
					    LCDmenu_write_string(devset[2][0],menucolor);

					    LCDmenu_set_XY(0,4); //dacus test menu
					    LCDmenu_write_string(devset[3][0],menucolor);

					    LCDmenu_set_XY(0,5); //dacus test menu
					    LCDmenu_write_string(devset[4][0],menucolor);

					    LCDmenu_set_XY(0,6); //dacus test menu
					    LCDmenu_write_string(devset[5][0],menucolor);

					    LCDmenu_set_XY(0,7); //dacus test menu
					    LCDmenu_write_string(devset[6][0],menucolor);

					    LCDmenu_set_XY(0,8); //dacus test menu
					    LCDmenu_write_string(devset[7][0],menucolor);


                        }

                        if (b3>55){b3=35;}
                        if (b4>55){b4=35;}

                        if ((b3!=0)&&(c3!=1))
                        {
                        LCDmenu_set_XY(0,c3-1); //dacus test menu
                        LCDmenu_write_string(devset[c3-2][0],menucolor);
                        }

                        if ((b3!=0)&&(c3==1))
                                 {
                         LCDmenu_set_XY(0,s); //dacus test menu
                         LCDmenu_write_string(devset[s-1][0],menucolor);
                                  }
                        if ((b4!=0)&&(c3!=s))
                          {
                           LCDmenu_set_XY(0,c3+1); //dacus test menu
                           LCDmenu_write_string(devset[c3][0],menucolor);
                          }

                         if ((b4!=0)&&(c3==s))
                           {
                            LCDmenu_set_XY(0,1); //dacus test menu
                            LCDmenu_write_string(devset[0][0],menucolor);
                           }

                         LCDmenu_set_XY(0,c3); //dacus test menu
                         LCDmenu_write_string_neg(devset[c3-1][0],menucolor);




					    for(i2=0;i2<30;i2++)  //clear text buffer for the first 30 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=devset[c3-1][1];
					    scrolllenght=29;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

new1=1;

			            }

	      }


		if (BOK==1)
			{
			if (c3!=6 && c3!=7) {Fill_menu(0x0000);}//LCDmenu_clear();
			BOK=0;
			if (c3==1){c2=31;}
			if (c3==2){c2=33;}
			if (c3==3){c2=35;}
			if (c3==4){c2=37;}
			if (c3==5){c2=39;}
			if (c3==6){savedevset=1;LCDmenu_set_XY(0,6);LCDmenu_write_string_neg("            busy...          ",menucolor);i5=0;}
			if (c3==7){if (factoryreset==3){factoryreset=1;} if (factoryreset==0){factoryreset=3;LCDmenu_set_XY(0,7);LCDmenu_write_string_neg("      Push OK to confirm     ",menucolor);}; i5=0;}

			if (c3!=6 && c3!=7)
			{
			c333=c3;
			c3=1;
			i5=0;
			c8=0;
			new1=0;
			}

	  }

		if (BMENU==1)
				{
				c2=100;
				c3=3;
				BMENU=0;
				key1=1;
				new1=0;
				}

		if (BBack==1)
		    {
		    c3=3;
		    key1=1;
		    c2=100;
		    BBack=0;
		    new1=0;
		    }

}

//=================================================================================================
//time and date submenu.

void timeanddate(void)
{
	if (c2!=31){return;}

    RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
    RTC_GetDate(RTC_Format_BIN, &RTC_DateStructure);

		unsigned int v2,s,i2,i3,f1,f2,f3;
	    s=5;
	    f1=0;
	    f2=20;
	    f3=26;

        td[0]=RTC_TimeStructure.RTC_Hours;
        td[1]=RTC_TimeStructure.RTC_Minutes;
        td[2]=RTC_DateStructure.RTC_Date;
        td[3]=RTC_DateStructure.RTC_Month;
        td[4]=RTC_DateStructure.RTC_Year+2000;

	 if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
		LCDmenu_set_XY(1,0);
		LCDmenu_write_string_neg("/",menucolor);
		LCDmenu_set_XY(2,0);
		LCDmenu_write_Dec1_neg(1,0,s,menucolor);
		LCDmenu_set_XY(3,0);
		LCDmenu_write_string_neg("                  ",menucolor);

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<5;i3++)
					    {
					    	LCDmenu_set_XY(f2,i3+1);
					    	LCDmenu_write_Dec(timedateval[i3][0],timedateval[i3][1],td[i3],menucolor);
					    	LCDmenu_set_XY(f1,i3+1); //line4
					    	LCDmenu_write_string(timedate[i3][0],menucolor);
					    	LCDmenu_set_XY(f3,i3+1);
					    	LCDmenu_write_string(timedate[i3][2],menucolor);
					    }

					    LCDmenu_set_XY(f2,c3);
					    LCDmenu_write_Dec(timedateval[c3-1][0],timedateval[c3-1][1],td[c3-1],menucolor);
					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(timedate[c3-1][0],menucolor);
					    LCDmenu_set_XY(f3,c3);
					    LCDmenu_write_string(timedate[c3-1][2],menucolor);


					    for(i2=0;i2<30;i2++)  //clear text buffer for the first 30 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=timedate[c3-1][1];
					    scrolllenght=20;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

		if (BOK==1)
		{
	 c22=c2;
	 c2=32;
	 BOK=0;
	 key1=1;
	 c33=c3;
	 c3=td[c33-1];
	 i5=0;
		}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

		if (BBack==1)
				    {
				    c3=1;
				    key1=1;
				    c2=3;
				    BBack=0;
				    Fill_menu(0x0000);
				    }

   //update1=1;

}

//=================================================================================================
//time and date value set.
void valueset2(void)
 {

	if (c2!=32){return;}

    buttondir=1;
	step1=timedateval[c33-1][4];
 	if(c3<timedateval[c33-1][2]){c3=timedateval[c33-1][2];}
 	if(c3>timedateval[c33-1][3]){c3=timedateval[c33-1][3];}


	 if (c33<5)
 	   {
 	   LCDmenu_set_XY(20,c33);
 	   LCDmenu_write_Dec_neg(timedateval[c33-1][0],timedateval[c33-1][1],c3,menucolor);
 	   }
 	   if (c33>=5)
 	   {
 	   LCDmenu_set_XY(20,5);
 	   LCDmenu_write_Dec_neg(timedateval[c33-1][0],timedateval[c33-1][1],c3,menucolor);
 	   }

 	   if (BOK==1)
 	   {
 		td[c33-1]=c3;

 		RTC_TimeStructure.RTC_Hours=td[0];
  		RTC_TimeStructure.RTC_Minutes=td[1];
  		RTC_DateStructure.RTC_Date=td[2];
  		RTC_DateStructure.RTC_Month=td[3];
  		RTC_DateStructure.RTC_Year=td[4]-2000;

  		if(RTC_SetTime(RTC_Format_BIN, &RTC_TimeStructure) == ERROR)
  		  {
  			LCDmenu_set_XY(0,0);
  			LCDmenu_write_string("Time set Error",menucolor);
  		    delay_ms(1500);
  		  }
  		else
  		{
  		}

  		if(RTC_SetDate(RTC_Format_BIN, &RTC_DateStructure) == ERROR)
  		  {
  			LCDmenu_set_XY(0,0);
  			LCDmenu_write_string("Date set Error",menucolor);
  			delay_ms(1500);
  		  }
  		  else
  		  {
  		RTC_WriteBackupRegister(RTC_BKP_DR0, 0x32F2);
  		  }


  		 		c2=c22;
  		 		BOK=0;
  		 		c3=c33;
  		 		buttondir=0;
  		 		step1=1;
  		 		key1=1;
  		 		lk1();
  		 		bk1();

 	   }

 	  if (BBack==1)
 	  		    {
 		        c2=c22;
 		   	    c3=c33;
 		   	    buttondir=0;
 		   	    step1=1;
 		   		key1=1;
 	  		    BBack=0;
 	  		    }

 	    if (BMENU==1)
 	    {
 	    c2=31;
 	    BOK=0;
 	    BMENU=0;
 	    step1=1;
 	    key1=1;
 	    }

 }

//=================================================================================================

void lk1 (void)

{
	RTC_TimeTypeDef RTC_TimeStructure;
	RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
	swtimer4=((RTC_TimeStructure.RTC_Minutes*60)+RTC_TimeStructure.RTC_Seconds);

}

//=================================================================================================
//backlight value set.
void bk1 (void)

{

	unsigned long test4;
	unsigned int loop1;

	RTC_TimeTypeDef RTC_TimeStructure;
	RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
	swtimer1=((RTC_TimeStructure.RTC_Minutes*60)+RTC_TimeStructure.RTC_Seconds);

		LCDLED_OFF;
		delay_ms(1);
		 for (loop1=bk[0];loop1<=32;loop1++)
		 {
			 LCDLED_OFF;

			 for (test4=0;test4<20;test4++)
		     {
			 __asm__ __volatile__ ("nop\n\t":::"memory");
			 }

			 LCDLED_ON;

			 for (test4=0;test4<20;test4++)
			 {
			 __asm__ __volatile__ ("nop\n\t":::"memory");
			 }

		 }


}

//=================================================================================================
//Backlight submenu
void backlight(void)
{
	if (c2!=33){return;}

		unsigned int v2,s,i2,i3,f1,f2,f3;
	    s=4;
	    f1=0;
	    f2=20;
	    f3=26;

	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
		LCDmenu_set_XY(1,0);
		LCDmenu_write_string_neg("/",menucolor);
		LCDmenu_set_XY(2,0);
		LCDmenu_write_Dec1_neg(1,0,s,menucolor);
		LCDmenu_set_XY(3,0);
		LCDmenu_write_string_neg("  LCD Back Light          ",menucolor);

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f2,i3+1);
					    	LCDmenu_write_Dec(backlightval[i3][0],backlightval[i3][1],bk[i3],menucolor);
					    	LCDmenu_set_XY(f1,i3+1); //line4
					    	LCDmenu_write_string(LCDbacklight[i3][0],menucolor);
					    	LCDmenu_set_XY(f3,i3+1);
					    	LCDmenu_write_string(LCDbacklight[i3][2],menucolor);
					    }

					    LCDmenu_set_XY(f2,c3);
					    LCDmenu_write_Dec(backlightval[c3-1][0],backlightval[c3-1][1],bk[c3-1],menucolor);
					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(LCDbacklight[c3-1][0],menucolor);
					    LCDmenu_set_XY(f3,c3);
					    LCDmenu_write_string(LCDbacklight[c3-1][2],menucolor);


					    for(i2=0;i2<30;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=LCDbacklight[c3-1][1];
					    scrolllenght=20;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

		if (BOK==1)
		{
		c22=c2;
		c2=34;
	 BOK=0;
	 key1=1;
	 c33=c3;
	 c3=bk[c33-1];
	 i5=0;
		}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

		if (BBack==1)
				{
			c3=2;
			key1=1;
		    c2=3;
		    BBack=0;
		    Fill_menu(0x0000);
			    }


}

//=================================================================================================
void valueset3(void)
 {

	if (c2!=34){return;}

    buttondir=1;
	step1=backlightval[c33-1][4];
 	if(c3<backlightval[c33-1][2]){c3=backlightval[c33-1][2];}
 	if(c3>backlightval[c33-1][3]){c3=backlightval[c33-1][3];}

 	if (c33==2)
 		{
 		bk[c33-1]=c3;
 		menucolor=c3;
 		}


	 if (c33<5)
 	   {
 	   LCDmenu_set_XY(20,c33);
 	   LCDmenu_write_Dec_neg(backlightval[c33-1][0],backlightval[c33-1][1],c3,menucolor);
 	   }
 	   if (c33>=5)
 	   {
 	   LCDmenu_set_XY(20,5);
 	   LCDmenu_write_Dec_neg(backlightval[c33-1][0],backlightval[c33-1][1],c3,menucolor);
 	   }

if (c33==1)
{
bk[c33-1]=c3;
bk1();
}



 	   if (BOK==1)
 	   {
 		bk[c33-1]=c3;
 		c2=c22;
 		BOK=0;
 		c3=c33;
 		buttondir=0;
 		step1=1;
 		bk1();
 	   }

		if (BBack==1)
				{
			c2=c22;
			c3=c33;
			buttondir=0;
			step1=1;
		    BBack=0;
			    }

 	    if (BMENU==1)
 	    {
 	    c2=31;
 	    BOK=0;
 	    BMENU=0;
 	    step1=1;
 	    key1=1;
 	    }


 }
//=================================================================================================
//USART-submenu
void USARTMenu(void)
{
	if (c2!=35){return;}

		unsigned int v2,s,i2,i3,f1,f2,f3;
	    s=3;
	    f1=0;
	    f2=20;
	    f3=26;
        //bk1();
	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}

		LCDmenu_set_XY(0,0);
		LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
		LCDmenu_set_XY(1,0);
		LCDmenu_write_string_neg("/",menucolor);
		LCDmenu_set_XY(2,0);
		LCDmenu_write_Dec1_neg(1,0,s,menucolor);
		LCDmenu_set_XY(3,0);
		LCDmenu_write_string_neg("      USART Settings      ",menucolor);


		if ( (c3<=(s+1)) && (c3>=1) )
			                    {
			            c9=1;
			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f2,i3+1);
					    	LCDmenu_write_Dec(USARTval[i3][0],USARTval[i3][1],Serial[i3],menucolor);
					    	LCDmenu_set_XY(f1,i3+1); //line4
					    	LCDmenu_write_string(USARTmenu[i3][0],menucolor);
					    	LCDmenu_set_XY(f3,i3+1);
					    	LCDmenu_write_string(USARTmenu[i3][2],menucolor);
					    }

					    LCDmenu_set_XY(f2,c3);
					    LCDmenu_write_Dec(USARTval[c3-1][0],USARTval[c3-1][1],Serial[c3-1],menucolor);
					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(USARTmenu[c3-1][0],menucolor);
					    LCDmenu_set_XY(f3,c3);
					    LCDmenu_write_string(USARTmenu[c3-1][2],menucolor);


					    for(i2=0;i2<30;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=USARTmenu[c3-1][1];
					    scrolllenght=20;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

		if (BOK==1)
		{
		c22=c2;
		c2=36;
	 BOK=0;
	 key1=1;
	 c33=c3;
	 c3=Serial[c33-1];
	 i5=0;
		}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

		if (BBack==1)
			{
			c3=3;
			key1=1;
		    c2=3;
		    BBack=0;
		    Fill_menu(0x0000);
		    }


}

//=================================================================================================
void valueset4(void)
 {


	if (c2!=36){return;}

    buttondir=1;
	step1=USARTval[c33-1][4];
 	if(c3<USARTval[c33-1][2]){c3=USARTval[c33-1][2];}
 	if(c3>USARTval[c33-1][3]){c3=USARTval[c33-1][3];}


	 if (c33<8)
 	   {
 	   LCDmenu_set_XY(20,c33);
 	   LCDmenu_write_Dec_neg(USARTval[c33-1][0],USARTval[c33-1][1],c3,menucolor);
 	   }
 	   if (c33>=8)
 	   {
 	   LCDmenu_set_XY(20,8);
 	   LCDmenu_write_Dec_neg(USARTval[c33-1][0],USARTval[c33-1][1],c3,menucolor);
 	   }


 	   if (BOK==1)
 	   {
 		Serial[c33-1]=c3;
 		if (c33==2){WiFi_Off();USART_Config();}
 		if (c33==1 && c3==1){WiFi_Off();USART_Config();UsartDataHeader();}
 		c2=c22;
 		BOK=0;
 		c3=c33;
 		buttondir=0;
 		step1=1;
 	   }

 	  if (BBack==1)
 	  	{
 	    c2=c22;
 	    c3=c33;
 		buttondir=0;
 		step1=1;
 	    BBack=0;
 	    }


 	    if (BMENU==1)
 	    {
 	    c2=31;
 	    BOK=0;
 	    BMENU=0;
 	    step1=1;
 	    key1=1;
 	    }


 }

//=================================================================================================
void WiFiMenu(void)
{
	if (c2!=37){return;}
	unsigned int v2,s,i2,i3,f1,f2,f3;
	    s=6;
	    f1=0;
	    f2=20;
	    f3=26;
        //bk1();
	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}

		LCDmenu_set_XY(0,0);
		LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
		LCDmenu_set_XY(1,0);
		LCDmenu_write_string_neg("/",menucolor);
		LCDmenu_set_XY(2,0);
		LCDmenu_write_Dec1_neg(1,0,s,menucolor);
		LCDmenu_set_XY(3,0);
		LCDmenu_write_string_neg("      WiFi Settings       ",menucolor);

		DrawHline(0,320,140,bmscolor[menucolor][25]);

		//if (c3==1){WiFi_CWSAP();}//---------
		//else if (WiFis[0]==0){WiFi_Off();}

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {
			            c9=1;
			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f2,i3+1);
					    	LCDmenu_write_Dec(WiFival[i3][0],WiFival[i3][1],WiFis[i3],menucolor);
					    	LCDmenu_set_XY(f1,i3+1); //line4
					    	LCDmenu_write_string(WiFimenu[i3][0],menucolor);
					    	LCDmenu_set_XY(f3,i3+1);
					    	LCDmenu_write_string(WiFimenu[i3][2],menucolor);
					    }

					    LCDmenu_set_XY(f2,c3);
					    LCDmenu_write_Dec(WiFival[c3-1][0],WiFival[c3-1][1],WiFis[c3-1],menucolor);
					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(WiFimenu[c3-1][0],menucolor);
					    LCDmenu_set_XY(f3,c3);
					    LCDmenu_write_string(WiFimenu[c3-1][2],menucolor);


					    for(i2=0;i2<30;i2++)  //clear text buffer for the first 10 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=WiFimenu[c3-1][1];
					    scrolllenght=20;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

			            }

			            }

		if (BOK==1)
		{
		c22=c2;
		c2=38;
	    BOK=0;
	    key1=1;
	    c33=c3;
	    c3=WiFis[c33-1];
	    i5=0;
		}

		if (BMENU==1)
			{
			c2=3;
	  BMENU=0;
	  BOK=0;
	 	c3=1;
	 	key1=1;
			}

	if (BBack==1)
				{
				c3=4;
				key1=1;
			    c2=3;
			    BBack=0;
			    Fill_menu(0x0000);
			    }


}

void valueset6(void)
 {


	if (c2!=38){return;}

    buttondir=1;
	step1=WiFival[c33-1][4];
 	if(c3<WiFival[c33-1][2]){c3=WiFival[c33-1][2];}
 	if(c3>WiFival[c33-1][3]){c3=WiFival[c33-1][3];}


	 if (c33<8)
 	   {
 	   LCDmenu_set_XY(20,c33);
 	   LCDmenu_write_Dec_neg(WiFival[c33-1][0],WiFival[c33-1][1],c3,menucolor);
 	   }
 	   if (c33>=8)
 	   {
 	   LCDmenu_set_XY(20,8);
 	   LCDmenu_write_Dec_neg(WiFival[c33-1][0],WiFival[c33-1][1],c3,menucolor);
 	   }


 	   if (BOK==1)
 	   {
 		WiFis[c33-1]=c3;
 		if (c33==1 && c3==1){WiFi_On();}
 		if (c33==1 && c3==0){WiFi_Off();}
 		//if (c33==1 && c3==1){UsartDataHeader();}
 		c2=c22;
 		BOK=0;
 		c3=c33;
 		buttondir=0;
 		step1=1;
 	   }

 	  if (BBack==1)
 	  	{
 	    c2=c22;
 	    c3=c33;
 		buttondir=0;
 		step1=1;
 	    BBack=0;
 	    }


 	    if (BMENU==1)
 	    {
 	    c2=31;
 	    BOK=0;
 	    BMENU=0;
 	    step1=1;
 	    key1=1;
 	    }


 }



//=================================================================================================
void uSDMenu(void)
{
	if (c2!=39){return;}
	unsigned int v2,s,i2,i3,f1,f2,f3;
		    s=2;
		    f1=0;
		    f2=20;
		    f3=26;
	        //bk1();
		    if (c3<1){c3=s;}
			if (c3>=(s+1)){c3=1;}

			LCDmenu_set_XY(0,0);
			LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
			LCDmenu_set_XY(1,0);
			LCDmenu_write_string_neg("/",menucolor);
			LCDmenu_set_XY(2,0);
			LCDmenu_write_Dec1_neg(1,0,s,menucolor);
			LCDmenu_set_XY(3,0);
			LCDmenu_write_string_neg("  Internal Data log Menu  ",menucolor);

			DrawHline(0,320,60,bmscolor[menucolor][25]);

			if ( (c3<=(s+1)) && (c3>=1) )
				                    {
				            c9=1;
				            v2=c3-1;
				            if (v2<s)
				            {

						    for (i3=0;i3<s;i3++)
						    {
						    	LCDmenu_set_XY(f2,i3+1);
						    	LCDmenu_write_Dec(uSDval[i3][0],uSDval[i3][1],uSD[i3],menucolor);
						    	LCDmenu_set_XY(f1,i3+1); //line4
						    	LCDmenu_write_string(uSDmenu[i3][0],menucolor);
						    	LCDmenu_set_XY(f3,i3+1);
						    	LCDmenu_write_string(uSDmenu[i3][2],menucolor);
						    }

						    LCDmenu_set_XY(f2,c3);
						    LCDmenu_write_Dec(uSDval[c3-1][0],uSDval[c3-1][1],uSD[c3-1],menucolor);
						    LCDmenu_set_XY(f1,c3); //selected line
						    LCDmenu_write_string_neg(uSDmenu[c3-1][0],menucolor);
						    LCDmenu_set_XY(f3,c3);
						    LCDmenu_write_string(uSDmenu[c3-1][2],menucolor);


						    for(i2=0;i2<30;i2++)  //clear text buffer for the first 10 locations
						    {
						    chr[i2]=0x20;
						    }

						    helptext=uSDmenu[c3-1][1];

						    scrolllenght=20;
						    if (c3==2){scrolllenght=29;}
						    nr1=0;
						    c33=c3;
						    z=0;
						    i4=0;   //delay time for scrolling text
						    i5=1;   //activate scrolling text

						    while (*helptext!='\0')
						                  {
						                     chr[z]=*helptext;
						                     z++;
						                     helptext++;
						                  }
						    if (z>=8)
						    {
						        for (i2=0;i2<scrolllenght;i2++)
						        {
						         chr[z+i2]=chr[i2];
						        }

						    }

				            }

				            }

			if (BOK==1)
			{
			if (c3==2)
			{
            sizeRX2=0;
			startdownloadflash=1;
			datanr=0;
			i5=0;
			//LCDmenu_set_XY(0,2);//
			//LCDmenu_write_string_neg("      Download complete.     ",menucolor);
			}
			else{
			c22=c2;
			c2=30;
		    BOK=0;
		    key1=1;
		    c33=c3;
		    c3=uSD[c33-1];
		    i5=0;
			}
			}

			if (BMENU==1)
				{
				startdownloadflash=0;
				c2=3;
		  BMENU=0;
		  BOK=0;
		 	c3=1;
		 	key1=1;
				}


	if (BBack==1)
				{
		        startdownloadflash=0;
				c3=5;
				key1=1;
			    c2=3;
			    BBack=0;
			    Fill_menu(0x0000);
			    }


}

//=================================================================================================

void valueset7(void)
 {


	if (c2!=30){return;}

	startdownloadflash=0;
    buttondir=1;
	step1=uSDval[c33-1][4];
 	if(c3<uSDval[c33-1][2]){c3=uSDval[c33-1][2];}
 	if(c3>uSDval[c33-1][3]){c3=uSDval[c33-1][3];}


	 if (c33<8)
 	   {
 	   LCDmenu_set_XY(20,c33);
 	   LCDmenu_write_Dec_neg(uSDval[c33-1][0],uSDval[c33-1][1],c3,menucolor);
 	   }
 	   if (c33>=8)
 	   {
 	   LCDmenu_set_XY(20,8);
 	   LCDmenu_write_Dec_neg(uSDval[c33-1][0],uSDval[c33-1][1],c3,menucolor);
 	   }


 	   if (BOK==1)
 	   {
 		uSD[c33-1]=c3;
 		//if (c33==2){USART_Config();}
 		//if (c33==1 && c3==1){UsartDataHeader();}
 		c2=c22;
 		BOK=0;
 		c3=c33;
 		buttondir=0;
 		step1=1;
 	   }

 	  if (BBack==1)
 	  	{
 	    c2=c22;
 	    c3=c33;
 		buttondir=0;
 		step1=1;
 	    BBack=0;
 	    }


 	    if (BMENU==1)
 	    {
 	    c2=31;
 	    BOK=0;
 	    BMENU=0;
 	    step1=1;
 	    key1=1;
 	    }


 }

//=================================================================================================
// ***************************          EXT IO and ADC Settings       ************************
//=================================================================================================

void extioadc(void)
{
	if (c2!=4){return;new1=0;}

		unsigned int v2,s,i2;
	    s=2;

	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
		LCDmenu_set_XY(1,0);
		LCDmenu_write_string_neg("/",menucolor);
		LCDmenu_set_XY(2,0);
		LCDmenu_write_Dec1_neg(1,0,s,menucolor);
		LCDmenu_set_XY(3,0);
		LCDmenu_write_string_neg("      EXT IO and ADC      ",menucolor);


		if (first1>=1){

			if (first1!=10) {DrawFilledRectangle(0, 120, 320, 240, 0x0000);}

             extconnector(0,90,menucolor);

			}
			first1=0;

		if ( (c3<=(s+1)) && (c3>=1) )
			                    {
			            v2=c3-1;
			            if (v2<s)
			            {

                        if (new1==0){
			            LCDmenu_set_XY(0,1); //dacus test menu
					    LCDmenu_write_string(extioadcset[0][0],menucolor);

			            LCDmenu_set_XY(0,2); //dacus test menu
					    LCDmenu_write_string(extioadcset[1][0],menucolor);

					    LCDmenu_set_XY(0,3); //dacus test menu
					    LCDmenu_write_string(extioadcset[2][0],menucolor);

					    LCDmenu_set_XY(0,4); //dacus test menu
					    LCDmenu_write_string(extioadcset[3][0],menucolor);

					    LCDmenu_set_XY(0,5); //dacus test menu
					    LCDmenu_write_string(extioadcset[4][0],menucolor);

					    LCDmenu_set_XY(0,6); //dacus test menu
					    LCDmenu_write_string(extioadcset[5][0],menucolor);

					    LCDmenu_set_XY(0,7); //dacus test menu
					    LCDmenu_write_string(extioadcset[6][0],menucolor);

					    LCDmenu_set_XY(0,8); //dacus test menu
					    LCDmenu_write_string(extioadcset[7][0],menucolor);

                        }

                        if (b3>55){b3=35;}
                        if (b4>55){b4=35;}

                        if ((b3!=0)&&(c3!=1))
                        {
                        LCDmenu_set_XY(0,c3-1); //dacus test menu
                        LCDmenu_write_string(extioadcset[c3-2][0],menucolor);
                        }

                        if ((b3!=0)&&(c3==1))
                                 {
                         LCDmenu_set_XY(0,2); //dacus test menu
                         LCDmenu_write_string(extioadcset[s-1][0],menucolor);
                                  }
                        if ((b4!=0)&&(c3!=s))
                          {
                           LCDmenu_set_XY(0,c3+1); //dacus test menu
                           LCDmenu_write_string(extioadcset[c3][0],menucolor);
                          }

                         if ((b4!=0)&&(c3==s))
                           {
                            LCDmenu_set_XY(0,1); //dacus test menu
                            LCDmenu_write_string(extioadcset[0][0],menucolor);
                           }

                         LCDmenu_set_XY(0,c3); //dacus test menu
                         LCDmenu_write_string_neg(extioadcset[c3-1][0],menucolor);




					    for(i2=0;i2<30;i2++)  //clear text buffer for the first 30 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=extioadcset[c3-1][1];
					    scrolllenght=29;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

new1=1;

			            }

	      }


		if (BOK==1)
			{
			Fill_menu(0x0000);//LCDmenu_clear();
			BOK=0;
			if (c3==1){c2=41;}
			if (c3==2){c2=43;}
			c333=c3;
			c3=1;
			i5=0;
			c8=0;
			new1=0;
			DrawFilledRectangle(0, 80, 320, 240, 0x0000);
			zz6=1;

	  }



		if (BBack==1)
		    {
		    c3=4;
		    key1=1;
		    c2=100;
		    BBack=0;
		    new1=0;
		    DrawFilledRectangle(0, 80, 320, 240, 0x0000);
		    zz6=1;
		    }

}

//=================================================================================================
void manual(void)
{
	if (c2!=6){return;new1=0;}

		unsigned int v2,s,i2;
	    s=2;

	    if (c3<1){c3=s;}
		if (c3>=(s+1)){c3=1;}


		LCDmenu_set_XY(0,0);
		LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
		LCDmenu_set_XY(1,0);
		LCDmenu_write_string_neg("/",menucolor);
		LCDmenu_set_XY(2,0);
		LCDmenu_write_Dec1_neg(1,0,s,menucolor);
		LCDmenu_set_XY(3,0);
		LCDmenu_write_string_neg("      Manual & Install    ",menucolor);



		if ( (c3<=(s+1)) && (c3>=1) )
			                    {
			            v2=c3-1;
			            if (v2<s)
			            {

                        if (new1==0){
			            LCDmenu_set_XY(0,1); //dacus test menu
					    LCDmenu_write_string(manualset[0][0],menucolor);

			            LCDmenu_set_XY(0,2); //dacus test menu
					    LCDmenu_write_string(manualset[1][0],menucolor);

					    LCDmenu_set_XY(0,3); //dacus test menu
					    LCDmenu_write_string(manualset[2][0],menucolor);

					    LCDmenu_set_XY(0,4); //dacus test menu
					    LCDmenu_write_string(manualset[3][0],menucolor);

					    LCDmenu_set_XY(0,5); //dacus test menu
					    LCDmenu_write_string(manualset[4][0],menucolor);

					    LCDmenu_set_XY(0,6); //dacus test menu
					    LCDmenu_write_string(manualset[5][0],menucolor);

					    LCDmenu_set_XY(0,7); //dacus test menu
					    LCDmenu_write_string(manualset[6][0],menucolor);

					    LCDmenu_set_XY(0,8); //dacus test menu
					    LCDmenu_write_string(manualset[7][0],menucolor);

                        }

                        if (b3>55){b3=35;}
                        if (b4>55){b4=35;}

                        if ((b3!=0)&&(c3!=1))
                        {
                        LCDmenu_set_XY(0,c3-1); //dacus test menu
                        LCDmenu_write_string(manualset[c3-2][0],menucolor);
                        }

                        if ((b3!=0)&&(c3==1))
                                 {
                         LCDmenu_set_XY(0,2); //dacus test menu
                         LCDmenu_write_string(manualset[s-1][0],menucolor);
                                  }
                        if ((b4!=0)&&(c3!=s))
                          {
                           LCDmenu_set_XY(0,c3+1); //dacus test menu
                           LCDmenu_write_string(manualset[c3][0],menucolor);
                          }

                         if ((b4!=0)&&(c3==s))
                           {
                            LCDmenu_set_XY(0,1); //dacus test menu
                            LCDmenu_write_string(manualset[0][0],menucolor);
                           }

                         LCDmenu_set_XY(0,c3); //dacus test menu
                         LCDmenu_write_string_neg(manualset[c3-1][0],menucolor);




					    for(i2=0;i2<30;i2++)  //clear text buffer for the first 30 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=manualset[c3-1][1];
					    scrolllenght=29;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }

new1=1;

			            }

	      }


		if (BOK==1)
			{
			Fill_menu(0x0000);//LCDmenu_clear();
			BOK=0;
			if (c3==1){c2=61;}
			if (c3==2){c2=62;}
			c333=c3;
			c3=1;
			i5=0;
			c8=0;
			new1=0;
			DrawFilledRectangle(0, 80, 320, 240, 0x0000);
			zz6=1;

	  }



		if (BBack==1)
		    {
		    c3=6;
		    key1=1;
		    c2=100;
		    BBack=0;
		    new1=0;
		    DrawFilledRectangle(0, 80, 320, 240, 0x0000);
		    zz6=1;
		    }

}

//=================================================================================================

void Install1(void)
{
	if (c2!=61){return;}
	uint32_t s;
		    s=8;
		    buttondir=1;

		    if (c3<1){c3=s;}
		    if (c3>=(s+1)){c3=1;}

	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
	LCDmenu_set_XY(1,0);
	LCDmenu_write_string_neg("/",menucolor);
	LCDmenu_set_XY(2,0);
	LCDmenu_write_Dec1_neg(1,0,s,menucolor);
	LCDmenu_set_XY(3,0);
	LCDmenu_write_string_neg("      Install steps       ",menucolor);



	if ( (c3<=(s+1)) && (c3>=1) )
	{
    if (c3<=s){
	Puts(0, 20, install1[c3][0], &TM_Font_11x18, bmscolor[menucolor][0], bmscolor[menucolor][30]);
	//Puts(0, 45, install1[0][1], &TM_Font_7x10, bmscolor[menucolor][62], bmscolor[menucolor][0]);
	Puts2(2, 44, install1[c3][1], &TM_Font_7x10, bmscolor[menucolor][62], 0,5);
    }
	}



	if (BBack==1)
				{
				c3=1;
				key1=1;
			    c2=6;
			    BBack=0;
			    buttondir=0;
			    Fill_menu(0x0000);

			    }

}

//=================================================================================================

void Manual1(void)
{
	if (c2!=62){return;}
	uint32_t s;
		    s=19;
		    buttondir=1;

		    if (c3<1){c3=s;}
		    if (c3>=(s+1)){c3=1;}

	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1_neg(2,0,c3,menucolor);
	LCDmenu_set_XY(2,0);
	LCDmenu_write_string_neg("/",menucolor);
	LCDmenu_set_XY(3,0);
	LCDmenu_write_Dec1_neg(2,0,s,menucolor);
	LCDmenu_set_XY(5,0);
	LCDmenu_write_string_neg("        Manual          ",menucolor);


	if ( (c3<=(s+1)) && (c3>=1) )
	{
    if (c3<=s){
	Puts(0, 20, manual1[c3][0], &TM_Font_11x18, bmscolor[menucolor][0], bmscolor[menucolor][30]);
	//Puts(0, 45, install1[0][1], &TM_Font_7x10, bmscolor[menucolor][62], bmscolor[menucolor][0]);
	Puts2(2, 44, manual1[c3][1], &TM_Font_7x10, bmscolor[menucolor][62], 0,5);
    }
	}


	if (BBack==1)
				{
				c3=2;
				key1=1;
			    c2=6;
			    BBack=0;
			    buttondir=0;
			    Fill_menu(0x0000);

			    }

}


//=================================================================================================
void EXT1ANDEXT2(void)
{
	if (c2!=41){return;}
	unsigned int v2,s,i2,i3,f1,f2,f3;
		    s=8;
		    f1=0;
		    f2=20;
		    f3=26;

		    if (c3<1){c3=s;}
		    if (c3>=(s+1)){c3=1;}

	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
	LCDmenu_set_XY(1,0);
	LCDmenu_write_string_neg("/",menucolor);
	LCDmenu_set_XY(2,0);
	LCDmenu_write_Dec1_neg(1,0,s,menucolor);
	LCDmenu_set_XY(3,0);
	LCDmenu_write_string_neg("       EXT IOx            ",menucolor);



	if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f2,i3+1);
					    	LCDmenu_write_Dec(EXT125val[i3][0],EXT125val[i3][1],EXT[i3],menucolor);
					    	LCDmenu_set_XY(f1,i3+1); //line4
					    	LCDmenu_write_string(EXT125[i3][0],menucolor);
					    	LCDmenu_set_XY(f3,i3+1);
					    	LCDmenu_write_string(EXT125[i3][2],menucolor);
					    }

					    LCDmenu_set_XY(f2,c3);
					    LCDmenu_write_Dec(EXT125val[c3-1][0],EXT125val[c3-1][1],EXT[c3-1],menucolor);
					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(EXT125[c3-1][0],menucolor);
					    LCDmenu_set_XY(f3,c3);
					    LCDmenu_write_string(EXT125[c3-1][2],menucolor);


					    for(i2=0;i2<15;i2++)  //clear text buffer for the first 15 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=EXT125[c3-1][1];
					    scrolllenght=20;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }




			            }

			            }

	if (BOK==1)
	{
	c22=c2;
	c2=42;
	BOK=0;
	key1=1;
	c33=c3;
	c3=EXT[c33-1];
	i5=0;
	}

	if (BBack==1)
				{
				c3=1;
				key1=1;
			    c2=4;
			    BBack=0;
			    Fill_menu(0x0000);
			    DrawFilledRectangle(0, 180, 320, 240, 0x0000);
			    first1=1;
			    }

}

//=================================================================================================
void valueset5(void)
 {

	if (c2!=42){return;}

    buttondir=1;
	step1=EXT125val[c33-1][4];
 	if(c3<EXT125val[c33-1][2]){c3=EXT125val[c33-1][2];}
 	if(c3>EXT125val[c33-1][3]){c3=EXT125val[c33-1][3];}


	 if (c33<8)
 	   {
 	   LCDmenu_set_XY(20,c33);
 	   LCDmenu_write_Dec_neg(EXT125val[c33-1][0],EXT125val[c33-1][1],c3,menucolor);
 	   }
 	   if (c33>=8)
 	   {
 	   LCDmenu_set_XY(20,8);
 	   LCDmenu_write_Dec_neg(EXT125val[c33-1][0],EXT125val[c33-1][1],c3,menucolor);
 	   }



 	   if (BOK==1)
 	   {
 		EXT[c33-1]=c3;
 		c2=c22;
 		BOK=0;
 		c3=c33;
 		buttondir=0;
 		step1=1;
 		key1=1;
 	   }

		if (BBack==1)
				{
			c2=c22;
			c3=c33;
			buttondir=0;
			step1=1;
		    BBack=0;
		    key1=1;
			    }

 	    if (BMENU==1)
 	    {
 	    c2=31;
 	    BOK=0;
 	    BMENU=0;
 	    step1=1;
 	    key1=1;
 	    }

 }

//=================================================================================================
void EXTADC(void)
{
	if (c2!=43){return;}
	unsigned int v2,s,i2,i3,f1,f2,f3;
		    s=8;
		    f1=0;
		    f2=19;
		    f3=25;

		    if (c3<1){c3=s;}
		    if (c3>=(s+1)){c3=1;}

	LCDmenu_set_XY(0,0);
	LCDmenu_write_Dec1_neg(1,0,c3,menucolor);
	LCDmenu_set_XY(1,0);
	LCDmenu_write_string_neg("/",menucolor);
	LCDmenu_set_XY(2,0);
	LCDmenu_write_Dec1_neg(1,0,s,menucolor);
	LCDmenu_set_XY(3,0);
	LCDmenu_write_string_neg("        EXT ADC           ",menucolor);



	if ( (c3<=(s+1)) && (c3>=1) )
			                    {

			            v2=c3-1;
			            if (v2<s)
			            {

					    for (i3=0;i3<s;i3++)
					    {
					    	LCDmenu_set_XY(f2,i3+1);
					    	LCDmenu_write_Dec(ADCval[i3][0],ADCval[i3][1],EXTADC1[i3],menucolor);
					    	LCDmenu_set_XY(f1,i3+1); //line4
					    	LCDmenu_write_string(ADC[i3][0],menucolor);
					    	LCDmenu_set_XY(f3,i3+1);
					    	LCDmenu_write_string(ADC[i3][2],menucolor);
					    }

					    LCDmenu_set_XY(f2,c3);
					    LCDmenu_write_Dec(ADCval[c3-1][0],ADCval[c3-1][1],EXTADC1[c3-1],menucolor);
					    LCDmenu_set_XY(f1,c3); //selected line
					    LCDmenu_write_string_neg(ADC[c3-1][0],menucolor);
					    LCDmenu_set_XY(f3,c3);
					    LCDmenu_write_string(ADC[c3-1][2],menucolor);


					    for(i2=0;i2<15;i2++)  //clear text buffer for the first 15 locations
					    {
					    chr[i2]=0x20;
					    }

					    helptext=ADC[c3-1][1];
					    scrolllenght=19;
					    nr1=0;
					    c33=c3;
					    z=0;
					    i4=0;   //delay time for scrolling text
					    i5=1;   //activate scrolling text

					    while (*helptext!='\0')
					                  {
					                     chr[z]=*helptext;
					                     z++;
					                     helptext++;
					                  }
					    if (z>=8)
					    {
					        for (i2=0;i2<scrolllenght;i2++)
					        {
					         chr[z+i2]=chr[i2];
					        }

					    }




			            }

			            }

	if (BOK==1)
	{
	c22=c2;
	c2=44;
	BOK=0;
	key1=1;
	c33=c3;
	c3=EXTADC1[c33-1];
	i5=0;
	}

	if (BBack==1)
				{
				c3=2;
				key1=1;
			    c2=4;
			    BBack=0;
			    Fill_menu(0x0000);
			    DrawFilledRectangle(0, 180, 320, 240, 0x0000);
			    first1=1;
			    }

}

//=================================================================================================
void valueset8(void)
 {

	if (c2!=44){return;}

    buttondir=1;
	step1=ADCval[c33-1][4];
 	if(c3<ADCval[c33-1][2]){c3=ADCval[c33-1][2];}
 	if(c3>ADCval[c33-1][3]){c3=ADCval[c33-1][3];}


	 if (c33<8)
 	   {
 	   LCDmenu_set_XY(19,c33);
 	   LCDmenu_write_Dec_neg(ADCval[c33-1][0],ADCval[c33-1][1],c3,menucolor);
 	   }
 	   if (c33>=8)
 	   {
 	   LCDmenu_set_XY(19,8);
 	   LCDmenu_write_Dec_neg(ADCval[c33-1][0],ADCval[c33-1][1],c3,menucolor);
 	   }



 	   if (BOK==1)
 	   {
 		EXTADC1
		[c33-1]=c3;
 		c2=c22;
 		BOK=0;
 		c3=c33;
 		buttondir=0;
 		step1=1;
 		key1=1;
 	   }

		if (BBack==1)
				{
			c2=c22;
			c3=c33;
			buttondir=0;
			step1=1;
		    BBack=0;
		    key1=1;
			    }

 	    if (BMENU==1)
 	    {
 	    c2=31;
 	    BOK=0;
 	    BMENU=0;
 	    step1=1;
 	    key1=1;
 	    }

 }



//=================================================================================================
// ***************************          Diagnostics           ************************
//=================================================================================================

void diagnostics(void)
{
	if (c2!=5){return;}

	unsigned int jj;



    //Fill2(0x0000);


	Puts(21, 5, "    PAR IC ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
	WriteDec(0, 5, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25], 3,0,i2c,0);
	c8=((RAMrx[37]*256)+RAMrx[36]);  //ext temp T2 that is solar current from MAX8040
    c9=(RAMrx[15]*256)+RAMrx[14];  //battery current
    WriteDec(7, 15, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 4,0,c8,0);
    WriteDec(7, 25, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 4,0,0,0);
    WriteDec(7, 35, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 4,0,c9,0);
    WriteDec(7, 45, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 4,0,c9cal,0);
    buttondir=1;
    ROMconvertdb();
	if (c3<=1){c3=1;}
	if (c3>=35){c3=34;}

	   if (c3<=22)
	{
	c6=(c3-1)*4;
	ROMread(c6);
	c10=0;
	Puts(49, 55, "ROM ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
	WriteDec(77, 15, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMt[0],0);
	WriteDec(77, 25, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMt[1],0);
	WriteDec(77, 35, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMt[2],0);
	WriteDec(77, 45, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMt[3],0);

	WriteDec(49, 15, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMtemp[c6+0],0);
	WriteDec(49, 25, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMtemp[c6+1],0);
	WriteDec(49, 35, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMtemp[c6+2],0);
	WriteDec(49, 45, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,ROMtemp[c6+3],0);

	WriteDec(77, 55, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25], 3,0,(c3-1)*4,0);

	}

	   if ((c3>=23)&&(c3<=35))
	{
	c6=(c3-23)*4;
	c10=1;
	Puts(49, 55, "RAM ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);


	WriteDec(77, 15, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,RAMrx[c6+0],0);
	WriteDec(77, 25, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,RAMrx[c6+1],0);
	WriteDec(77, 35, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,RAMrx[c6+2],0);
	WriteDec(77, 45, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15], 3,0,RAMrx[c6+3],0);

	Puts(49, 15, "   ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15]);
	Puts(49, 25, "   ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15]);
	Puts(49, 35, "   ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15]);
	Puts(49, 45, "   ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][15]);

	WriteDec(77, 55, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][35], 3,0,((c3-1)*4)-8,0);

	}
	Puts(0, 55, "Address", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);

	Puts(107, 55, "Batt ISL", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
	WriteDec(165, 55, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][35], 4,1,BatteryCurrent,0);
	Puts(207, 55, "Batt STM", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
	WriteDec(265, 55, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][35], 5,3,BatteryCurrent2,0);

	Puts(107, 44, "ExtTemp ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
	WriteDec(165, 44, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][35], 4,0,ETEMP,0);
	Puts(207, 44, "IntTemp ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
	WriteDec(265, 44, &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][35], 4,0,ITEMP,0);

//-----------------------------------------------------

//get_SDADC036();
//temp1=ADC16[6][0];
//get_SDADC146();
//temp1=(ADC16[6][0]+temp1);
//get_SDADC256();
//ADC16[6][0]=(ADC16[6][0]+temp1)/3;

  ADC16[1][1]=0;
  read_all_SDADC_noautorange(16);


//WriteDec(120, 180, &TM_Font_11x18, 0xFFFF, 0x0000, 4,3,Time2[2],1);
//WriteDec(120, 200, &TM_Font_11x18, 0xFFFF, 0x0000, 4,3,Time2[0],1);
//WriteDec(120, 220, &TM_Font_11x18, 0xFFFF, 0x0000, 4,3,Time2[1],1);

//Puts(10, 180, "ADC", &TM_Font_11x18, 0xFFF0, 0x0000);
//Puts(10, 200, "Graph", &TM_Font_11x18, 0xFFF0, 0x0000);
//Puts(10, 220, "Text+Calc", &TM_Font_11x18, 0xFFF0, 0x0000);

//Puts(188, 180, "s", &TM_Font_11x18, 0xFFEA, 0x0000);
//Puts(188, 200, "s", &TM_Font_11x18, 0xFFEA, 0x0000);
//Puts(188, 220, "s", &TM_Font_11x18, 0xFFEA, 0x0000);

//delay_ms(80);

//Time=0;


Puts(0, 95+(0*12), "PV2+        ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
Puts(0, 95+(1*12), "EXTADC2     ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
Puts(0, 95+(2*12), "EXTLoad     ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
Puts(0, 95+(3*12), "EXTADC3     ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
Puts(0, 95+(4*12), "EXTADC4     ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
Puts(0, 95+(5*12), "PV1+        ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);
Puts(0, 95+(6*12), "Battery+    ", &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25]);



for (jj=0 ;jj<7;++jj){
//WriteDec(135, 95+(jj*12), &TM_Font_7x10, 0xFFFF, 0x02A0, 5,3,ADC16[jj][7]/5,1);
WriteDec(190, 95+(jj*12), &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25], 5,3,ADC16[jj][5]/5,1);
WriteDec(245, 95+(jj*12), &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25], 5,3,ADC16[jj][0]/5,1);
WriteDec(300, 95+(jj*12), &TM_Font_7x10, bmscolor[menucolor][63], bmscolor[menucolor][25], 1,0,gain_real[ADC16[jj][1]],0);
Puts(145, 95+(jj*12), "V", &TM_Font_7x10, 0xFFF0, 0x0000);
Puts(310, 95+(jj*12), "x", &TM_Font_7x10, 0xFFF0, 0x0000);
Puts(290, 95+(jj*12), "d", &TM_Font_7x10, 0xFFF0, 0x0000);
}



for (jj=0 ;jj<7;++jj){

if (ADC16[jj][0]>150){

if (((ADC16[jj][0]-150)<ADC16[jj][3]) && (ADC16[jj][3]<(ADC16[jj][0]+150)) && ADC16[jj][4]==0)
{
	ADC16[jj][3]=(ADC16[jj][0]+(ADC16[jj][3]*7))/8;
	//delay_ms(80);
}

else
{ADC16[jj][3]=ADC16[jj][0];}

}


else
{
	if ((ADC16[jj][3]<(ADC16[jj][0]+150)) && ADC16[jj][4]==0)
			{
				ADC16[jj][3]=(ADC16[jj][0]+(ADC16[jj][3]*7))/8;
				//delay_ms(80);
			}
			else
			{ADC16[jj][3]=ADC16[jj][0];}
}



if (jj==6){
if (ADC16[jj][3]>ADC16[jj][5]){
Puts(92, 95+(jj*12), "+", &TM_Font_7x10, 0xFFFF, 0x0000);
WriteDec(100, 95+(jj*12), &TM_Font_7x10, 0xFFFF, 0x0000, 5,4,(((ADC16[jj][3]-ADC16[jj][5])*0.275)/(gain_real[ADC16[6][1]]))/5,1);
}
else
{
Puts(92, 95+(jj*12), "-", &TM_Font_7x10, 0xFFFF, 0x0000);
WriteDec(100, 95+(jj*12), &TM_Font_7x10, 0xFFFF, 0x0000, 5,4,(((ADC16[jj][5]-ADC16[jj][3])*0.275)/(gain_real[ADC16[6][1]]))/5,1);
}
}

else
{
if (ADC16[jj][3]>=ADC16[jj][5]){
WriteDec(100, 95+(jj*12), &TM_Font_7x10, 0xFFFF, 0x0000, 5,4,(((ADC16[jj][3]-ADC16[jj][5])*0.275)/(gain_real[ADC16[jj][1]]))/5,1);
}
else{WriteDec(100, 95+(jj*12), &TM_Font_7x10, 0xFFFF, 0x0000, 5,4,0,1);}
}


}


if (BBack==1)
	{
	c2=100;
	c3=5;
    BBack=0;
    key1=1;
    buttondir=0;
    first1=0;
    Fill_menu(0x0000);
	}

update1=1;

}

void decbin(uint8_t n){uint8_t i; for (i=0;i<6;i++) {logo11[5-i]=(n%2==0 ?0:1); n/=2;}}

//=================================================================================================
// ***************************          About            ************************
//=================================================================================================

void about(void)
{

if (c2!=7){return;}
//everything else is in Logo1
	if (BBack==1)
		{
		c2=100;
		c3=7;
		new1=0;
 	    BBack=0;
 	    key1=1;
 	   first1=0;
 	   Fill_menu(0x0000);
		}

}

//=================================================================================================
// ***************************          Load ON/OFF      ************************
//=================================================================================================

void LoadONOFF(void)
{
//	if (BLoad!=1){return;}
	bk1();
	if (lockkey!=0){key1=1;return;}
	RAMreadall();

	tempH=((RAMrx[6]) & 0x01);
    if(tempH!=0){
        tempL=(((RAMrx[7])&0xBF)+64);
    	RAMwrite(0x87,tempL); //uC manual control
    	LCDmenu_set_XY(18,0);
    	LCDmenu_write_string("Load&PV OFF",menucolor);
    	tempL=(RAMrx[6] & 0xFC);
    	RAMwrite(0x86,tempL);//load and PV is OFF
    	// need to do also charge OFF
        delay_ms(300);
    }
 	else
 	{
 		tempL=(((RAMrx[7])&0xE7)+(16+8));
 		RAMwrite(0x87,tempL); //uC manual load monitor control
 		tempL=(((RAMrx[6])&0x0F)+(128+32));
 		RAMwrite(0x86,tempL); //uC manual load monitor control

 		tempL=(((RAMrx[7])&0xBF));
 		RAMwrite(0x87,tempL);        //FET auto enable
 		LCDmenu_set_XY(18,0);
 		LCDmenu_write_string("Load&PV ON ",menucolor);
 		delay_ms(300);
 	}
RAMreadall();
}

//=================================================================================================
// ***************************        Scrolling Help.    ************************
//=================================================================================================

void scrollhelp(void)
{

  if ((i5!=1)||(lockkey!=0)){return;}

	unsigned int pos,i1;
    pos=c33;

                if (pos>=8)//5
                 {
                	pos=8;
                 }

                 if (i4<=100)//200
                 {
                 i4++;
                 return;
                 }
                 if ((i3<8)||(c33==0)) {i3++;return;}//5

                 for (i1=0;i1<scrolllenght;i1++)
                 {
                    if (nr1>=z+1)
                       {
                        nr1=0;
                       }
                 LCDmenu_set_XY(i1,pos);
                 LCDmenu_write_char_neg(chr[nr1+i1],menucolor);
                 }
                nr1++;
                i3=0;


}

//=================================================================================================
//                                  Touch Key processing.
//=================================================================================================

void ProcessSensors(void)
{

	if (TEST_TKEY(1-buttondir))  // UP button
	  {
		if(b3==0){c3=c3+step1;key1=1;}
		if(b3>30){c3=c3+step1;key1=1;}
		if(b3>100){c3=c3+(step1*3);key1=1;}
		b3++;
	  }
	  else
	  {
		  b3=0;
	  }
    if (TEST_TKEY(0+buttondir))  //Down Button
    {
    	if(b4==0){c3=c3-step1;key1=1;}
    	if(b4>30){c3=c3-step1;key1=1;}
    	if(b4>100){c3=c3-(step1*3);key1=1;}
    	b4++;
     }
    else
    {
    b4=0;
     }

  if (TEST_TKEY(4))   //OK Button
        {
    BOK=1;
    if (b1==0){key1=1;}
    b1=1;
        }
        else
        {
        BOK=0;
        b1=0;
        }

  if (TEST_TKEY(3))   // Menu Button
            {
      BMENU=1;
      if ((b2==0)&&((c2==2)||(c2==23)||(c2==22))){ROMconvertbd();}
      if ((lockkey!=1)&&(c2!=100)){c2=100;first1=1;zz6=1;}
      if (b2==0){key1=1;Fill_menu(0x0000);new1=0;}

      b2++;
      if (lockkey!=1){c3=1;buttondir=0;}
      if (b2>90){if(b5==0){LoadONOFF();}b2=1;b5=1;}
            }
            else
            {
      BMENU=0;
      b2=0;
      b5=0;
            }


   if (TEST_TKEY(2))   // Back Button
              {
          BBack=1;
          if (b6==0){key1=1;}
          b6=1;
              }
              else
             {
         BBack=0;
         b6=0;
             }


return;
}

//----------------------------------------------------------------------------------------------------

void Logo1 (void)

{

	char const lg1[]="#B33A33B##333333B##B33A33B##B33333B##B''''''##A44A544##B44444B##;75444A##144B444##B33333B##444444B##2331$$A##B8:B:8B#";//the electrodacus animated logo data
	uint8_t n,x,y,pix2,pix,col1,col2;
	char kwh[9],kwh1[1],h2[6];


	n = secb;
	if (n>=40){n=n-40;}
	if (n>=20){n=n-20;}
	if (n>12){n=12;}
	//DrawFilledRectangle(0,0,70,120,0x0000);
	for (y=0;y<9;y++){
	for (x=0;x<7;x++){
	decbin(lg1[y+(12*9)]-35);
	pix2=logo11[x];
	decbin(lg1[y+(n*9)]-35);
	pix=logo11[x];
	if (pix==1){col1=62;col2=16;}
	else if (pix2==1){col1=27;col2=3;}
	else {col1=18;col2=1;}
	DrawFilledRectangle(x*5,y*5,(x*5)+4,(y*5)+4,bmscolor[menucolor][col1]);
	if(y>=6){DrawFilledRectangle(x*5,(17.5-y)*5,(x*5)+4,((17.5-y)*5)+4,bmscolor[menucolor][(col2)+((y-5)*2)]);}
	}}

#if SBMS100>0
	Puts(60, 10, "SBMS100", &TM_Font_11x18,bmscolor[menucolor][62], bmscolor[menucolor][0]);
#endif
#if SBMS100==0
	Puts(60, 10, "SBMS60", &TM_Font_11x18,bmscolor[menucolor][62], bmscolor[menucolor][0]);
#endif

	Puts(175, 40, "www.ElectroDacus.com", &TM_Font_7x10,bmscolor[menucolor][55], bmscolor[menucolor][0]);


	Puts(0, 60,     "Hardware:0.3b  ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);
	Puts(166, 60,   "Software:1.0a ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);

	Puts(0, 80,            "PV1            ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);

	kwh[0] = ((PV1uWh2/100000) / 1000000000 ) + 0x30;
	kwh[1] = ((PV1uWh2/100000) % 1000000000/100000000 ) + 0x30;
	kwh[2] = ((PV1uWh2/100000) % 100000000/10000000 ) + 0x30;
	kwh[3] = ((PV1uWh2/100000) % 10000000/1000000 ) + 0x30;
	kwh[4] = ((PV1uWh2/100000) % 1000000/100000 ) + 0x30;
	kwh[5] = ((PV1uWh2/100000) % 100000/10000 ) + 0x30;
	kwh[6] = ((PV1uWh2/100000) % 10000/1000 ) + 0x30;
	kwh[7] = ((PV1uWh2/100000) % 1000/100 ) + 0x30;
	kwh[8] = ((PV1uWh2/100000) % 100/10 ) + 0x30;
	Puts(166, 80, kwh, &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	Puts(265, 80, ".", &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	kwh1[0] = ((PV1uWh2/100000) % 10 );
	WriteDec(276, 80,  &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15], 1,0,kwh1[0],0);
	//Puts(276, 80, kwh1 , &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][12]);
	Puts(287, 80,"Wh ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);


	Puts(0, 100,            "PV2            ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);
	kwh[0] = ((PV2uWh2/100000) / 1000000000 ) + 0x30;
	kwh[1] = ((PV2uWh2/100000) % 1000000000/100000000 ) + 0x30;
	kwh[2] = ((PV2uWh2/100000) % 100000000/10000000 ) + 0x30;
	kwh[3] = ((PV2uWh2/100000) % 10000000/1000000 ) + 0x30;
	kwh[4] = ((PV2uWh2/100000) % 1000000/100000 ) + 0x30;
	kwh[5] = ((PV2uWh2/100000) % 100000/10000 ) + 0x30;
	kwh[6] = ((PV2uWh2/100000) % 10000/1000 ) + 0x30;
	kwh[7] = ((PV2uWh2/100000) % 1000/100 ) + 0x30;
	kwh[8] = ((PV2uWh2/100000) % 100/10 ) + 0x30;
	Puts(166, 100, kwh, &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	Puts(265, 100, ".", &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	kwh1[0] = ((PV2uWh2/100000) % 10 );
	WriteDec(276, 100,  &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15], 1,0,kwh1[0],0);
	//Puts(276, 100, kwh1 , &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	Puts(287, 100,"Wh ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);


	Puts(0, 120,            "Battery        ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);

	kwh[0] = ((BatteryuWh2/100000) / 1000000000 ) + 0x30;
	kwh[1] = ((BatteryuWh2/100000) % 1000000000/100000000 ) + 0x30;
	kwh[2] = ((BatteryuWh2/100000) % 100000000/10000000 ) + 0x30;
	kwh[3] = ((BatteryuWh2/100000) % 10000000/1000000 ) + 0x30;
	kwh[4] = ((BatteryuWh2/100000) % 1000000/100000 ) + 0x30;
	kwh[5] = ((BatteryuWh2/100000) % 100000/10000 ) + 0x30;
	kwh[6] = ((BatteryuWh2/100000) % 10000/1000 ) + 0x30;
	kwh[7] = ((BatteryuWh2/100000) % 1000/100 ) + 0x30;
	kwh[8] = ((BatteryuWh2/100000) % 100/10 ) + 0x30;
	Puts(166, 120, kwh, &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	Puts(265, 120, ".", &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	kwh1[0] = ((BatteryuWh2/100000) % 10 );
	WriteDec(276, 120,  &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15], 1,0,kwh1[0],0);
	//Puts(276, 120, kwh1 , &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	Puts(287, 120,"Wh ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);


	Puts(0, 140,            "UP Time        ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);
	h2[0] = (((daycounter1*24)+hourcounter1) / 100000 ) + 0x30;
	h2[1] = (((daycounter1*24)+hourcounter1) % 100000/10000 ) + 0x30;
	h2[2] = (((daycounter1*24)+hourcounter1) % 10000/1000 ) + 0x30;
	h2[3] = (((daycounter1*24)+hourcounter1) % 1000/100 ) + 0x30;
	h2[4] = (((daycounter1*24)+hourcounter1) % 100/10 ) + 0x30;
	h2[5] = (((daycounter1*24)+hourcounter1) % 10 ) + 0x30;
	Puts(166, 140, "     ", &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	Puts(221, 140, h2, &TM_Font_11x18,bmscolor[menucolor][48], bmscolor[menucolor][15]);
	Puts(287, 140,"h  ", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);


	Puts(0, 160,           "Date:", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);
	WriteDec(55, 160,  &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15], 4,0,yearb+2000,0);
	Puts(99, 160,           "/", &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15]);
	WriteDec(110, 160,  &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15], 2,0,monthb,0);
	Puts(132, 160,           "/", &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15]);
	WriteDec(143, 160,  &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15], 2,0,dayb,0);
	Puts(166, 160,           " Time:", &TM_Font_11x18,bmscolor[menucolor][30], bmscolor[menucolor][15]);
}


//=================================================================================================
//                                      Time Display.
//=================================================================================================


void Time11 (void)

{

	    RTC_TimeTypeDef RTC_TimeStructure;
	    RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);

	    WriteDec(232, 160,  &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15], 2,0,RTC_TimeStructure.RTC_Hours,0);
	    Puts(254, 160,           ":", &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15]);
	    WriteDec(265, 160,  &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15], 2,0,RTC_TimeStructure.RTC_Minutes,0);
	    Puts(287, 160,           ":", &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15]);
	    WriteDec(298, 160,  &TM_Font_11x18,bmscolor[menucolor][45], bmscolor[menucolor][15], 2,0,RTC_TimeStructure.RTC_Seconds,0);

	    zz1=0;

}


void Time1 (uint8_t r)

{

	    RTC_TimeTypeDef RTC_TimeStructure;
	    RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);
	//    subsec = 100 - ((uint32_t)((uint32_t)RTC_GetSubSecond() * 100) / (uint32_t)8275);

	    LCDmenu_set_XY(21,r);
	    LCDmenu_write_char_neg((RTC_TimeStructure.RTC_Hours / 10) + 0x30,menucolor);
	    LCDmenu_set_XY(22,r);
	    LCDmenu_write_char_neg((RTC_TimeStructure.RTC_Hours % 10) + 0x30,menucolor);
	    LCDmenu_set_XY(23,r);
	    LCDmenu_write_string_neg(":",menucolor);
	    LCDmenu_set_XY(24,r);
	    LCDmenu_write_char_neg((RTC_TimeStructure.RTC_Minutes / 10) + 0x30,menucolor);
	    LCDmenu_set_XY(25,r);
	    LCDmenu_write_char_neg((RTC_TimeStructure.RTC_Minutes % 10) + 0x30,menucolor);
	    LCDmenu_set_XY(26,r);
	    LCDmenu_write_string_neg(":",menucolor);
	    LCDmenu_set_XY(27,r);
	    LCDmenu_write_char_neg((RTC_TimeStructure.RTC_Seconds / 10) + 0x30,menucolor);
	    LCDmenu_set_XY(28,r);
	    LCDmenu_write_char_neg((RTC_TimeStructure.RTC_Seconds % 10) + 0x30,menucolor);

	    zz1=0;

}

//=================================================================================================
//                                   Time Stamp.
//=================================================================================================

void timestamp1 (void)

{
	 RTC_TimeTypeDef RTC_TimeStructure;
	 RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);

	    timestamp[0]=(RTC_TimeStructure.RTC_Hours / 10) + 0x30;
	    timestamp[1]=(RTC_TimeStructure.RTC_Hours % 10) + 0x30;
	    timestamp[2]=44;
	    timestamp[3]=(RTC_TimeStructure.RTC_Minutes / 10) + 0x30;
	    timestamp[4]=(RTC_TimeStructure.RTC_Minutes % 10) + 0x30;
	    timestamp[5]=44;
	    timestamp[6]=(RTC_TimeStructure.RTC_Seconds / 10) + 0x30;
	    timestamp[7]=(RTC_TimeStructure.RTC_Seconds % 10) + 0x30;
}


//=================================================================================================
//                                       Date Stamp.
//=================================================================================================

void datestamp1 (void)

{
	 RTC_DateTypeDef RTC_DateStructure;
	 RTC_GetDate(RTC_Format_BIN, &RTC_DateStructure);

	    datestamp[0]=(RTC_DateStructure.RTC_Year / 10) + 0x30;
	    datestamp[1]=(RTC_DateStructure.RTC_Year % 10) + 0x30;
	    datestamp[2]=44;
	    datestamp[3]=(RTC_DateStructure.RTC_Month / 10) + 0x30;
	    datestamp[4]=(RTC_DateStructure.RTC_Month % 10) + 0x30;
	    datestamp[5]=44;
	    datestamp[6]=(RTC_DateStructure.RTC_Date / 10) + 0x30;
	    datestamp[7]=(RTC_DateStructure.RTC_Date % 10) + 0x30;
}



//=================================================================================================
//                                  Configures the USART.
//=================================================================================================

void USART_Config(void)
{
	  USART_InitTypeDef USART_InitStructure;
	  GPIO_InitTypeDef GPIO_InitStructure;
	  //NVIC_InitTypeDef NVIC_InitStructure;

	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3,ENABLE);

	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_7);
	  GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_7);

	  /* Configure USART3 pins:  Rx  ----------------------------*/
	  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_9;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);

	  /* Configure USART3 pins:  Tx  ----------------------------*/
	  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_8;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);


	  /* Enable USART3 IRQ */
	  //NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
	  //NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	  //NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	  //NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	  //NVIC_Init(&NVIC_InitStructure);


	  USART_InitStructure.USART_BaudRate = (Serial[1]*100);
	  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	  USART_InitStructure.USART_StopBits = USART_StopBits_1;
	  USART_InitStructure.USART_Parity = USART_Parity_No;
	  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	  USART_Init(USART3, &USART_InitStructure);

	  //USART_ITConfig(USART3, USART_IT_RXNE, ENABLE); // Receive interrupt ...

	  USART_Cmd(USART3,ENABLE);

}


#if USE_LPM > 0
// Enter in Low-Power mode when all sensors are in Release state

void LowPower(void)
{
  // Check that all sensors are in Release state
  if (MyObjGroup.StateMask == TSL_STATE_RELEASE_BIT_MASK)
  {

    // Set alarm to wakeup the MCU from STOP mode
    RTC_AlarmConfig();



    // Set GPIOs in "Low-Power mode"
    Init_LPM_GPIOs();

    //+++++++++++++
    // +++ STOP +++ --> Exit by RTC alarm
    //+++++++++++++
    // Request to enter STOP mode with regulator in low power mode
    PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFI);



    // Disable the RTC Alarm interrupt
    RTC_ITConfig(RTC_IT_ALRA, DISABLE);
    RTC_AlarmCmd(RTC_Alarm_A, DISABLE);

    // Configures system clock after wake-up from STOP
    SYSCLKConfig_STOP();

    // Re-initializes the GPIOs
    Init_Reset_GPIOs();
    Init_Std_GPIOs();
    //Init_TS_GPIOs();
    TSL_user_Init();
    //LCDmenu_GPIO_Config();


  }
}

//=================================================================================================
//                         SDADC read values all no autorange
//=================================================================================================
void read_all_SDADC_noautorange(uint16_t nrep)
{
	uint32_t tempadc6;
	uint8_t nrep1;

		//for (j1=0 ;j1<7;++j1){
		//	ADC16[j1][0]=0;  // reset ADC values
		//	}
    tempadc6=0;
    nrep1=nrep;
    nrep=nrep*5;

	init_SDADC1(SDADC_Channel_4,sdadc_gain[ADC16[0][1]]); //SDADC_Gain_1_2,SDADC_Gain_1,SDADC_Gain_2,SDADC_Gain_4,SDADC_Gain_8
	init_SDADC3(SDADC_Channel_6,sdadc_gain[ADC16[3][1]]);
	init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);

	read_all_SDADC(nrep);
    ADC16[0][0]=ADC16temp[0]/nrep1;
	ADC16[3][0]=ADC16temp[1]/nrep1;
	tempadc6=ADC16temp[2]/nrep1;


	init_SDADC1(SDADC_Channel_5,sdadc_gain[ADC16[1][1]]);
	init_SDADC3(SDADC_Channel_7,sdadc_gain[ADC16[4][1]]);
	init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);

	read_all_SDADC(nrep);
	ADC16[1][0]=ADC16temp[0]/nrep1;
	ADC16[4][0]=ADC16temp[1]/nrep1;
	tempadc6=tempadc6+(ADC16temp[2]/nrep1);

	init_SDADC1(SDADC_Channel_6,sdadc_gain[ADC16[2][1]]);
	init_SDADC3(SDADC_Channel_8,sdadc_gain[ADC16[5][1]]);
	init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);

	read_all_SDADC(nrep);
	ADC16[2][0]=ADC16temp[0]/nrep1;
	ADC16[5][0]=ADC16temp[1]/nrep1;
	ADC16[6][0]=(tempadc6+(ADC16temp[2]/nrep1))/3;


}


//=================================================================================================
//                                SDADC read values.
//=================================================================================================
void read_all_SDADC(uint16_t nrep)
{

ADC16temp[0]=0;
ADC16temp[1]=0;
ADC16temp[2]=0;
//discard first one
SDADC_SoftwareStartConv(SDADC1);
SDADC_SoftwareStartConv(SDADC2);
SDADC_SoftwareStartConv(SDADC3);
while(SDADC_GetFlagStatus(SDADC1,SDADC_FLAG_REOC)!=SET);
SDADC_GetConversionValue(SDADC1);
while(SDADC_GetFlagStatus(SDADC2,SDADC_FLAG_REOC)!=SET);
SDADC_GetConversionValue(SDADC2);
while(SDADC_GetFlagStatus(SDADC3,SDADC_FLAG_REOC)!=SET);
SDADC_GetConversionValue(SDADC3);


//uint32_t r=0;
do
	{
	SDADC_SoftwareStartConv(SDADC1);
	SDADC_SoftwareStartConv(SDADC2);
	SDADC_SoftwareStartConv(SDADC3);

	while(SDADC_GetFlagStatus(SDADC1,SDADC_FLAG_REOC)!=SET);
	ADC16temp[0] += (uint16_t) ((uint16_t)32768+SDADC_GetConversionValue(SDADC1));
	while(SDADC_GetFlagStatus(SDADC2,SDADC_FLAG_REOC)!=SET);
	ADC16temp[2] += (uint16_t) ((uint16_t)32768+SDADC_GetConversionValue(SDADC2));
	while(SDADC_GetFlagStatus(SDADC3,SDADC_FLAG_REOC)!=SET);
	ADC16temp[1] += (uint16_t) ((uint16_t)32768+SDADC_GetConversionValue(SDADC3));

	}
while(--nrep);

}
/*

void get_SDADC036()
{
	int16_t j1;

	for (j1=0 ;j1<7;++j1){
		ADC16[j1][4]=0;  // reset gain flag
		}



	        init_SDADC1(SDADC_Channel_4,sdadc_gain[ADC16[0][1]]); //SDADC_Gain_1_2,SDADC_Gain_1,SDADC_Gain_2,SDADC_Gain_4,SDADC_Gain_8
			init_SDADC3(SDADC_Channel_6,sdadc_gain[ADC16[3][1]]);
			init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);

            j1=0;
			while (j1==0) //autorange
						{
                          j1=1;
				            read_all_SDADC(3);

							ADC16[0][0]=ADC16temp[0]/3;
							ADC16[3][0]=ADC16temp[1]/3;
							ADC16[6][0]=ADC16temp[2]/3;


						   if (ADC16[0][0]>=64534 && ADC16[0][1]!=0)//65534
						    {
						    --ADC16[0][1];
						    ADC16[0][4]=1;
						    init_SDADC1(SDADC_Channel_4,sdadc_gain[ADC16[0][1]]);
						    j1=0;
						    }

						   if (ADC16[0][0]<=43000 && ADC16[0][1]!=4)//30000
						    {
						    ++ADC16[0][1];
						    ADC16[0][4]=1;
						    init_SDADC1(SDADC_Channel_4,sdadc_gain[ADC16[0][1]]);
						    j1=0;
						    }


						    if (ADC16[3][0]>=64534 && ADC16[3][1]!=0)
						   	     {
						   	     --ADC16[3][1];
						   	     ADC16[3][4]=1;
						   	     init_SDADC3(SDADC_Channel_6,sdadc_gain[ADC16[3][1]]);
						   	     j1=0;
						   	     }
						   	if (ADC16[3][0]<=43000 && ADC16[3][1]!=4)
						   	    {
						   	    ++ADC16[3][1];
						   	    ADC16[3][4]=1;
						   	    init_SDADC3(SDADC_Channel_6,sdadc_gain[ADC16[3][1]]);
						   	    j1=0;
						   	    }


						   	if (ADC16[6][0]>=64534 && ADC16[6][0]<=1000 && ADC16[6][1]!=0)
						   		 {
						   		 --ADC16[6][1];
						   		 ADC16[6][4]=1;
						   		 init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);
						   		 j1=0;
						   		 }
						   	if (ADC16[6][0]<=43768 && ADC16[6][0]>=21768 && ADC16[6][1]!=4)
						   		 {
						   		 ++ADC16[6][1];
						   		 ADC16[6][4]=1;
						   		 init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);
						   		 j1=0;
						   		 }

						 }



			read_all_SDADC(80);
			ADC16[0][0]=ADC16temp[0]/80;
			ADC16[3][0]=ADC16temp[1]/80;
			ADC16[6][0]=ADC16temp[2]/80;

			init_SDADC1(SDADC_Channel_3,sdadc_gain[ADC16[0][1]]); //Set to some unused internal channels
			init_SDADC3(SDADC_Channel_5,sdadc_gain[ADC16[3][1]]);
			init_SDADC2(SDADC_Channel_7,sdadc_gain[ADC16[6][1]]);

}

void get_SDADC146()
{
	int16_t j1;

	for (j1=0 ;j1<7;++j1){
		ADC16[j1][4]=0;  // reset gain flag
		}


    ADC16[1][1]=2;
	init_SDADC1(SDADC_Channel_5,sdadc_gain[ADC16[1][1]]);
	init_SDADC3(SDADC_Channel_7,sdadc_gain[ADC16[4][1]]);
	init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);

				 j1=0;
							while (j1==0) //autorange
										{
				                          j1=1;
								            read_all_SDADC(3);
											ADC16[1][0]=ADC16temp[0]/3;
											ADC16[4][0]=ADC16temp[1]/3;
											//ADC16temp[2]=80;

										   if (ADC16[1][0]>=64534 && ADC16[1][1]!=0)
										    {
										    //--ADC16[1][1];
											//ADC16[1][1]=4;
										    //ADC16[1][4]=1;
										    //init_SDADC1(SDADC_Channel_5,sdadc_gain[ADC16[1][1]]);
										    //j1=0;
										    }
										    if (ADC16[1][0]<=43000 && ADC16[1][1]!=4)
										    {
										    //++ADC16[1][1];
										    //ADC16[1][1]=4;
										    //ADC16[1][4]=1;
										    //init_SDADC1(SDADC_Channel_5,sdadc_gain[ADC16[1][1]]);
										    //j1=0;
										    }

										    if (ADC16[4][0]>=64534 && ADC16[4][1]!=0)
										   	{
										   	--ADC16[4][1];
										   	ADC16[4][4]=1;
										   	init_SDADC3(SDADC_Channel_7,sdadc_gain[ADC16[4][1]]);
										   	j1=0;
										   	}
										   	if (ADC16[4][0]<=43000 && ADC16[4][1]!=4)
										   	{
										   	++ADC16[4][1];
										   	ADC16[4][4]=1;
										   	init_SDADC3(SDADC_Channel_7,sdadc_gain[ADC16[4][1]]);
										   	j1=0;
										   	}


										 }


				read_all_SDADC(80);
				ADC16[1][0]=ADC16temp[0]/80;
				ADC16[4][0]=ADC16temp[1]/80;
				ADC16[6][0]=ADC16temp[2]/80;


			init_SDADC1(SDADC_Channel_3,sdadc_gain[ADC16[1][1]]); //Set to some unused internal channels
			init_SDADC3(SDADC_Channel_5,sdadc_gain[ADC16[4][1]]);
			init_SDADC2(SDADC_Channel_7,sdadc_gain[ADC16[6][1]]);

}

//=================================================================================================
void get_SDADC256()
{
	int16_t j1;

	for (j1=0 ;j1<7;++j1){
		ADC16[j1][4]=0;  // reset gain flag
		}



	init_SDADC1(SDADC_Channel_6,sdadc_gain[ADC16[2][1]]);
	init_SDADC3(SDADC_Channel_8,sdadc_gain[ADC16[5][1]]);
	init_SDADC2(SDADC_Channel_8,sdadc_gain[ADC16[6][1]]);


				 j1=0;
							while (j1==0) //autorange
										{
				                          j1=1;

											            read_all_SDADC(3);
														ADC16[2][0]=ADC16temp[0]/3;
														ADC16[5][0]=ADC16temp[1]/3;
														//ADC16temp[2]=160;

													   if (ADC16[2][0]>=64534 && ADC16[2][1]!=0)
													    {
													    --ADC16[2][1];
													    ADC16[2][4]=1;
													    init_SDADC1(SDADC_Channel_6,sdadc_gain[ADC16[2][1]]);
													    j1=0;
													    }
													    if (ADC16[2][0]<=43000 && ADC16[2][1]!=4)
													    {
													    ++ADC16[2][1];
													    ADC16[2][4]=1;
													    init_SDADC1(SDADC_Channel_6,sdadc_gain[ADC16[2][1]]);
													    j1=0;
													    }

													    if (ADC16[5][0]>=64534 && ADC16[5][1]!=0)
													   	{
													   	--ADC16[5][1];
													   	ADC16[5][4]=1;
													   	init_SDADC3(SDADC_Channel_8,sdadc_gain[ADC16[5][1]]);
													   	j1=0;
													   	}
													   	if (ADC16[5][0]<=43000 && ADC16[5][1]!=4)
													   	{
													   	++ADC16[5][1];
													   	ADC16[5][4]=1;
													   	init_SDADC3(SDADC_Channel_8,sdadc_gain[ADC16[5][1]]);
													   	j1=0;
													   	}


													 }

				read_all_SDADC(80);
				ADC16[2][0]=ADC16temp[0]/80;
				ADC16[5][0]=ADC16temp[1]/80;
				ADC16[6][0]=ADC16temp[2]/80;


				init_SDADC1(SDADC_Channel_3,sdadc_gain[ADC16[2][1]]); //Set to some unused internal channels
				init_SDADC3(SDADC_Channel_5,sdadc_gain[ADC16[5][1]]);
				init_SDADC2(SDADC_Channel_7,sdadc_gain[ADC16[6][1]]);

}

*/

//=================================================================================================
// ***************************          WiFi              ************************
//=================================================================================================

void WiFi_On(void)
{
	WiFis[0]=1;
	WIFI_ON;
	Serial[0]=0;
	Serial[1]=9216;
	USART_Config();
	delay_ms(20);
	//WiFi_UART();
	delay_ms(20);
	WiFi_Init();

}

void WiFi_On_new(void)
{
	WiFis[0]=1;
	WIFI_ON;
	Serial[0]=0;
	Serial[1]=1152;
	USART_Config();
	delay_ms(2000);
	WiFi_UART();
}

void WiFi_logo(void)
{
if (c2==2 || c2==22 || c2==4){return;}
uint8_t i,color1,color2;
color1=WiFis[0];
color2=WiFis[6];
for (i=0;i<4;i++){ DrawVline(198-(i*3),205,299+(i*5),bmscolor[8-color2][15+(28*color1)]); DrawVline(198-(i*3),205,300+(i*5),bmscolor[8-color2][15+(37*color1)]); DrawVline(198-(i*3),205,301+(i*5),bmscolor[8-color2][15+(47*color1)]);}
if (yearb!=0 && c2!=31 && c2!=32 && c2!=39 && c2!=30) {Puts(298, 212, "mem", &TM_Font_7x10, bmscolor[menucolor][0], bmscolor[menucolor][60]);}
else{Puts(298, 212, "mem", &TM_Font_7x10, bmscolor[menucolor][20],bmscolor[menucolor][5]);}
}


void WiFi_Off(void)
{
	WiFis[0]=0;
	WIFI_OFF;
	if (c2==35||c2==36||c2==37||c2==38){
	Puts(5, 145, "                                             ", &TM_Font_7x10, 0xFFFF, 0x0000);
	Puts(5, 155, "                                             ", &TM_Font_7x10, 0xFFFF, 0x0000);
	Puts(5, 165, " WiFi OFF                                    ", &TM_Font_7x10, 0xFFFF, 0x0000);
	}
}


void WiFi_set(void)

{

	char *cl1,*cl2,*cl3,*cl4;

    char command1[] = "ATE0\r\n";
    char command2[] = "AT+CWMODE=2\r\n";
    char command3[] = "AT+CIPMUX=1\r\n";
    char command4[] = "AT+CIPSERVER=1,80\r\n";

    cl1 = command1;
    cl2 = command2;
    cl3 = command3;
    cl4 = command4;

    while(*cl1){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl1++);}
    USART3_read(1);
    while(*cl2){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl2++);}
    USART3_read(1);
    while(*cl3){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl3++);}
    USART3_read(1);
    while(*cl4){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl4++);}
    USART3_read(1);


}


void WiFi_close(void)
{
	char *cl9;
	char command9[] = "AT+CIPCLOSE=5\r\n";

	cl9 = command9;

	while(*cl9){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl9++);}
	USART3_read(2);
}


void WiFi_DataHeader(void)
{
	char *cl8,*cl9;
			uint16_t cll1,jj,i;

//webpage="//ElectroDacus SBMS100 Month: 01";

for (i=0;i<32;i++){webpage[i]=web01[i];}

webpage[33]=13;
webpage[32]=32;

webpage[31]=(uSD[0] %10)+0x30;
webpage[30]=(uSD[0] %100/10)+0x30;

#if SBMS100>0
webpage[19]=49;//for SBMS100
webpage[21]=48;
#endif


		for (jj=0 ;jj<512;++jj){RXBUFF[jj]=0;}


		char command9[] = "AT+CIPBUFRESET=0\r\n";

		USART_ClearFlag(USART3, USART_FLAG_ORE);
		cl9 = command9;
		while(*cl9){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl9++);}
		USART3_read(1);

		char command8[] = "AT+CIPSENDBUF=0,34\r\n";

		USART_ClearFlag(USART3, USART_FLAG_ORE);
		cl8 = command8;
		while(*cl8){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl8++);}
		USART3_read(1);



	if(sizeRX!=28){wifib=0; if(sizeRX!=13){}//need to add WiFi_ON/OFF condition


    WiFis[6]=1;


    for (cll1=0;cll1<(34);cll1++)
    {

    	while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	if ((webpage[cll1]==92)&&(c2!=39))
    	{
    	USART_SendData(USART3, 92);
    	while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	}
    	USART_SendData(USART3, webpage[cll1]);

    }


	}else{wifib=1;WiFis[6]=0;
	//sizeRX2++;
	//WriteDec(180, 150, &TM_Font_7x10, 0xffff, 0x0000, 2,0,sizeRX2,0);

	};


}




void WiFi_send_flash(void)
{

	char *cl8,*cl9;
		uint16_t cll1,i,jj;


		for (jj=0 ;jj<512;++jj){RXBUFF[jj]=0;}



		char command9[] = "AT+CIPBUFRESET=0\r\n";

		USART_ClearFlag(USART3, USART_FLAG_ORE);
		cl9 = command9;
		while(*cl9){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl9++);}
		USART3_read(1);

		//sizeRX2=sizeRX;


		char command8[] = "AT+CIPSENDBUF=0,1815\r\n";

		USART_ClearFlag(USART3, USART_FLAG_ORE);
		cl8 = command8;
		while(*cl8){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl8++);}
		USART3_read(1);

  //if (sizeRX!=13) {return;}

	if(sizeRX!=28){wifib=0; if(sizeRX!=13){ sizeRX2=sizeRX;}//need to add WiFi_ON/OFF condition
    if ((sizeRX!=13)&&(sizeRX!=0)){datanr--;return;}//(sizeRX==22)||(sizeRX==37)||(sizeRX==19)||(sizeRX==32)
    if (sizeRX==0){datanr=datanr-2; for (i=0;i<1830;i++){webpage[i]=32;} }

    WiFis[6]=1;

    USART_ClearFlag(USART3, USART_FLAG_ORE);
    for (cll1=0;cll1<(1815);cll1++)
    {

    	while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	if ((webpage[cll1]==92)&&(c2!=39))
    	{
    	USART_SendData(USART3, 92);
    	while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	}
    	USART_SendData(USART3, webpage[cll1]);
    }


    //if (sizeRX!=19){WriteDec(140, 190, &TM_Font_7x10, 0xffff, 0x0000, 2,0,sizeRX,0);WriteDec(140, 200, &TM_Font_7x10, 0xffff, 0x0000, 2,0,sizeRX2,0);}
    //if (sizeRX==0){datanr=datanr-2;}

	}else{wifib=1;WiFis[6]=0;
	//sizeRX2++;
	//WriteDec(180, 150, &TM_Font_7x10, 0xffff, 0x0000, 2,0,sizeRX2,0);

	};
}




void WiFi_send(void)
{

	char *cl8,*cl9;
		uint16_t cll1,jj;


		for (jj=0 ;jj<512;++jj){RXBUFF[jj]=0;}


		char command9[] = "AT+CIPBUFRESET=0\r\n";

		USART_ClearFlag(USART3, USART_FLAG_ORE);
		cl9 = command9;
		while(*cl9){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl9++);}
		USART3_read(1);

		char command8[] = "AT+CIPSENDBUF=0,1860\r\n";

		USART_ClearFlag(USART3, USART_FLAG_ORE);
		cl8 = command8;
		while(*cl8){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl8++);}
		USART3_read(1);



	if(sizeRX!=28){wifib=0; if(sizeRX!=13){}//need to add WiFi_ON/OFF condition


    WiFis[6]=1;


    for (cll1=0;cll1<(1860);cll1++)
    {

    	while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	if ((webpage[cll1]==92)&&(c2!=39))
    	{
    	USART_SendData(USART3, 92);
    	while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);
    	}
    	USART_SendData(USART3, webpage[cll1]);

    }


	}else{wifib=1;WiFis[6]=0;
	//sizeRX2++;
	//WriteDec(180, 150, &TM_Font_7x10, 0xffff, 0x0000, 2,0,sizeRX2,0);

	};



}

void WiFi_CWSAP(void)

{
	uint16_t jj;
	char *cl9;
	char command9[] = "AT+CWSAP?\r\n";

	cl9 = command9;
	for (jj=0 ;jj<512;++jj){RXBUFF[jj]=0;}
	while(*cl9){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl9++);}
	USART3_read(2);

	for (jj=0 ;jj<512;++jj){if (RXBUFF[jj]==13){RXBUFF[jj]=32;} if (RXBUFF[jj]==10){RXBUFF[jj]=32;}}

	Puts(5, 145, RXBUFF, &TM_Font_7x10, 0xFFFF, 0x0000);
	if (sizeRX!=0){Puts(5, 165, " WiFi ON                                     ", &TM_Font_7x10, 0xFFFF, 0x0000);}
	else
	{
		Puts(5, 155, "                                             ", &TM_Font_7x10, 0xFFFF, 0x0000);
		Puts(5, 145, "                                             ", &TM_Font_7x10, 0xFFFF, 0x0000);
		Puts(5, 165, " WiFi OFF                                    ", &TM_Font_7x10, 0xFFFF, 0x0000);
	}


}

void WiFi_UART(void)
{
	char *cl9;
	char command9[] = "AT+UART_DEF=921600,8,1,0,0\r\n";

	cl9 = command9;

	while(*cl9){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl9++);}
	USART3_read(2);
}



void WiFi_Init(void)
{
	    uint16_t jj;
	    char command1[] = "AT+RST\r\n";
		char command2[] = "ATE0\r\n";
		char command3[] = "AT\r\n";
		//char command4[] = "AT+RFPOWER=60\r\n";
		char command4[] = "AT+CWSAP_DEF=\"SBMS-100\",\"             \",02,0\r\n";

		command4[19] = (WiFis[2] / 100 ) + 0x30;
		command4[20] = (WiFis[2] % 100/10 ) + 0x30;
		command4[21] = (WiFis[2] % 10 ) + 0x30;

		command4[25] = (WiFis[4] / 10000 ) + 0x30;
		command4[26] = (WiFis[4] % 10000/1000 ) + 0x30;
		command4[27] = (WiFis[4] % 1000/100 ) + 0x30;
		command4[28] = (WiFis[4] % 100/10 ) + 0x30;
		command4[29] = (WiFis[4] % 10 ) + 0x30;

		command4[30] = (WiFis[5] / 10000 ) + 0x30;
		command4[31] = (WiFis[5] % 10000/1000 ) + 0x30;
		command4[32] = (WiFis[5] % 1000/100 ) + 0x30;
		command4[33] = (WiFis[5] % 100/10 ) + 0x30;
		command4[34] = (WiFis[5] % 10 ) + 0x30;

		command4[35] = 66;//B
		command4[36] = 77;//M
		command4[37] = 83;//S

		command4[40]=(WiFis[1] / 10 ) + 0x30;
		command4[41]=(WiFis[1] % 10 ) + 0x30;

		command4[43]=(WiFis[3]*3) + 0x30;

		char *cl1,*cl2,*cl3,*cl4;
		cl1 = command1;
		cl2 = command2;
		cl3 = command3;
		cl4 = command4;



while(*cl1){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl1++);}
USART3_read(1000);
while(*cl2){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl2++);}
USART3_read(1);
for (jj=0 ;jj<512;++jj){RXBUFF[jj]=0;}
while(*cl3){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl3++);}
USART3_read(1);
for (jj=0 ;jj<512;++jj){RXBUFF[jj]=0;}
while(*cl4){while(USART_GetFlagStatus(USART3, USART_FLAG_TXE) == RESET);USART_SendData(USART3, *cl4++);}
USART3_read(1);

}


void USART3_read(uint16_t delayx)
{
	uint32_t count33;
	sizeRX=0;
	 for (count33=0 ;count33<(2000*delayx);count33++)
	                      {
	                      if((USART_GetFlagStatus(USART3, USART_FLAG_RXNE) == SET)&&(sizeRX<480)){RXBUFF[sizeRX] = USART_ReceiveData(USART3);sizeRX++;count33=1;}
	                      else {count33=count33+1;}
	                      }
	 USART_ClearFlag(USART3, USART_FLAG_ORE);
}

uint16_t pow(int base, int exp)
    {   int result = 1;
        while (exp)
        {
            if (exp & 1)
            result *= base;
            exp >>= 1;
            base *= base;
        }
        return result;
    }


//=================================================================================================
//                                    Configures the RTC.
//=================================================================================================

void RTC_Config(void)
{
  RTC_TimeTypeDef   RTC_TimeStructure;
  RTC_InitTypeDef   RTC_InitStructure;
  EXTI_InitTypeDef EXTI_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  /* Configures the KEY button */
  // Uncomment to enable KEY button on the board
  // Used to exit from STOP mode manually without waiting the RTC alarm.
  //STM_EVAL_PBInit(BUTTON_KEY, BUTTON_MODE_EXTI);

  /* Enable the PWR clock */
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

  /* Allow access to RTC */
  PWR_BackupAccessCmd(ENABLE);

  /* Enable the LSE or LSI */
  RCC_LSEConfig(RCC_LSE_ON);
  // RCC_LSICmd(ENABLE);

  /* Wait till LSE or LSI is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
 // while (RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET)
  {}

  /* Select the RTC Clock Source = LSE or LSI */
  RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);
  //RCC_RTCCLKConfig(RCC_RTCCLKSource_LSI);

  /* Enable the RTC Clock */
  RCC_RTCCLKCmd(ENABLE);

  /* Wait for RTC APB registers synchronisation */
  RTC_WaitForSynchro();

  /* Set RTC calendar clock to 1 Hz */
  RTC_InitStructure.RTC_HourFormat = RTC_HourFormat_24;
  // RTC_AsynchPrediv max =   127
  // RTC_SynchPrediv  max = 32767
  // For LSI (40000 Hz)
  RTC_InitStructure.RTC_AsynchPrediv = 127; // 40000 Hz / 2 = 20000 Hz
  RTC_InitStructure.RTC_SynchPrediv  = 255; //   20000 Hz / 20000 =   1 Hz

  if (RTC_Init(&RTC_InitStructure) == ERROR)
  {
    while(1);
  }

  /* Set the time to 00h 00mn 00s AM */
  RTC_TimeStructure.RTC_H12     = RTC_H12_AM;
  RTC_TimeStructure.RTC_Hours   = 0x00;
  RTC_TimeStructure.RTC_Minutes = 0x00;
  RTC_TimeStructure.RTC_Seconds = 0x00;

  RTC_SetTime(RTC_Format_BCD, &RTC_TimeStructure);

  /* Configure EXTI line 17 (connected to the RTC Alarm event) */
  EXTI_ClearITPendingBit(EXTI_Line17);
  EXTI_InitStructure.EXTI_Line = EXTI_Line17;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  /* Enable the RTC Interrupt */
  //NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
  //NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}


/**
  * @brief  Configures the RTC alarm.
  * @note   You can change the alarm configuration: 1s or less than 1s by commenting/uncommenting
  *         the code below.
  * @param  None
  * @retval None
  */
//=================================================================================================
//                                  Configures the RTC alarm.
//=================================================================================================

void RTC_AlarmConfig(void)
{
  RTC_TimeTypeDef   RTC_TimeStructure;
  RTC_AlarmTypeDef  RTC_AlarmStructure;

  /* Get current time */
  RTC_GetTime(RTC_Format_BIN, &RTC_TimeStructure);

  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Set the alarm to current time + 1s
/*
  RTC_AlarmStructure.RTC_AlarmTime.RTC_H12     = RTC_H12_AM;
  RTC_AlarmStructure.RTC_AlarmTime.RTC_Hours   = RTC_TimeStructure.RTC_Hours;
  RTC_AlarmStructure.RTC_AlarmTime.RTC_Minutes = RTC_TimeStructure.RTC_Minutes;
  RTC_AlarmStructure.RTC_AlarmTime.RTC_Seconds = RTC_TimeStructure.RTC_Seconds + 2;
  RTC_AlarmStructure.RTC_AlarmDateWeekDay = 0x31;
  RTC_AlarmStructure.RTC_AlarmDateWeekDaySel = RTC_AlarmDateWeekDaySel_Date;
  RTC_AlarmStructure.RTC_AlarmMask = RTC_AlarmMask_DateWeekDay | RTC_AlarmMask_Minutes |
                                     RTC_AlarmMask_Hours;


  RTC_SetAlarm(RTC_Format_BIN, RTC_Alarm_A, &RTC_AlarmStructure);
*/
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  // Set the alarm to a value below 1 second

  RTC_AlarmStructure.RTC_AlarmMask = RTC_AlarmMask_All;
  RTC_SetAlarm(RTC_Format_BIN, RTC_Alarm_A, &RTC_AlarmStructure);

  // Set AlarmA subseconds and enable SubSec Alarm:
  //RTC_AlarmSubSecondConfig(RTC_Alarm_A, 0xFF, RTC_AlarmSubSecondMask_SS14_5); // One interrupt every 125 ms
  RTC_AlarmSubSecondConfig(RTC_Alarm_A, 0xFF, RTC_AlarmSubSecondMask_SS14_6); // One interrupt every 250 ms

  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  /* Enable the RTC Alarm A interrupt */
  RTC_ITConfig(RTC_IT_ALRA, ENABLE);

  /* Enable the alarm */
  RTC_AlarmCmd(RTC_Alarm_A, ENABLE);

  /* Clear the Alarm A Pending Bit */
  RTC_ClearITPendingBit(RTC_IT_ALRA);

}

//=================================================================================================
//      Configures system clock after wake-up from STOP: enable HSE, PLL  (not implemented)
//=================================================================================================

void SYSCLKConfig_STOP(void)
{
  /* After wake-up from STOP reconfigure the system clock */
  /* Enable HSE */
	RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET)
  {}

  /* Enable PLL */
  RCC_PLLCmd(ENABLE);

  /* Wait till PLL is ready */
  while (RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
  {}

  /* Select PLL as system clock source */
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

  /* Wait till PLL is used as system clock source */
  while (RCC_GetSYSCLKSource() != 0x08)
  {}
}

//=================================================================================================
//                               Reset GPIOs MODER registers.
//=================================================================================================

void Init_Reset_GPIOs(void)
{
  // Enable GPIO clocks
  RCC->AHBENR |= (RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN | RCC_AHBENR_GPIODEN | RCC_AHBENR_GPIOFEN);

  GPIOA->MODER = 0;
  GPIOB->MODER = 0;
  GPIOC->MODER = 0;
  GPIOD->MODER = 0;
  GPIOF->MODER = 0;
}

//=================================================================================================
//            Configure GPIOs in "Low Power mode" : Analog Mode, No Pull-Up nor Pull-Down
//=================================================================================================

void Init_LPM_GPIOs(void)
{
  // Enable GPIO clocks
  RCC->AHBENR |= (RCC_AHBENR_GPIOAEN | RCC_AHBENR_GPIOBEN | RCC_AHBENR_GPIOCEN | RCC_AHBENR_GPIODEN | RCC_AHBENR_GPIOFEN);

  GPIOA->MODER |= 0xFFFCFFFF; // If PA8 is used (DEBUG_IO)
  //GPIOA->MODER |= 0xFFFFFFFF; // If PA8 is not used (DEBUG_IO)
  GPIOA->PUPDR = 0;

  GPIOB->MODER = 0xFFFFFFFF;
  GPIOB->PUPDR = 0;

  GPIOC->MODER = 0xFFFFFFFF;
  GPIOC->PUPDR = 0;

  GPIOD->MODER = 0xFFFFFFFF;
  GPIOD->PUPDR = 0;

  GPIOF->MODER = 0xFFFFFFFF;
  GPIOF->PUPDR = 0;
}
#endif // USE_LPM

//=================================================================================================
//                        In case of I2C error print and LCD message.
//=================================================================================================

uint32_t sEE_TIMEOUT_UserCallback(void)
{
  /* Use application may try to recover the communication by resetting I2C
    peripheral (calling the function I2C_SoftwareResetCmd()) then re-start
    the transmission/reception from a previously stored recover point.
    For simplicity reasons, this example only shows a basic way for errors
    managements which consists of stopping all the process and requiring system
    reset. */

	//LCDmenu_clear();
	i2c++;
	LCDmenu_set_XY(0,0);
	LCDmenu_write_string_neg("ISL94203 communication error ",menucolor);
	if (USBpower!=0){USBpower=2;}
	delay_ms(40);

  /* Block communication and all processes */
return 0;
}


void MyTKeys_ErrorStateProcess(void)
{
  // Add here your own processing when a sensor is in Error state
  //TSL_tkey_SetStateOff();
}


void MyTKeys_OffStateProcess(void)
{
  // Add here your own processing when a sensor is in Off state
  //process_sensor = 0;
}

void MyLinRots_ErrorStateProcess(void)
{
  // Add here your own processing when a sensor is in Error state
 // TSL_linrot_SetStateOff();

}

void MyLinRots_OffStateProcess(void)
{
  // Add here your own processing when a sensor is in Off state
//  process_sensor = 0;
}


//-------------------
// CallBack functions
//-------------------

/**
  * @brief  Executed at each timer interruption (option must be enabled)
  * @param  None
  * @retval None
  */
void TSL_CallBack_TimerTick(void)
{
  //LED1_ON;
  //delay_ms(200);
  //LED2_OFF;
}

#ifdef USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  for (;;)
  {
  }
}
#endif


void clock_init(){
   /* Start with HSI clock (internal 8mhz), divide by 2 and multiply by 9 to
   * get maximum allowed frequency: 36Mhz
   * Enable PLL, wait till it's stable, then select it as system clock*/
  RCC_PLLConfig(RCC_PLLSource_HSI_Div2, RCC_PLLMul_10);
  RCC_PLLCmd(ENABLE);
  while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET) {}
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

  /* Set HCLK, PCLK1, and PCLK2 to SCLK (these are default */
  RCC_HCLKConfig(RCC_SYSCLK_Div1);
  RCC_PCLK1Config(RCC_HCLK_Div1);
  RCC_PCLK2Config(RCC_HCLK_Div1);

}

void SysTick_Handler(void)
{
    // if the variable Time (in ms, useful to save the time elapsed between two presses)
    // is <4seconds, increment Time
    if (Time < 99000) Time++;
}
